Package: MALDIquant
Version: 1.22.3
Date: 2024-08-19
Title: Quantitative Analysis of Mass Spectrometry Data
Authors@R: c(
    person(
        given = "Sebastian", family = "Gibb",
        email="mail@sebastiangibb.de",
        comment=c(ORCID="0000-0001-7406-4443"),
        role = c("aut", "cre")
    ),
    person(
        given = "Korbinian", family = "Strimmer",
        comment=c(ORCID="0000-0001-7917-2056"),
        role="ths"
    ),
    person(given = "Sigurdur", family = "Smarason", role = "ctb"),
    person(
        given = "Laurent", family = "Gatto",
        email = "laurent.gatto@uclouvain.be",
        comment = c(ORCID = "0000-0002-1520-2268"),
        role = "ctb"
    ),
    person(given = "Paolo", family = "Inglese", role = "ctb"))
Depends: R (>= 4.0.0), methods
Imports: parallel
Suggests: knitr, testthat (>= 0.8)
Description: A complete analysis pipeline for matrix-assisted laser
        desorption/ionization-time-of-flight (MALDI-TOF) and other
        two-dimensional mass spectrometry data. In addition to commonly
        used plotting and processing methods it includes distinctive
        features, namely baseline subtraction methods such as
        morphological filters (TopHat) or the statistics-sensitive
        non-linear iterative peak-clipping algorithm (SNIP), peak
        alignment using warping functions, handling of replicated
        measurements as well as allowing spectra with different
        resolutions.
License: GPL (>= 3)
URL: https://strimmerlab.github.io/software/maldiquant/,
        https://github.com/sgibb/MALDIquant/
BugReports: https://github.com/sgibb/MALDIquant/issues/
LazyLoad: yes
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-08-19 20:58:22 UTC; sebastian
Author: Sebastian Gibb [aut, cre] (<https://orcid.org/0000-0001-7406-4443>),
  Korbinian Strimmer [ths] (<https://orcid.org/0000-0001-7917-2056>),
  Sigurdur Smarason [ctb],
  Laurent Gatto [ctb] (<https://orcid.org/0000-0002-1520-2268>),
  Paolo Inglese [ctb]
Maintainer: Sebastian Gibb <mail@sebastiangibb.de>
Repository: CRAN
Date/Publication: 2024-08-19 23:00:03 UTC

import("methods")

importFrom("parallel",
              "mclapply",
              "mcmapply")

importFrom("graphics",
              "abline",
              "arrows",
              "lines",
              "par",
              "plot.default",
              "points",
              "rasterImage",
              "rect",
              "strheight",
              "strwidth",
              "text")

importFrom("grDevices",
              "as.raster",
              "colorRamp",
              "dev.cur",
              "dev.interactive",
              "dev.new",
              "dev.off",
              "pdf",
              "rgb")

importFrom("stats",
              "approxfun",
              "coef",
              "dpois",
              "filter",
              "lm",
              "lowess",
              "mad",
              "median",
              "runmed",
              "supsmu")

importFrom("utils",
              "head",
              "modifyList",
              "object.size",
              "packageVersion",
              "relist",
              "tail")


exportClasses("MassPeaks",
              "MassSpectrum")

export("alignSpectra",
       "averageMassSpectra",
       "binPeaks",
       "createMassPeaks",
       "createMassSpectrum",
       "determineWarpingFunctions",
       "filterPeaks",
       "findEmptyMassObjects",
       "intensityMatrix",
       "isMassPeaks",
       "isMassPeaksList",
       "isMassSpectrum",
       "isMassSpectrumList",
       "match.closest",
       "mergeMassPeaks",
       "msiSlices",
       "referencePeaks",
       "removeEmptyMassObjects",
       "warpMassPeaks",
       "warpMassSpectra")

exportMethods("as.matrix",
              "calibrateIntensity",
              "coordinates",
              "coordinates<-",
              "detectPeaks",
              "estimateBaseline",
              "estimateNoise",
              "intensity",
              "intensity<-",
              "isEmpty",
              "isRegular",
              "labelPeaks",
              "length",
              "lines",
              "mass",
              "mass<-",
              "metaData",
              "metaData<-",
              "monoisotopicPeaks",
              "mz",
              "mz<-",
              "plot",
              "plotMsiSlice",
              "points",
              "removeBaseline",
              "smoothIntensity",
              "snr",
              "totalIonCurrent",
              "transformIntensity",
              "trim")

useDynLib(MALDIquant, .registration = TRUE)

## This is a MALDIquant example file. It is released into public domain with the
## right to use it for any purpose but without any warranty.


## peaks demo


## load necessary packages
## requires MALDIquant >= 1.9
library("MALDIquant")

## load example spectra
data("fiedler2009subset", package="MALDIquant")

## choose only spectrum 1
s1 <- fiedler2009subset[[1]]


## preprocessing
## sqrt transform (for variance stabilization)
s2 <- transformIntensity(s1, method="sqrt")


## 21 point Savitzky-Golay-Filter for smoothing spectra
## (maybe you have to adjust the halfWindowSize;
## you could use a simple moving average instead)
## see ?smoothIntensity
s3 <- smoothIntensity(s2, method="SavitzkyGolay", halfWindowSize=10)


## remove baseline
## (maybe you have to adjust iterations to your spectra; high resolution
## spectra need a much lower iteration number (halfWindowSize, for some other
## baseline estimation algorithms)
## see ?removeBaseline, ?estimateBaseline
s4 <- removeBaseline(s3, method="SNIP", iterations=100)


## run peak detection
## (maybe you need to adjust halfWindowSize [decreasing it for high resolution
## spectra] and SNR [a higher value increase the True-Positive-Rate but decrease
## sensitivity])
## see ?detectPeaks, ?estimateNoise
p <- detectPeaks(s4, method="MAD", halfWindowSize=20, SNR=2)


## produce some plots
par(mfrow=c(2,3))

xlim <- range(mass(s1)) # use same xlim on all plots for better comparison

plot(s1, main="1: raw", sub="", xlim=xlim)
plot(s2, main="2: variance stabilization", sub="", xlim=xlim)
plot(s3, main="3: smoothing", sub="", xlim=xlim)
plot(s4, main="4: baseline correction", sub="", xlim=xlim)

plot(s4, main="5: peak detection", sub="", xlim=xlim)
points(p)


## label top 20 peaks
top20 <- intensity(p) %in% sort(intensity(p), decreasing=TRUE)[1:20]
labelPeaks(p, index=top20, underline=TRUE)

plot(p, main="6: peak plot", sub="", xlim=xlim)
labelPeaks(p, index=top20, underline=TRUE)

par(mfrow=c(1,1))

## This is a MALDIquant example file. It is released into public domain with the
## right to use it for any purpose but without any warranty.


## warping demo


## load necessary packages
## requires MALDIquant >= 1.9
library("MALDIquant")

## load example spectra
data("fiedler2009subset", package="MALDIquant")

## use only 4 spectra
spectra <- fiedler2009subset[seq(1, 16, by=4)]

## some preprocessing

## sqrt transform (for variance stabilization)
spectra <- transformIntensity(spectra, method="sqrt")

## 21 point Savitzky-Golay-Filter for smoothing spectra
spectra <- smoothIntensity(spectra, method="SavitzkyGolay", halfWindowSize=10)

## remove baseline
spectra <- removeBaseline(spectra, method="SNIP", iterations=100)

## calibrate intensity values by "Total Ion Current"
spectra <- calibrateIntensity(spectra, method="TIC")

## run peak detection
peaks <- detectPeaks(spectra, method="MAD", halfWindowSize=20, SNR=2)

## warping
par(mfrow=c(2, 2))
warpingFunctions <- determineWarpingFunctions(peaks, tolerance=0.001,
                                              plot=TRUE, plotInteractive=TRUE)

## warp spectra
warpedSpectra <- warpMassSpectra(spectra, warpingFunctions)
## warp peaks
warpedPeaks <- warpMassPeaks(peaks, warpingFunctions)

## compare some regions in a plot
par(mfrow=c(2, 2))

## helper function to avoid double coding
plotSpectra <- function(unwarped, warped, range) {
  plot(unwarped[[1]], main=paste0("unwarped spectra (mass ",
                                  paste0(range, collapse=":"), " Da)"),
       xlim=range, ylim=c(0, 2e-3), type="n")

  color <- rainbow(length(unwarped))

  for (i in seq(along=unwarped)) {
    lines(unwarped[[i]], col=color[i])
  }

  plot(unwarped[[1]], main=paste0("warped spectra (mass ",
                                 paste0(range, collapse=":"), " Da)"),
       xlim=range, ylim=c(0, 2e-3), type="n")

  for (i in seq(along=warped)) {
    lines(warped[[i]], col=color[i])
  }
}

plotSpectra(spectra, warpedSpectra, c(4180, 4240))
plotSpectra(spectra, warpedSpectra, c(9200, 9400))

par(mfrow=c(1, 1))

## This is a MALDIquant example file. It is released into public domain with the
## right to use it for any purpose but without any warranty.


## all MALDIquant demos


library("MALDIquant")
demo("baseline", package="MALDIquant")
demo("peaks", package="MALDIquant")
demo("warping", package="MALDIquant")
message("demo(\"workflow\") produces no graphical output")
demo("workflow", package="MALDIquant")

## This is a MALDIquant example file. It is released into public domain with the
## right to use it for any purpose but without any warranty.


## baseline demo


## load necessary packages
## requires MALDIquant >= 1.9
library("MALDIquant")

## load example spectra
data("fiedler2009subset", package="MALDIquant")

## choose only spectrum 1
s <- fiedler2009subset[[1]]

## test different baseline estimation methods
bSnip <- estimateBaseline(s, method="SNIP")
plot(s, main="SNIP Baseline [default]")
lines(bSnip, lwd=2, col=2)

bTopHat <- estimateBaseline(s, method="TopHat")
plot(s, main="TopHat Baseline")
lines(bTopHat, lwd=2, col=2)

bConvexHull <- estimateBaseline(s, method="ConvexHull")
plot(s, main="ConvexHull Baseline")
lines(bConvexHull, lwd=2, col=2)

bMedian <- estimateBaseline(s, method="median")
plot(s, main="Median Baseline")
lines(bMedian, lwd=2, col=2)

## This is a MALDIquant example file. It is released into public domain with the
## right to use it for any purpose but without any warranty.


## workflow demo


## load necessary packages
## requires MALDIquant >= 1.9
library("MALDIquant")


## load example spectra
data("fiedler2009subset", package="MALDIquant")


## check raw data
## any empty spectra? (empty spectra are ignored in subsequent baseline
## correction/peak detection; you could find/remove them by calling
## findEmptyMassObjects/removeEmptyMassObjects)
## see ?isEmpty, ?findEmptyMassObjects, ?removeEmptyMassObjects
any(sapply(fiedler2009subset, isEmpty))
# FALSE

## any spectra with irregular mass values/intervals? (spectra with
## missing/filtered mass values/irregular mass intervals may compromise
## subsequent baseline correction and peak detection.)
any(!sapply(fiedler2009subset, isRegular))
# FALSE

## do length of spectra differ? (if they differ you have to adjust the
## corresponding halfWindowSize in subsequent baseline correction and peak
## detection.)
table(sapply(fiedler2009subset, length))
# 42388
#    16


## preprocessing
## sqrt transform (for variance stabilization)
spectra <- transformIntensity(fiedler2009subset, method="sqrt")


## 21 point Savitzky-Golay-Filter for smoothing spectra
## (maybe you have to adjust the halfWindowSize;
## you could use a simple moving average instead)
## see ?smoothIntensity
spectra <- smoothIntensity(spectra, method="SavitzkyGolay", halfWindowSize=10)


## remove baseline
## (maybe you have to adjust iterations to your spectra; high resolution
## spectra need a much lower iteration number (halfWindowSize, for some other
## baseline estimation algorithms)
## see ?removeBaseline, ?estimateBaseline
spectra <- removeBaseline(spectra, method="SNIP", iterations=100)


## calibrate (normalize) intensities (different calibration methods available)
## see ?calibrateIntensity
spectra <- calibrateIntensity(spectra, method="TIC")


## spectra alignment
## (the spectra alignment is peak based, maybe you need to adjust
## halfWindowSize, SNR, tolerance, warpingMethod)
## see ?alignSpectra
spectra <- alignSpectra(spectra,
                        halfWindowSize=20, SNR=2,
                        tolerance=0.002, warpingMethod="lowess")


## average technical replicates
## 1. create factors for correct assignment
## (e.g. sample name is stored in metaData(x)$sampleName; maybe you have to
## adjust the metaData or use your own ID table for your data here)
samples <- factor(sapply(spectra, function(x)metaData(x)$sampleName))

## 2. average technical replicates
## see ?averageMassSpectra
avgSpectra <- averageMassSpectra(spectra, labels=samples, method="mean")


## run peak detection
## (maybe you need to adjust halfWindowSize [decreasing it for high resolution
## spectra] and SNR [a higher value increase the True-Positive-Rate but decrease
## sensitivity])
## see ?detectPeaks, ?estimateNoise
peaks <- detectPeaks(avgSpectra, method="MAD", halfWindowSize=20, SNR=2)

## bin peaks
## (After alignment peak positions (mass) are similar but not identical. Binning
## is needed to make similar peak mass values identical.)
## see ?binPeaks
peaks <- binPeaks(peaks, tolerance=0.002)


## prepare for statistical analysis
## 1. get cancer/control indices
filenames <- sapply(peaks, function(x)metaData(x)$file[1])
cancer <- grepl(pattern="/tumor/", x=filenames)
classes <- factor(ifelse(cancer, "cancer", "control"),
                  levels=c("cancer", "control"))

## 2. export expression/training matrix
## (and fill missing peaks by interpolated values)
## see ?intensityMatrix
training <- intensityMatrix(peaks, avgSpectra)


## 'training' and 'classes' could now used by any statistical tool e.g. sda

## MassSpectrum
setMethod(f="estimateNoise",
          signature=signature(object="MassSpectrum"),
          definition=function(object, method=c("MAD", "SuperSmoother"),
                              ...) {
  if (.isEmptyWarning(object)) {
    return(0L)
  }

  cbind(mass=object@mass,
        intensity=.estimateNoise(x=object@mass, y=object@intensity,
                                 method=method, ...))
})

## AbstractMassObject
setMethod(f="[",
          signature=signature(x="AbstractMassObject", i="numeric", j="missing"),
          definition=function(x, i, j, ..., drop=TRUE) {

  x@mass <- x@mass[i]
  x@intensity <- x@intensity[i]

  x
})

## AbstractMassObject
setMethod(f="[",
          signature=signature(x="AbstractMassObject", i="logical", j="missing"),
          definition=function(x, i, j, ..., drop=TRUE) {

  ## seems to be faster than evaluating the logical expression twice
  i <- which(i)
  x@mass <- x@mass[i]
  x@intensity <- x@intensity[i]

  x
})

setMethod(f="[",
          signature=signature(x="MassPeaks", i="numeric", j="missing"),
          definition=function(x, i, j, ..., drop=TRUE) {

  x@mass <- x@mass[i]
  x@intensity <- x@intensity[i]
  x@snr <- x@snr[i]

  x
})

## AbstractMassObject
setMethod(f="[",
          signature=signature(x="MassPeaks", i="logical", j="missing"),
          definition=function(x, i, j, ..., drop=TRUE) {

  ## seems to be faster than evaluating the logical expression twice
  i <- which(i)
  x@mass <- x@mass[i]
  x@intensity <- x@intensity[i]
  x@snr <- x@snr[i]

  x
})

#' .pseudoCluster
#'
#' Find possible isotopic cluster in mass/mz data.
#'
#' @param x double, mass
#' @param size integer, cluster size, number of peaks per cluster
#' @param distance double, distance between isotopes (mass of a neutron; see
#' Park et al 2008); could be of length > 1 (if > 1: order will affect later
#' removal in .monoisotopicPattern).
#' @param tolerance double, mass tolerance
#' @return a matrix of indices (nrow(x) == n) of potential clusters
#' @references
#' K. Park, J.Y. Yoon, S. Lee, E. Paek, H. Park, H.J. Jung, and S.W. Lee. 2008.
#' Isotopic peak intensity ratio based algorithm for determination of isotopic
#' clusters and monoisotopic masses of polypeptides from high-resolution
#' mass spectrometric data.
#' Analytical Chemistry, 80: 7294-7303.
#' @noRd
.pseudoCluster <- function(x, size=3L, distance=1.00235, tolerance=1e-4) {
  if (size < 2L) {
    stop("The ", sQuote("size"), " of a cluster has to be at least 2!")
  }
  mm <- matrix(x, nrow=size, ncol=length(x) * length(distance), byrow=TRUE)
  ms <- mm + (rep(distance, each=size) * 0L:(size - 1L))

  i <- match.closest(ms, x, tolerance=mm * tolerance)
  dim(i) <- dim(ms)

  i[, !is.na(colSums(i)), drop=FALSE]
}

#' .F
#'
#' Map mass to poisson mean/lambda.
#'
#' @param mass double, mass from experimental peak list
#' @return double suitable to pass to `dpois`
#' @references
#' E.J. Breen, F.G. Hopwood, K.L. Williams, and M.R. Wilkins. 2000.
#' Automatic poisson peak harvesting for high throughput protein identification.
#' Electrophoresis 21 (2000): 2243-2251.
#' @noRd
.F <- function(x)0.000594 * x + 0.03091

#' .P
#'
#' Model isotopic distribution by poisson distribution.
#'
#' @param mass double, mass from experimental peak list
#' @param isotopes integer, which isotopes
#' @return double, isotopic distribution
#' @references
#' E.J. Breen, F.G. Hopwood, K.L. Williams, and M.R. Wilkins. 2000.
#' Automatic poisson peak harvesting for high throughput protein identification.
#' Electrophoresis 21 (2000): 2243-2251.
#' @noRd
.P <- function(x, isotopes)dpois(isotopes, .F(x))

#' .Psum
#'
#' Model isotopic distribution by poisson distribution and sum to 1 (similar to
#' TIC).
#'
#' @param mass double, mass from experimental peak list
#' @param isotopes integer, which isotopes
#' @return double, isotopic distribution
#' @references
#' E.J. Breen, F.G. Hopwood, K.L. Williams, and M.R. Wilkins. 2000.
#' Automatic poisson peak harvesting for high throughput protein identification.
#' Electrophoresis 21 (2000): 2243-2251.
#' @noRd
.Psum <- function(x, isotopes) {
  ni <- length(isotopes)
  nx <- length(x)
  p <- .P(rep.int(x, rep.int(ni, nx)), isotopes)
  dim(p) <- c(ni, nx)
  t(t(p) / colSums(p))
}

#' .monoisotopicPattern
#'
#' Model isotopic distribution by poisson distribution.
#'
#' @param x double, mass from experimental peak list
#' @param y double, intensity from experimental peak list
#' @param tolerance double, mass tolerance for .pseudoCluster
#' @param minCor double, minimal correlation between experimental and model
#' intensities
#' @param distance double, distance between isotopes (mass of a neutron; see
#' Park et al 2008); could be of length > 1; if length > 1 the order matters.
#' The first distance elements are prefered (the last elements are possible
#' removed because the contain duplicated indices).
#' @param size integer, cluster size (number of peaks for a possible cluster),
#' see .pseudoCluster
#' @return matrix, index of monoisotopic masses in first row
#' @references
#' E.J. Breen, F.G. Hopwood, K.L. Williams, and M.R. Wilkins. 2000.
#' Automatic poisson peak harvesting for high throughput protein identification.
#' Electrophoresis 21 (2000): 2243-2251.
#' @noRd
.monoisotopicPattern <- function(x, y, minCor=0.95, tolerance=1e-4,
                                 distance=1.00235, size=3L) {
  pc <- .pseudoCluster(x, size=size, distance=distance, tolerance=tolerance)
  y <- y[pc]
  dim(y) <- dim(pc)
  y <- t(t(y)/colSums(y))
  p <- .Psum(x[pc[1L,]], isotopes=0L:(size-1L))
  cr <- .colCors(y, p)
  pc <- pc[, cr > minCor, drop=FALSE]
  pc[duplicated(as.vector(pc))] <- NA_real_
  pc[, !is.na(colSums(pc)), drop=FALSE]
}

#' .monoisotopic
#'
#' Loop through multiple .monoisotopicPattern outputs and remove duplicated
#' peaks.
#'
#' @param x double, mass from experimental peak list
#' @param y double, intensity from experimental peak list
#' @param size integer vector, cluster size
#' @param \ldots further arguments passed to .monoisotopicPattern
#' @return double, index of monoisotopic masses
#' @noRd
.monoisotopic <- function(x, y, size=3L:10L, ...) {
  if (length(x) && length(x) == length(y)) {
    pattern <- lapply(sort.int(size, decreasing=TRUE),
                      function(s).monoisotopicPattern(x=x, y=y, size=s, ...))
    upattern <- .unlist(pattern)
    upattern[duplicated(upattern)] <- NA_real_
    upattern <- relist(upattern, pattern)
    sort.int(.unlist(lapply(upattern,
                            function(p)p[1L, !is.na(colSums(p))])))
  } else {
    double()
  }
}

context(".monoisotopic")

test_that(".pseudoCluster", {
  x <- c(1, 2, 3, 5, 8, 9, 10, 12, 15)
  m1s2 <- matrix(c(1, 2, 2, 3, 5, 6, 6, 7), nrow=2)
  m1s3 <- matrix(c(1, 2, 3, 5, 6, 7), nrow=3)
  m3s3 <- matrix(c(2, 4, 5, 6, 8, 9), nrow=3)
  m5s4 <- matrix(NA_real_, nrow=4, ncol=0)
  m12s3 <- matrix(c(1:3, 5:7, 1, 3, 4, 5, 7, 8), nrow=3)

  expect_error(MALDIquant:::.pseudoCluster(x, size=1),
               "The .*size.* of a cluster has to be at least 2")
  expect_equal(MALDIquant:::.pseudoCluster(x, size=2, distance=1), m1s2)
  expect_equal(MALDIquant:::.pseudoCluster(x, size=3, distance=1), m1s3)
  expect_equal(MALDIquant:::.pseudoCluster(x, size=3, distance=3), m3s3)
  expect_equal(MALDIquant:::.pseudoCluster(x, size=4, distance=5), m5s4)
  expect_equal(MALDIquant:::.pseudoCluster(x, size=3, distance=1:2), m12s3)
})

test_that(".F", {
  x <- seq(1000, 5000, by=10)
  expect_equal(MALDIquant:::.F(x), 0.000594 * x + 0.03091)
})

test_that(".P", {
  x <- seq(1000, 5000, by=10)
  expect_equal(MALDIquant:::.P(x, 0:3), dpois(0:3, MALDIquant:::.F(x)))
})

test_that(".Psum", {
  x <- c(1000, 2000)
  isotopes <- 0:3
  p <- sapply(x, function(xx) {
              pp <- MALDIquant:::.P(xx, isotopes=isotopes)
              pp/sum(pp)
  })
  expect_equal(MALDIquant:::.Psum(x, isotopes), p)
})

test_that(".monoisotopicPattern", {
  x <- c(1, 2, 3, 5, 8, 9, 10, 12, 15)
  y <- c(96, 3, 1, 5, 78, 20, 2, 12, 15)

  expect_equal(MALDIquant:::.monoisotopicPattern(1:10, 1:10),
               matrix(NA_real_, nrow=3, ncol=0))
  expect_equal(MALDIquant:::.monoisotopicPattern(x, y, distance=1, size=3),
               cbind(1:3, 5:7))
  expect_equal(MALDIquant:::.monoisotopicPattern(x, y, distance=1:2, size=3),
               cbind(1:3, 5:7))
  expect_equal(MALDIquant:::.monoisotopicPattern(x, y, distance=2:1, size=3),
               cbind(c(1, 3, 4), c(5, 7, 8)))
  expect_equal(MALDIquant:::.monoisotopicPattern(x, y, distance=1, size=2),
               cbind(1:2, 5:6))
  expect_equal(MALDIquant:::.monoisotopicPattern(x, y, distance=1, minCor=0.99), as.matrix(1:3))
})

test_that(".monoisotopic", {
  x <- c(1, 2, 3, 5, 8, 9, 10, 12, 15)
  y <- c(96, 3, 1, 5, 78, 20, 2, 12, 15)

  expect_equal(MALDIquant:::.monoisotopic(double(), double()), numeric())
  expect_equal(MALDIquant:::.monoisotopic(1:10, 1:5), numeric())
  expect_equal(MALDIquant:::.monoisotopic(1:10, 1:10), numeric())
  expect_equal(MALDIquant:::.monoisotopic(x, y, distance=1, size=2:5), c(1, 5))
  expect_equal(MALDIquant:::.monoisotopic(x, y, distance=1, minCor=0.99), 1)
})

## AbstractMassObject
setMethod(f="smoothIntensity",
          signature=signature(object="MassSpectrum"),
          definition=function(object,
                              method=c("SavitzkyGolay", "MovingAverage"),
                              halfWindowSize, ...) {
  ## empty spectrum?
  if (.isEmptyWarning(object)) {
    return(object)
  }

  method <- match.arg(method)

  fun <- switch(method,
                "SavitzkyGolay" = {
                  if (missing(halfWindowSize)) {
                    halfWindowSize <- 10L
                  }
                  .savitzkyGolay
                },
                "MovingAverage" = {
                  if (missing(halfWindowSize)) {
                    halfWindowSize <- 2L
                  }
                  .movingAverage
                }
  )

  .transformIntensity(object, fun=fun, halfWindowSize=halfWindowSize, ...)
})

## list
setMethod(f="smoothIntensity",
          signature=signature(object="list"),
          definition=function(object, ...) {

  ## test arguments
  .stopIfNotIsMassSpectrumList(object)

  .mapply(smoothIntensity, object, ...)
})

## AbstractMassObject
setMethod(f="intensity",
          signature=signature(object="AbstractMassObject"),
          definition=function(object, ...) {

  object@intensity
})

## AbstractMassObject
setReplaceMethod(f="intensity",
                 signature=signature(object="AbstractMassObject",
                                     value="numeric"),
                 definition=function(object, value) {

  if (length(object@intensity) == length(value)) {
    object@intensity <- as.double(value)
  } else {
    stop("Lengths of intensity(", length(object@intensity),
         ") and value (", length(value), ") have to be equal.")
  }
  object
})

## .as.occurrence.list
##  internal function to create a list of peaks occurrence
##
## params:
##  l: list of AbstractMassObject objects
##
## returns:
##  a list, where sample is the sample id, i is the index of the uniqueMass,
##  and mass is the unique mass vector
.as.occurrence.list.MassObjectList <- function(l) {
    .stopIfNotIsMassObjectList(l)

    mass <- .unlist(lapply(l, function(x)x@mass))
    uniqueMass <- sort.int(unique(mass))
    n <- lengths(l)

    list(
        sample = rep.int(seq_along(l), lengths(l)),
        i = findInterval(mass, uniqueMass),
        mass = uniqueMass
    )
}

p <- list(createMassPeaks(mass=1:4, intensity=11:14),
          createMassPeaks(mass=2:5, intensity=22:25))

l <- list(
    sample = rep(1:2, each = 4),
    i = c(1:4, 2:5),
    mass = 1:5
)

test_that(".as.occurrence.list.MassObjectList throws errors", {
  expect_error(.as.occurrence.list.MassObjectList(p[[1]]),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(.as.occurrence.list.MassObjectList(list()),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(.as.occurrence.list.MassObjectList(list(p, l)),
               "no list of MALDIquant::AbstractMassObject objects")
})

test_that(".as.occurrence.list.MassObjectList", {
  expect_identical(.as.occurrence.list.MassObjectList(p), l)
})

## .lapply
##  wrapper for lapply with different defaults
##
## params:
##  see ?lapply
##
## returns:
##  see ?lapply
##
.lapply <- function(X, FUN, ..., mc.cores=1L) {
  parallel::mclapply(X, FUN, ..., mc.cores=mc.cores)
}

## removeEmptyMassObjects
##  find and remove empty AbstractMassObject objects in a MassObjectsList
##
## params:
##  l: list of AbstractMassObject objects
##
## returns:
##  a list without empty objects
##
removeEmptyMassObjects <- function(l) {

  .stopIfNotIsMassObjectList(l)

  ## find empty MassPeaks objects
  notEmpty <- !(seq_along(l) %in% findEmptyMassObjects(l))

  ## exclude empty elements
  l[notEmpty]
}

context("removeEmptyMassObjects")

s <- createMassSpectrum(mass=1:10, intensity=1:10)
e <- createMassSpectrum(double(), double())

test_that("removeEmptyMassObjects throws errors", {
  expect_error(removeEmptyMassObjects(e),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(removeEmptyMassObjects(list()),
               "no list of MALDIquant::AbstractMassObject objects")
})

test_that("removeEmptyMassObjects", {
  expect_identical(removeEmptyMassObjects(list(e, e)), list())
  expect_identical(removeEmptyMassObjects(list(s, s)), list(s, s))
  expect_identical(removeEmptyMassObjects(list(s, e)), list(s))
  expect_identical(removeEmptyMassObjects(list(e, s)), list(s))
})

## MassSpectrum

## createMassSpectrum
##  default constructor: MassSpectrum class
##
## params:
##  mass: vector, spectrum mass
##  intensity: vector, spectrum intensities
##  metaData: list, metadata
##
## returns:
##  a MassSpectrum object
##
createMassSpectrum <- function(mass, intensity, metaData=list()) {
  object <- new(Class="MassSpectrum", mass=mass, intensity=intensity, metaData=metaData)
  .reorder(object)
}

## end of MassSpectrum

## MassPeaks

## createMassPeaks
##  default constructor: MassPeaks class
##
## params:
##  mass: vector, peaks mass
##  intensity: vector, peaks intensities
##  snr: vector, peaks snr
##  metaData: list, metadata
##
## returns:
##  a MassPeaks object
##
createMassPeaks <- function(mass, intensity,
                            snr=rep.int(NA_real_, length(intensity)),
                            metaData=list()) {
  object <- new(Class="MassPeaks", mass=mass, intensity=intensity, snr=snr,
                metaData=metaData)
  .reorder(object)
}

## end of MassPeaks

context("isRegular")

test_that("isRegular", {
  expect_false(isRegular(createMassSpectrum(double(), double())))
  expect_false(isRegular(createMassSpectrum(double(1), double(1))))
  expect_false(isRegular(createMassSpectrum(double(2), double(2))))
  expect_true(isRegular(createMassSpectrum(double(3), double(3))))
  expect_true(isRegular(createMassSpectrum(1:10, 1:10)))
  expect_false(isRegular(createMassSpectrum((1:10)[-8], 1:9)))
  expect_true(isRegular(createMassSpectrum((1:10)[-8], 1:9), threshold=0.15))
})

context(".replaceNegativeIntensityValues")

test_that(".replaceNegativeIntensityValues", {
  s <- suppressWarnings(createMassSpectrum(mass=1:5, intensity=-(1:5)))
  expect_warning(MALDIquant:::.replaceNegativeIntensityValues(s),
                 "Negative intensity values are replaced by zeros.")
  expect_equal(intensity(
                 MALDIquant:::.replaceNegativeIntensityValues(s, warn=FALSE)),
               rep(0L, 5L))
})

context("mass")

s <- createMassSpectrum(mass=1:10, intensity=11:20)

test_that("mass", {
  expect_identical(mass(s), 1:10)
  expect_identical(mass(s)[1:3], 1:3)
})

test_that("mass<- throws errors", {
  expect_error(mass(s) <- LETTERS[1:10])
  expect_error(mass(s) <- 1, "have to be equal")
})

test_that("mass<-", {
  mass(s) <- 11:20
  expect_equal(mass(s), 11:20)
  mass(s)[5:10] <- 5:10
  expect_equal(mass(s), c(11:14, 5:10))
})

context("coordinates")

s <- list(createMassSpectrum(mass=1:5, intensity=6:10,
                             metaData=list(imaging=list(pos=c(5, 2)))),
          createMassSpectrum(mass=1:5, intensity=6:10,
                             metaData=list(imaging=list(pos=c(6, 2)))))

test_that("coordinates", {
  expect_identical(coordinates(s[[1]]), c(5, 2))
  expect_identical(coordinates(s), cbind(x=5:6, y=c(2, 2)))
  expect_identical(coordinates(s, adjust=TRUE), cbind(x=1:2, y=c(1, 1)))
  expect_null(coordinates(list(createMassSpectrum(1:5, 1:5),
                               createMassSpectrum(1:5, 1:5))))
})

test_that("coordinates<- throws errors", {
  expect_error(coordinates(s[[1]]) <- LETTERS[1:10])
  expect_error(coordinates(s[[1]]) <- 1, "2 or 3 coordinates are needed!")
  expect_error(coordinates(s[[1]]) <- 4, "2 or 3 coordinates are needed!")
  expect_error(coordinates(s[[1]]) <- cbind(x=1, y=1, z=1, zz=1),
               "2 or 3 coordinates are needed!")
  expect_error(coordinates(s) <- cbind(x=1:4), "2 or 3 coordinates are needed!")
  l <- list(1:10)
  expect_error(coordinates(l) <- cbind(x=2:3, y=3:4),
               "is no list of MALDIquant::AbstractMassObject objects!")
})

test_that("coordinates<- throws warnings", {
  expect_warning(coordinates(s[[1]]) <- cbind(x=1:2, y=1:2),
                 "all rows but the first are ignored")
})

test_that("coordinates<-", {
  coordinates(s[[1]]) <- 2:3
  expect_equal(coordinates(s[[1]]), 2:3)
  coordinates(s[[1]]) <- cbind(x=2, y=3)
  expect_equal(coordinates(s[[1]]), c(x=2, y=3))
  coordinates(s[[1]])[2] <- 5
  expect_equal(coordinates(s[[1]]), c(x=2, y=5))
  coordinates(s) <- cbind(x=2:3, y=3:4)
  expect_equal(coordinates(s), cbind(x=2:3, y=3:4))
})

context("approxfun")

test_that("approxfun", {
  s <- createMassSpectrum(mass=1:10, intensity=11:20)
  e <- createMassSpectrum(mass=double(), intensity=double())
  p <- createMassPeaks(mass=1:10, intensity=11:20)

  expect_error(MALDIquant:::approxfun(p))

  expect_equal(approxfun(s)(seq(0, 11, by=0.5)),
               c(NA, NA, seq(11, 20, by=0.5), NA, NA))
  expect_equal(approxfun(e)(1:20), rep(NA, 20))
})

context("filterPeaks")

p <- createMassPeaks(mass=1:5, intensity=1:5)
l <- list(p, p[1:4], p[1:3], p[1:2])

test_that("filterPeaks throws errors", {
  expect_error(filterPeaks(p), "no list of MALDIquant::MassPeaks")
  expect_error(filterPeaks(list()), "no list of MALDIquant::MassPeaks")
  expect_error(filterPeaks(l, minFrequency=NA, minNumber=NA),
                 " has to be a meaningful number")
  expect_error(filterPeaks(l=l, labels=as.factor("a"), minFrequency=1),
               "For each item in .*l.* there must be a label in .*labels.*")
})

test_that("filterPeaks shows warnings", {
  expect_warning(filterPeaks(l, minFrequency=2),
                 "Empty peak whitelist for level")
  expect_warning(filterPeaks(l, minFrequency=-1),
                 " < 0 does not make sense! Using 0 instead")
  expect_warning(filterPeaks(l, minNumber=10),
                 "Empty peak whitelist for level")
  expect_warning(filterPeaks(l, minNumber=-1),
                 " < 0 does not make sense! Using 0 instead")
  expect_warning(filterPeaks(l, minFrequency=2/3, minNumber=2),
                 " arguments are given. Choosing the higher one.")
  expect_warning(filterPeaks(l, minNumber=2, labels=c(1, 2, 2, 2)),
                 "Empty peak whitelist for level")
})

test_that("filterPeaks", {
  expect_identical(filterPeaks(l, minFrequency=1),
                   list(p[1:2], p[1:2], p[1:2], p[1:2]))

  expect_identical(filterPeaks(l, minFrequency=0.5),
                   list(p[1:4], p[1:4], p[1:3], p[1:2]))

  expect_identical(filterPeaks(l, minFrequency=0), l)

  expect_identical(filterPeaks(l, minFrequency=1,
                               labels=factor(rep(letters[1:2], each=2),
                                             levels=letters[1:2])),
                   list(p[1:4], p[1:4], p[1:2], p[1:2]))

  ## test unused levels
  expect_identical(filterPeaks(l, minFrequency=1,
                               labels=factor(rep(letters[1:2], each=2),
                                             levels=letters[1:5])),
                   list(p[1:4], p[1:4], p[1:2], p[1:2]))

  ## test numbers
  expect_identical(filterPeaks(l, minFrequency=1,
                               labels=rep(1:2, each=2)),
                   list(p[1:4], p[1:4], p[1:2], p[1:2]))
  ## test case for #22 (unexpected results for different number of technical
  ## replicates because using of floor(minFrequency))
  expect_identical(filterPeaks(list(p, p[1:4], p[1:3], p, p[1:4], p[1:3],
                                    p[1:2]), minFrequency=2/3,
                               labels=c(rep(1, 3), rep(2, 4))),
                   list(p[1:4], p[1:4], p[1:3], p[1:3], p[1:3], p[1:3], p[1:2]))

  ## use absolute threshold
  expect_identical(filterPeaks(list(p, p[1:4], p[1:3], p, p[1:4], p[1:3],
                                    p[1:2]), minNumber=2,
                               labels=c(rep(1, 3), rep(2, 4))),
                   list(p[1:4], p[1:4], p[1:3], p[1:4], p[1:4], p[1:3], p[1:2]))

  ## test case for #26 (minNumber > n removes all peaks)
  ## since MALDIquant 1.8.16 min{Frequency,minNumber} could be vectors so this
  ## is not needed any more and we could use min{Frequency,Number} as feature to
  ## remove all peaks (or avoid the generation of a whitelist if
  ## mergeWhitelists=TRUE)
  expect_equal(suppressWarnings(filterPeaks(list(p, p[1:4], p),
                                                minNumber=2,
                                                labels=c(1, 2, 2))),
               list(createMassPeaks(double(), double()), p[1:4], p[1:4]))
})

test_that("filterPeaks mergeWhitelists argument works", {
  p2 <- list(
    createMassPeaks(1:5, 1:5),
    createMassPeaks(1:4, 1:4),
    createMassPeaks(4:9, 4:9),
    createMassPeaks(4:8, 4:8),
    createMassPeaks(1:5, 1:5),
    createMassPeaks(2:6, 2:6))

  expect_identical(filterPeaks(p2, minFrequency=1, labels=rep(1:3, each=2),
                               mergeWhitelists=FALSE),
                   c(p2[[2]], p2[[2]], p2[[4]], p2[[4]],
                     createMassPeaks(2:5, 2:5), createMassPeaks(2:5, 2:5)))
  expect_identical(filterPeaks(p2, minFrequency=1, labels=rep(1:3, each=2),
                               mergeWhitelists=TRUE),
                   c(p2[1:2], p2[[4]], p2[[4]], p2[5:6]))
  expect_identical(suppressWarnings(filterPeaks(p2, minFrequency=c(1, 1, 2),
                                                labels=rep(1:3, each=2),
                                                mergeWhitelists=TRUE)),
                   c(p2[1:2], p2[[4]], p2[[4]], p2[5:6]))
  ## test with different numbers and frequencies
  expect_identical(suppressWarnings(filterPeaks(p2, minFrequency=c(1, NA, NA),
                                                minNumber=c(NA, 3, 1),
                                                labels=rep(1:3, each=2),
                                                mergeWhitelists=TRUE)),
                   c(p2[1:2], createMassPeaks(4:6, 4:6),
                     createMassPeaks(4:6, 4:6), p2[5:6]))
})

context("referencePeaks")

p <- c(createMassPeaks(mass=1:5, intensity=1:5),
       createMassPeaks(mass=1:4, intensity=1:4),
       createMassPeaks(mass=1:3, intensity=1:3),
       createMassPeaks(mass=1:2, intensity=1:2))

p2 <- c(createMassPeaks(mass=c(1, 1.001, 3), intensity=c(2, 1, 1)),
        createMassPeaks(mass=c(0.99, 3), intensity=rep(1, 2)),
        createMassPeaks(mass=c(1.02, 3), intensity=rep(1, 2)))

test_that("referencePeaks throws errors", {
  expect_error(referencePeaks(list(a="a", b="b"), minFrequency=0.5),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(referencePeaks(p[[1]], minFrequency=0.5),
               "no list of MALDIquant::MassPeaks objects")
})

test_that("referencePeaks shows warnings", {
  expect_warning(referencePeaks(p, minFrequency=-1),
                 " < 0 does not make sense! Using 0 instead")
  expect_warning(referencePeaks(p, minFrequency=10),
                 "Empty peak whitelist for level")
})

test_that("referencePeaks works with different frequencies", {
  expect_equal(referencePeaks(p, minFrequency=1),
               createMassPeaks(mass=1:2, intensity=rep(1, 2)))

  expect_equal(referencePeaks(p, minFrequency=0.5),
               createMassPeaks(mass=1:4, intensity=c(1, 1, 3/4, 0.5)))
})

test_that("referencePeaks works with different methods", {
  expect_equal(referencePeaks(p2, minFrequency=1, tolerance=0.05),
               createMassPeaks(mass=3, intensity=1))
  expect_equal(referencePeaks(p2, method="strict", minFrequency=1,
                              tolerance=0.05),
               createMassPeaks(mass=3, intensity=1))
  expect_equal(referencePeaks(p2, method="relaxed", minFrequency=1,
                              tolerance=0.05),
               createMassPeaks(mass=c(1+1/300, 3), intensity=rep(1, 2)))
})

context("labelPeaks")

test_that("labelPeaks", {
  pdf(NULL)
  on.exit(dev.off())
  plot.new()

  p <- createMassPeaks(1:5, 1:5)
  expect_error(labelPeaks(p, index=-1),
               "No valid .*index.* nor .*mass.* given.")
  expect_error(labelPeaks(p, index=1:10),
               "No valid .*index.* nor .*mass.* given.")
  expect_error(labelPeaks(p, index=1, labels=LETTERS[1:5]),
               "Lenghts of .*index.*/.*mass.* and .*labels.* have to be equal")
  expect_error(labelPeaks(p, srt=45, avoidOverlap=TRUE),
               ".*avoidOverlap = TRUE.* and .*srt != x \\* 90.* is not supported.")
})

context("determineWarpingFunctions")

r <- createMassPeaks(mass=1:10, intensity=1:10)
p <- createMassPeaks(mass=(1:10)+0.01, intensity=1:10)

test_that("determineWarpingFunctions throws errors", {
  expect_error(determineWarpingFunctions(1:10),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(determineWarpingFunctions(p, reference=createMassPeaks(double(),
                                                                      double())),
               "Reference MassPeaks object contains no peaks")
  expect_error(determineWarpingFunctions(createMassPeaks(mass=20, intensity=20),
                                         reference=r),
               "Could not match any peak in spectrum 1 to a reference peak")
})

test_that("determineWarpingFunctions throws warnings", {
  expect_warning(determineWarpingFunctions(p, reference=r[6:10]),
                 "Reference MassPeaks object contains very few peaks")
  expect_error(tryCatch(determineWarpingFunctions(list(p, p), plot=TRUE),
                        warning=function(w)stop(conditionMessage(w))),
               paste0(".*plot.* is .*TRUE.* but no non-interactive device ",
                      "is available. Using pdf.* to create a default one."))
  expect_error(tryCatch(determineWarpingFunctions(list(p, p), plot=TRUE,
                                                  plotInteractive=TRUE),
                        warning=function(w)stop(conditionMessage(w))),
               paste0(".*plot.* is .*TRUE.* but no interactive device is ",
                      "available. Using dev.new.* to create a default one."))
  expect_warning(determineWarpingFunctions(list(p, createMassPeaks(11, 11)),
                                           reference=r, allowNoMatches=TRUE),
                 "Could not match any peak in spectrum 2 to a reference peak")
})

test_that("determineWarpingFunctions works with single MassPeaks object", {
  w <- determineWarpingFunctions(p, reference=r, method="linear")
  expect_equal(attr(w, "nmatch"), 6)
  wp <- warpMassPeaks(list(p), w)[[1]]
  expect_equal(r, wp)
})

test_that("determineWarpingFunctions works with list of MassPeaks objects", {
  w <- determineWarpingFunctions(list(a=p, b=p), reference=r, method="linear")
  expect_equal(attr(w, "nmatch"), c(a=6, b=6))
  wp <- warpMassPeaks(list(p, p), w)
  expect_equal(list(r, r), wp)
})

test_that("determineWarpingFunctions supports allowNoMatches argument", {
  suppressWarnings(
    w <- determineWarpingFunctions(
        list(p, createMassPeaks(11, 11), p), reference=r, allowNoMatches=TRUE)
  )
  expect_equal(sapply(w, is.function), c(TRUE, FALSE, TRUE))
  expect_equal(is.na(w), c(FALSE, TRUE, FALSE))
  expect_equal(attr(w, "nmatch"), c(6, 0, 6))
  wp <- warpMassPeaks(list(p, createMassPeaks(11, 11), p), w, emptyNoMatches=TRUE)
  expect_equal(list(r, createMassPeaks(11, 0), r), wp)
})

context("monoisotopicPeaks")

p <- createMassPeaks(mass=995:1005,
                     intensity=c(100, 10, 30, 10, 40, 550, 330, 110, 10, 5, 15))
m <- createMassPeaks(mass=1000, intensity=550)

test_that("monoisotopicPeaks", {
  expect_equal(monoisotopicPeaks(p), m)
})

test_that("detectPeaks works with list of MassPeaks objects", {
  expect_error(monoisotopicPeaks(list(x=1, y=1)),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(monoisotopicPeaks(list(createMassSpectrum(1, 1),
                                      createMassSpectrum(1, 1)),
               "no list of MALDIquant::MassPeaks objects"))
  expect_equal(monoisotopicPeaks(list(p, p)), list(m, m))
})

context(".as.matrix")

p <- list(createMassPeaks(mass=1:4, intensity=11:14),
          createMassPeaks(mass=2:5, intensity=22:25))
s <- list(createMassSpectrum(mass=1:5, intensity=11:15),
          createMassSpectrum(mass=1:5, intensity=21:25))

mp <- matrix(c(11:14, NA_real_, NA_real_, 22:25), byrow=TRUE, ncol=5, nrow=2,
             dimnames=list(NULL, 1:5))
ms <- matrix(as.double(c(11:15, 21:25)), byrow=TRUE, ncol=5, nrow=2,
             dimnames=list(NULL, 1:5))

mb <- matrix(c(rep(1L, 4), 0L, 0L, rep(1L, 4)), byrow=TRUE, ncol=5, nrow=2,
             dimnames=list(NULL, 1:5))
attr(mp, "mass") <- attr(ms, "mass") <- attr(mb, "mass") <- 1:5

test_that(".as.matrix.MassObjectsList throws errors", {
  expect_error(MALDIquant:::.as.matrix.MassObjectList(p[[1]]),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(MALDIquant:::.as.matrix.MassObjectList(list()),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(MALDIquant:::.as.matrix.MassObjectList(list(p, mp)),
               "no list of MALDIquant::AbstractMassObject objects")
})

test_that(".as.binary.matrix", {
  expect_error(MALDIquant:::.as.binary.matrix(p[[1]]))
})

test_that(".as.matrix.MassObjectsList", {
  expect_identical(MALDIquant:::.as.matrix.MassObjectList(p), mp)
  expect_identical(MALDIquant:::.as.matrix.MassObjectList(s), ms)
})

test_that(".as.binary.matrix", {
  expect_identical(MALDIquant:::.as.binary.matrix(mp), mb)
})

context("smoothIntensity")

i <- c(1:20, 20:1)^2
s <- createMassSpectrum(mass=1:40, intensity=i)

test_that("smoothIntensity shows warnings", {
  expect_warning(smoothIntensity(
                   createMassSpectrum(mass=double(), intensity=double())),
                 "empty")
})

test_that("smoothIntensity", {
  expect_equal(intensity(smoothIntensity(s, method="SavitzkyGolay",
                                         halfWindowSize=4)),
               MALDIquant:::.savitzkyGolay(i, halfWindowSize=4))
  expect_equal(intensity(smoothIntensity(s, method="SavitzkyGolay")),
               MALDIquant:::.savitzkyGolay(i, halfWindowSize=10))
  expect_equal(length(smoothIntensity(s, method="MovingAverage",
                                      halfWindowSize=2)), 40)
  expect_equal(intensity(smoothIntensity(s, method="MovingAverage",
                                         halfWindowSize=2)),
               MALDIquant:::.movingAverage(i, halfWindowSize=2))
  expect_equal(length(smoothIntensity(s, method="MovingAverage",
                                      halfWindowSize=2, weighted=TRUE)), 40)
  expect_equal(intensity(smoothIntensity(s, method="MovingAverage",
                                         halfWindowSize=2, weighted=TRUE)),
               MALDIquant:::.movingAverage(i, halfWindowSize=2, weighted=TRUE))
  expect_equal(intensity(smoothIntensity(s, method="MovingAverage",
                                         halfWindowSize=2, weighted=TRUE)),
               intensity(smoothIntensity(s, method="MovingAverage", 
                                          weighted=TRUE)))
  expect_equal(intensity(smoothIntensity(s, method="MovingAverage", 
                                          halfWindowSize=2)),
               intensity(smoothIntensity(s, method="MovingAverage")))
})

test_that("smoothIntensity works with list of MassSpectrum objects", {
  expect_error(smoothIntensity(list(x=1, y=1)),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(smoothIntensity(list(s, createMassPeaks(1, 1)),
               "no list of MALDIquant::MassSpectrum objects"))
  r <- createMassSpectrum(mass=1:40, intensity=
                          MALDIquant:::.movingAverage(i))
  expect_equal(smoothIntensity(list(s, s), method="MovingAverage"), list(r, r))
})

context("valid")

test_that(".validAbstractMassObject", {
  s <- createMassSpectrum(mass=1:10, intensity=1:10)
  expect_true(MALDIquant:::.validAbstractMassObject(s))
  s@intensity <- 1
  expect_identical(MALDIquant:::.validAbstractMassObject(s),
                   "Lengths of mass (10) and intensity (1) have to be equal.")
  s@mass <- -1
  expect_warning(MALDIquant:::.validAbstractMassObject(s),
                 "Negative mass values found.")
  s@mass <- 1
  s@intensity <- -1
  expect_warning(MALDIquant:::.validAbstractMassObject(s),
                 "Negative intensity values found.")

  s@mass <- 2:1
  s@intensity <- 1:2
  expect_warning(MALDIquant:::.validAbstractMassObject(s),
                 "Unsorted mass values found.")
})

test_that(".validMassPeaks", {
  p <- createMassPeaks(mass=1:10, intensity=1:10, snr=1:10)

  expect_true(MALDIquant:::.validMassPeaks(p))
  p@intensity <- 1
  expect_identical(MALDIquant:::.validMassPeaks(p),
                   "Lengths of intensity (1) and snr (10) have to be equal.")
})

context("as.matrix")

m <- matrix(c(1:10, 11:20), ncol=2, dimnames=list(c(), c("mass", "intensity")))
s <- createMassSpectrum(mass=1:10, intensity=11:20)
p <- createMassPeaks(mass=1:10, intensity=11:20)

test_that("as.matrix works without indices", {
  expect_equal(as.matrix(s), m)
  expect_equal(as.matrix(p), m)
})

test_that("as.matrix works with indices", {
  expect_equal(as.matrix(s)[3:4,], m[3:4,])
  expect_equal(as.matrix(s)[7, 1], m[7, 1])
  expect_equal(as.matrix(s)[mass(s) > 5, 2], 16:20)
})

context("binPeaks")

p <- list(createMassPeaks(mass=seq(100, 500, 100), intensity=1:5),
          createMassPeaks(mass=c(seq(100.2, 300.2, 100), 395), intensity=1:4))

p2 <- c(createMassPeaks(mass=c(1.009, 1.01, 3), intensity=c(2, 1, 1), snr=1:3),
        createMassPeaks(mass=c(1, 3), intensity=1:2, snr=1:2),
        createMassPeaks(mass=c(1.03, 3), intensity=1:2, snr=1:2))

test_that("binPeaks throws errors", {
  expect_error(binPeaks(list()), "no list of MALDIquant::MassPeaks")
  expect_error(binPeaks(p, method="foobar"),
               ".*arg.* should be one of .*strict.*, .*relaxed.*")

})

test_that("binPeaks bins peaks strict", {
  b <- binPeaks(p, tolerance=0.002)
  expect_true(all(b[[1]]@mass[1:3]==b[[2]]@mass[1:3]))
  expect_false(all(b[[1]]@mass[4]==b[[2]]@mass[4]))
  expect_true(length(b[[1]])==5)
  expect_true(length(b[[2]])==4)
  expect_false(all(p[[1]]@mass==b[[1]]@mass))
  expect_false(all(p[[2]]@mass==b[[2]]@mass))

  b <- binPeaks(p, tolerance=0.1)
  expect_true(all(b[[1]]@mass[1:4]==b[[2]]@mass[1:4]))
  expect_true(length(b[[1]])==5)
  expect_true(length(b[[2]])==4)
  expect_false(all(p[[1]]@mass==b[[1]]@mass))
  expect_false(all(p[[2]]@mass==b[[2]]@mass))
})

test_that("binPeaks bins peaks releaxed", {
  b <- binPeaks(p2, method="relaxed", tolerance=0.05)
  expect_true(all(b[[1]]@mass==c(1.01, 1.013, 3)))
  ip <- sort(p2[[1]]@mass, index.return=TRUE)
  ib <- sort(b[[1]]@mass, index.return=TRUE)
  expect_equal(ip$ix, ib$ix)
  expect_false(all(p2[[1]]@intensity == b[[1]]@intensity))
  expect_false(all(p2[[1]]@snr== b[[1]]@snr))
  expect_true(all(b[[1]]@intensity == c(1, 2, 1)))
  expect_true(all(b[[2]]@intensity == 1:2))
  expect_true(all(b[[3]]@intensity == 1:2))
  expect_true(all(b[[1]]@snr == c(2, 1, 3)))
  expect_true(all(b[[2]]@snr == 1:2))
  expect_true(all(b[[3]]@snr == 1:2))
})

test_that("binPeaks bins peaks to reference", {
  ref <- createMassPeaks(mass=1:3, intensity=1:3, snr=1:3)
  r <- c(createMassPeaks(mass=c(1, 1.01, 3), intensity=c(2, 1, 1), snr=1:3),
         createMassPeaks(mass=c(1, 3), intensity=1:2, snr=1:2),
         createMassPeaks(mass=c(1, 3), intensity=1:2, snr=1:2))

  expect_equal(binPeaks(c(ref, p2), method="reference", tolerance=0.05)[-1], r)
})

test_that("binPeaks don't introduce new peaks; issue 61", {
    p1 <- createMassPeaks(1:5, 1:5, metaData=list(name="foo"))
    p0 <- createMassPeaks(numeric(), numeric(), metaData=list(name="bar"))
    expect_equal(binPeaks(list(p1, p0)), list(p1, p0))
    expect_equal(binPeaks(list(p0, p1)), list(p0, p1))
    expect_equal(binPeaks(list(p0, p0, p1)), list(p0, p0, p1))
    expect_equal(binPeaks(list(p0, p1, p1)), list(p0, p1, p1))
})

context("as")

test_that("as", {
  p <- createMassPeaks(mass=1:10, intensity=11:20, snr=rep(1, 10),
                       metaData=list(file="foo"))
  s <- createMassSpectrum(mass=1:10, intensity=11:20, metaData=list(file="foo"))
  expect_equal(as(p, "MassSpectrum"), s)
})

context(".reorder")

test_that(".reorder", {
  s <- createMassSpectrum(mass=1:5, intensity=1:5)
  ## do nothing
  expect_equal(intensity(MALDIquant:::.reorder(s)), 1:5)

  ## reorder
  s@mass <- 5:1
  expect_warning(MALDIquant:::.reorder(s),
                 "Mass and intensity values are reordered.")
  expect_equal(intensity(MALDIquant:::.reorder(s, warn=FALSE)), 5:1)
})

context("plotMsiSlice")

m <- c(createMassSpectrum(mass=1:10, intensity=1:10),
       createMassSpectrum(mass=2:11, intensity=2:11))
coordinates(m) <- cbind(1:2, c(1, 1))

test_that("plotMsiSlice throws errors", {
  expect_error(plotMsiSlice(m, center=c(3, 5), tolerance=0.5,
                            colRamp=list(A=1, B=2, C=3)),
               paste0(".*number of centers.* has to be the same ",
                      "as the length of the list .*colRamp.*"))
})

test_that("plotMsiSlice throws warnings", {
  expect_error(tryCatch(plotMsiSlice(m, center=c(3, 5), tolerance=0.5),
                        warning=function(w)stop(conditionMessage(w))),
               paste0(".*plotMsiSlice.* was called for multiple slices on an ",
                      "interactive device. Only the first slice is plotted."))
})

context("metaData")

s <- createMassSpectrum(mass=1:10, intensity=11:20, metaData=list(name="test spectrum"))

test_that("metaData", {
  expect_identical(metaData(s)$name, "test spectrum")
})

test_that("metaData<-", {
  l <- list(a=1:3, b=letters[1:3])
  metaData(s) <- l
  expect_identical(metaData(s), l)
})

context("intensityMatrix")

p <- list(createMassPeaks(mass=1:4, intensity=11:14),
          createMassPeaks(mass=2:5, intensity=22:25))
s <- list(createMassSpectrum(mass=1:5, intensity=11:15),
          createMassSpectrum(mass=1:5, intensity=21:25))

m <- matrix(c(11:14, NA_real_, NA_real_, 22:25), byrow=TRUE, ncol=5, nrow=2,
            dimnames=list(NULL, 1:5))
e <- matrix(c(11:15, 21:25), byrow=TRUE, ncol=5, nrow=2,
            dimnames=list(NULL, 1:5))
attr(m, "mass") <- attr(e, "mass") <- 1:5

test_that("intensityMatrix throws errors", {
  expect_error(intensityMatrix(p[[1]]),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(intensityMatrix(list()),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(intensityMatrix(p, list()),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(intensityMatrix(p, s[1]),
               "Incompatible number of spectra")
})

test_that("intensityMatrix", {
    expect_identical(intensityMatrix(p), m)
    expect_equal(intensityMatrix(p, s), e)
})

context("show")

test_that("show", {
  x <- createMassSpectrum(mass=1:10, intensity=11:20,
                          metaData=list(name="example", file="example.mzML"))
  r <- paste0(format(c("S4 class type",
                       "Number of m/z values",
                       "Range of m/z values",
                       "Range of intensity values",
                       "Memory usage",
                       "Name",
                       "File"), justify="left"), ": ",
              c(format(c("MassSpectrum",
                         "10",
                         "1 - 10",
                         "11 - 20",
                         MALDIquant:::.memoryUsageStr(x),
                         "example"), justify="left"),
                "example.mzML"))

  expect_identical(capture.output(x), r)

  x <- createMassPeaks(mass=1:10, intensity=11:20,
                       metaData=list(name="example"))
  r <- paste0(format(c("S4 class type",
                       "Number of m/z values",
                       "Range of m/z values",
                       "Range of intensity values",
                       "Range of snr values",
                       "Memory usage",
                       "Name"), justify="left"), ": ",
              format(c("MassPeaks",
                       "10",
                       "1 - 10",
                       "11 - 20",
                       "NA - NA",
                       MALDIquant:::.memoryUsageStr(x),
                       "example"), justify="left"))

  expect_identical(capture.output(x), r)

  x <- createMassPeaks(mass=1:10, intensity=11:20, snr=1:10,
                       metaData=list(name=c("example1", "example2"),
                                     file=c("example1.txt", "example2.txt")))
  r <- paste0(format(c("S4 class type",
                       "Number of m/z values",
                       "Range of m/z values",
                       "Range of intensity values",
                       "Range of snr values",
                       "Memory usage",
                       "Name1",
                       "Name2",
                       "File1",
                       "File2"), justify="left"), ": ",
              c(format(c("MassPeaks",
                         "10",
                         "1 - 10",
                         "11 - 20",
                         "1 - 10",
                         MALDIquant:::.memoryUsageStr(x),
                         "example1",
                         "example2"), justify="left"),
                "example1.txt", "example2.txt"))

  expect_identical(capture.output(x), r)

  x <- createMassPeaks(double(), double())
  r <- paste0(format(c("S4 class type",
                       "Number of m/z values",
                       "Range of m/z values",
                       "Range of intensity values",
                       "Range of snr values",
                       "Memory usage"), justify="left"), ": ",
              format(c("MassPeaks",
                       "0",
                       "NA",
                       "NA",
                       "NA",
                       MALDIquant:::.memoryUsageStr(x)), justify="left"))

  expect_identical(capture.output(x), r)
})

context("calibrateIntensity")

s <- list(createMassSpectrum(mass=1:5, intensity=1:5),
          createMassSpectrum(mass=1:5, intensity=2:6),
          createMassSpectrum(mass=1:5, intensity=3:7))

test_that("calibrateIntensity,MassSpectrum throws errors", {
  expect_error(calibrateIntensity(s[[1]], method="foobar"),
               ".*arg.* should be one of .*TIC.*, .*PQN.*, .*median.*")
  expect_error(calibrateIntensity(s[[1]], method="PQN"),
               ".*PQN.* is not supported for a single MassSpectrum object")
})

test_that("calibrateIntensity,list throws errors", {
  expect_error(calibrateIntensity(list(1:10), method="TIC"),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(calibrateIntensity(s, method="foobar"),
               ".*arg.* should be one of .*TIC.*, .*PQN.*, .*median.*")
})

test_that("calibrateIntensity,MassSpectrum throws warnings", {
  # median == zero; see #51
  m <- createMassSpectrum(mass=1:5, intensity=c(rep(0, 4), 5))
  expect_warning(calibrateIntensity(m, method="median"),
               "Scaling factor is zero. No calibration applied.")
})

test_that("calibrateIntensity works with TIC", {
  sTIC <- calibrateIntensity(s[[1]], method="TIC")
  expect_equal(totalIonCurrent(sTIC), 1)
  sTIC <- calibrateIntensity(s, method="TIC")
  expect_equal(unlist(lapply(sTIC, totalIonCurrent)), rep(1, 3))
  sTIC <- calibrateIntensity(s, method="TIC", range=c(2, 4))
  expect_equal(unlist(lapply(sTIC, totalIonCurrent)), rep(2, 3))
})

test_that("calibrateIntensity works with median", {
  sMed <- calibrateIntensity(s[[1]], method="median")
  expect_equal(intensity(sMed), intensity(s[[1]])/median(intensity(s[[1]])))
  sMed <- calibrateIntensity(s, method="median")
  expect_equal(lapply(sMed, intensity),
               lapply(s, function(x)intensity(x)/median(intensity(x))))
  sMed <- calibrateIntensity(s, method="median", range=c(2, 4))
  expect_equal(unlist(lapply(sMed, intensity)),
               as.vector(mapply(function(sp, m)intensity(sp)/m, sp=s, m=3:5)))
  # median == zero; see #51
  m <- createMassSpectrum(mass=1:5, intensity=c(rep(0, 4), 5))
  expect_equal(suppressWarnings(calibrateIntensity(m, method="median")), m)
})

test_that("calibrateIntensity works with PQN", {
  sPQN <- calibrateIntensity(s, method="PQN")
  expect_equal(sPQN, calibrateIntensity(s, method="TIC"))
  m <- list(s[[1]], createMassSpectrum(1:5, rep(2, 5)),
            createMassSpectrum(1:5, c(4:6, 2:1)))
  mPQN <- calibrateIntensity(m, method="PQN")
  expect_equal(unlist(lapply(mPQN, totalIonCurrent)),
               c(1, 1, 0.96875))
  sPQN <- calibrateIntensity(s, method="PQN", range=c(2, 4))
  expect_equal(sPQN, calibrateIntensity(s, method="TIC", range=c(2, 4)))
})

context("intensity")

s <- createMassSpectrum(mass=1:10, intensity=11:20)

test_that("intensity", {
  expect_identical(intensity(s), 11:20)
  expect_identical(intensity(s)[1:3], 11:13)
})

test_that("intensity<- throws errors", {
  expect_error(intensity(s) <- LETTERS[1:10])
  expect_error(intensity(s) <- 1, "have to be equal")
})

test_that("intensity<-", {
  intensity(s) <- 1:10
  expect_equal(intensity(s), 1:10)
  intensity(s)[5:10] <- 15:20
  expect_equal(intensity(s), c(1:4, 15:20))
})

context("totalIonCurrent")

s <- createMassSpectrum(mass=1:10, intensity=1:10)
i <- createMassSpectrum(mass=1:2, intensity=rep.int(.Machine$integer.max, 2))
e <- createMassSpectrum(mass=1:10, intensity=rep(0, 10))

test_that("totalIonCurrent", {
  expect_equal(totalIonCurrent(s), 49.5)
  expect_equal(totalIonCurrent(i), as.double(.Machine$integer.max))
})

context(".doByLabels")

s <- createMassSpectrum(mass=1:5, intensity=1:5)

test_that(".doByLabels throws errors", {
  expect_error(MALDIquant:::.doByLabels(l=list(x=1, y=1),
                                        labels=as.factor(c("a", "b")),
                                        FUN=sum),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(MALDIquant:::.doByLabels(l=list(s, s), labels=as.factor("a"),
                                        FUN=sum),
               "For each item in .*l.* there must be a label in .*labels.*")
  expect_error(MALDIquant:::.doByLabels(l=list(s, s),
                                        labels=as.factor(c("a", "b"))))
})

test_that(".doByLabels runs a function for group labels", {
  expect_identical(MALDIquant:::.doByLabels(l=list(s, s),
                                            FUN=function(x){ return(1) }), 1)
  expect_identical(unname(MALDIquant:::.doByLabels(l=list(s, s),
                            labels=as.factor(c("a", "b")),
                            FUN=function(x)1)),
                   c(1, 1))

  l <- list(s, s, s, s)
  l[[1]]@metaData$file <- 1
  l[[2]]@metaData$file <- 2
  l[[3]]@metaData$file <- 10
  l[[4]]@metaData$file <- 11

  ## preserve order
  m <- MALDIquant:::.doByLabels(l=l,
                  ## error because of wrong alphabetical order
                  ## always add levels
                  #labels=as.factor(paste("s", c(1, 2, 10, 11))),
                  labels=factor(paste("s", c(1, 2, 10, 11)),
                        levels=paste("s", c(1, 2, 10, 11))),
                  FUN=function(x)x)

  expect_equal(c(1:2, 10:11), unname(sapply(m, function(x)metaData(x)$file)))

  ## bug #19; order changes if length of output is smaller than input
  m <- MALDIquant:::.doByLabels(l=l,
                  ## results in factor(c(2, 2, 1, 1), levels=2:1)
                  labels=rep(2:1, each=2),
                  FUN=function(x)x[[1]])

  expect_equal(c(1, 10), unname(sapply(m, function(x)metaData(x)$file)))

  ## respect order of factor
  m <- MALDIquant:::.doByLabels(l=l,
                  ## results in factor(c(2, 2, 1, 1), levels=1:2)
                  labels=factor(rep(2:1, each=2)),
                  FUN=function(x)x[[1]])

  expect_equal(c(10, 1), unname(sapply(m, function(x)metaData(x)$file)))


  ## see https://github.com/sgibb/MALDIquant/issues/1
  expect_equal(unname(MALDIquant:::.doByLabels(l, 1:4, function(x)x[[1]])), l)
  expect_equal(unname(MALDIquant:::.doByLabels(l, rep(1, 4), function(x)x)), l)
})

context("subset")

s <- createMassSpectrum(mass=1:10, intensity=11:20)
p <- createMassPeaks(mass=1:10, intensity=11:20, snr=21:30)

test_that("numerical index based subsetting works on MassSpectrum", {
  expect_equal(s[2:9], createMassSpectrum(mass=2:9, intensity=12:19))
  expect_equal(s[-1], createMassSpectrum(mass=2:10, intensity=12:20))
})

test_that("logical index based subsetting works on MassSpectrum", {
  expect_equal(s[mass(s) <= 9],
               createMassSpectrum(mass=1:9, intensity=11:19))
  expect_equal(s[intensity(s) <= 15],
               createMassSpectrum(mass=1:5, intensity=11:15))
})

test_that("numerical index based subsetting works on MassPeaks", {
  expect_equal(p[2:9], createMassPeaks(mass=2:9, intensity=12:19, snr=22:29))
  expect_equal(p[-1], createMassPeaks(mass=2:10, intensity=12:20, snr=22:30))
})

test_that("logical index based subsetting works on MassPeaks", {
  expect_equal(p[mass(p) <= 9],
               createMassPeaks(mass=1:9, intensity=11:19, snr=21:29))
  expect_equal(p[intensity(p) <= 15],
               createMassPeaks(mass=1:5, intensity=11:15, snr=21:25))
})

context("removeBaseline")

s <- createMassSpectrum(mass=1:20, intensity=rep(10:1, 2))
e <- createMassSpectrum(double(), double())

test_that("removeBaseline throws errors", {
  expect_error(removeBaseline(s, method="foobar"),
               ".*arg.* should be one of .*SNIP.*, .*ConvexHull.*, .*median.*")
})

test_that("removeBaseline shows warnings", {
  expect_warning(estimateBaseline(e), "empty")
})

test_that("removeBaseline returns an empty spectrum if input is empty", {
  expect_identical(suppressWarnings(removeBaseline(e)), e)
})

test_that("removeBaseline works with ConvexHull", {
  expect_equal(removeBaseline(s, method="ConvexHull"),
               createMassSpectrum(mass=1:20, intensity=c(rep(0, 10), 9:0)))
})

test_that("removeBaseline works with median", {
  ## halfWindowSize
  expect_error(removeBaseline(s, method="median", halfWindowSize=0),
               "too small")
  expect_error(removeBaseline(s, method="median"),
               "too large")
  expect_equal(removeBaseline(s, method="median", halfWindowSize=2),
               createMassSpectrum(mass=1:20, intensity=c(rep(0, 8), -1:-2, 2:1,
                                                         rep(0, 8))))
})

test_that("removeBaseline works with SNIP", {
  expect_equal(removeBaseline(s, iterations=2),
               createMassSpectrum(mass=1:20, intensity=c(rep(0, 10), 7.5, 5, 2.5,
                                                         rep(0, 7))))
  expect_equal(removeBaseline(s, method="SNIP", iterations=2),
               createMassSpectrum(mass=1:20, intensity=c(rep(0, 10), 7.5, 5, 2.5,
                                                         rep(0, 7))))
})

test_that("removeBaseline works with list of MassSpectrum objects", {
  expect_error(removeBaseline(list(x=1, y=1)),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(removeBaseline(list(s, createMassPeaks(1, 1)),
               "no list of MALDIquant::MassSpectrum objects"))
  r <- createMassSpectrum(mass=1:20,
                          intensity=c(rep(0, 10), 7.5, 5, 2.5, rep(0, 7)))
  expect_equal(removeBaseline(list(s, s), iterations=2), list(r, r))
})

context("findEmptyMassObjects")

s <- createMassSpectrum(mass=1:10, intensity=1:10)
e <- createMassSpectrum(double(), double())

test_that("findEmptyMassObjects throws errors", {
  expect_error(findEmptyMassObjects(e),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(findEmptyMassObjects(list()),
               "no list of MALDIquant::AbstractMassObject objects")
})

test_that("findEmptyMassObjects", {
  expect_identical(findEmptyMassObjects(list(e, e)), 1L:2L)
  expect_identical(findEmptyMassObjects(list(s, s)), integer())
  expect_identical(findEmptyMassObjects(list(s, e)), 2L)
  expect_identical(findEmptyMassObjects(list(e, s)), 1L)
})

context("detectPeaks")

s <- createMassSpectrum(mass=1:5, intensity=c(1, 2, 1, 2, 1))
p <- createMassPeaks(c(2, 4), c(2, 2), c(Inf, Inf))

test_that("detectPeaks", {
  expect_equal(detectPeaks(s, halfWindowSize=1), p)
})

test_that("detectPeaks shows warnings", {
  expect_warning(detectPeaks(createMassSpectrum(mass=double(),
                                                intensity=double()), "empty"))
})

test_that("detectPeaks works with list of MassSpectrum objects", {
  expect_error(detectPeaks(list(x=1, y=1)),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(detectPeaks(list(s, createMassPeaks(1, 1)),
               "no list of MALDIquant::MassSpectrum objects"))
  expect_equal(detectPeaks(list(s, s), halfWindowSize=1), list(p, p))
})

context(".findLocalMaximaLogical")

s <- createMassSpectrum(mass=1:5, intensity=c(1, 2, 1, 2, 1))

test_that(".findLocalMaximaLogical throws errors", {
  expect_error(MALDIquant:::.findLocalMaximaLogical(s, halfWindowSize=0),
               "too small")
  expect_error(MALDIquant:::.findLocalMaximaLogical(s, halfWindowSize=3),
               "too large")
})

test_that(".findLocalMaxima(Logical) shows warnings", {
  expect_warning(MALDIquant:::.findLocalMaximaLogical(
                   createMassSpectrum(mass=double(), intensity=double()),
                 "empty"))
  expect_warning(MALDIquant:::.findLocalMaxima(
                   createMassSpectrum(mass=double(), intensity=double()),
                 "empty"))
})

test_that(".findLocalMaximaLogical works with different window sizes", {
  expect_identical(suppressWarnings(MALDIquant:::.findLocalMaximaLogical(
                      createMassSpectrum(mass=double(), intensity=double()),
                      halfWindowSize=1)), logical())
  expect_identical(MALDIquant:::.findLocalMaximaLogical(s, halfWindowSize=1),
                   c(FALSE, TRUE, FALSE, TRUE, FALSE))
  expect_identical(MALDIquant:::.findLocalMaximaLogical(s, halfWindowSize=2),
                   c(FALSE, TRUE, FALSE, FALSE, FALSE))
})

test_that(".findLocalMaxima returns matrix", {
  m <- matrix(ncol=2, dimnames=list(list(), list("mass", "intensity")))
  expect_equal(suppressWarnings(MALDIquant:::.findLocalMaxima(
                  createMassSpectrum(mass=double(), intensity=double()),
                  halfWindowSize=1)), m)
  m <- matrix(c(2, 2, 4, 2), ncol=2, byrow=TRUE,
              dimnames=list(list(), list("mass", "intensity")))
  expect_identical(MALDIquant:::.findLocalMaxima(s, halfWindowSize=1), m)
})

context("averageMassSpectra")

s <- c(createMassSpectrum(mass=1:3, intensity=1:3, metaData=list(a=1, b=2)),
       createMassSpectrum(mass=1:3, intensity=4:6, metaData=list(a=1, b=3)))

meanS <- createMassSpectrum(mass=1:3, intensity=(2:4)+0.5,
                            metaData=list(a=1, b=2:3))
sumS <- createMassSpectrum(mass=1:3, intensity=c(5, 7, 9),
                           metaData=list(a=1, b=2:3))
p <- c(createMassPeaks(mass=1:3, intensity=1:3, snr=1:3,
                       metaData=list(a=1, b=2)),
       createMassPeaks(mass=1:3, intensity=4:6, snr=4:6,
                       metaData=list(a=1, b=3)))

test_that("averageMassSpectra", {
  expect_equal(unname(averageMassSpectra(list(s[[1]], s[[1]], s[[2]], s[[2]]),
                                         labels=c(1:2, 1:2))),
               list(meanS, meanS))
  expect_equal(unname(averageMassSpectra(list(s[[1]], s[[1]], s[[2]], s[[2]]),
                                         labels=c(1:2, 1:2), method="median")),
               list(meanS, meanS))
  expect_equal(unname(averageMassSpectra(list(s[[1]], s[[1]], s[[2]], s[[2]]),
                                         labels=c(1, 1, 2, 2), method="median")),
               s)
  expect_equal(unname(averageMassSpectra(list(s[[1]], s[[1]], s[[2]], s[[2]]),
                                         labels=c(1:2, 1:2), method="sum")),
               list(sumS, sumS))
})

test_that("averageMassSpectra throws errors", {
  expect_error(averageMassSpectra(1:3),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(averageMassSpectra(list()),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(averageMassSpectra(p),
               "no list of MALDIquant::MassSpectrum objects")
})

test_that("averageMassSpectra works with empty spectra", {
  e <- createMassSpectrum(mass=double(), intensity=double(),
                          metaData=list(a=1, b=2))
  expect_equal(unname(averageMassSpectra(list(s[[1]], e))), s[[1]])
  expect_equal(unname(averageMassSpectra(list(e, e))), e)
})

test_that("averageMassSpectra works with NA in metaData", {
  s1 <- createMassSpectrum(
                mass=1:3, intensity=1:3, metaData=list(foo=1:3, bar=c(1, NA, 2))
  )
  expect_equal(
    unname(averageMassSpectra(list(s1, s1), labels = 1:2)), list(s1, s1)
  )
  expect_equal(
    unname(averageMassSpectra(list(s1, s1), labels = c(1, 1))), list(s1)
  )
  s2 <- createMassSpectrum(
                mass=1:3, intensity=1:3, metaData=list(foo=1:3, bar=c(NA, 1, 2))
  )
  expect_equal(
        unname(averageMassSpectra(list(s1, s2), labels = c(1, 1))),
        list(createMassSpectrum(
            mass=1:3, intensity=1:3,
            metaData=list(foo=1:3, bar=c(1, NA, 2, NA, 1, 2))
        ))
  )
})

test_that(".averageMassSpectra", {
  expect_equal(MALDIquant:::.averageMassSpectra(s), meanS)
  expect_equal(MALDIquant:::.averageMassSpectra(s, fun=colSums), sumS)
})

context("transformIntensity")

s <- createMassSpectrum(mass=1:10, intensity=(1:10)^2)

test_that("transformIntensity shows warnings", {
  expect_warning(transformIntensity(
                   createMassSpectrum(mass=double(), intensity=double()),
                   method="sqrt"), "empty")
})

test_that("transformIntensity", {
  expect_equal(intensity(transformIntensity(s, method="sqrt")), 1:10)
  expect_equal(intensity(transformIntensity(s, method="log")), log((1:10)^2))
  expect_equal(intensity(transformIntensity(s, method="log2")), log2((1:10)^2))
  expect_equal(intensity(transformIntensity(s, method="log10")),
               log10((1:10)^2))
})

test_that("transformIntensity works with list of MassSpectrum objects", {
  expect_error(transformIntensity(list(x=1, y=1)),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(transformIntensity(list(s, createMassPeaks(1, 1)),
               "no list of MALDIquant::AbstractMassObject objects"))
  r <- createMassSpectrum(mass=1:10, intensity=1:10)
  expect_equal(transformIntensity(list(s, s), method="sqrt"), list(r, r))
})

test_that(".transformIntensity throws errors", {
  expect_error(.transformIntensity(s), "is missing")
})

test_that(".transformIntensity shows warnings", {
  expect_warning(.transformIntensity(s, fun=function(x)return(-x)),
                 "Negative intensity values are replaced by zeros.")
})

test_that(".transformIntensity", {
  expect_equal(intensity(.transformIntensity(s, sqrt)), 1:10)
  expect_equal(length(.transformIntensity(s, MALDIquant:::.movingAverage,
                                          halfWindowSize=2)), 10)
  expect_equal(length(.transformIntensity(s,
                        function(x)as.double(filter(x, rep(1, 5)/5, sides=2)))),
               6)
  expect_equal(intensity(suppressWarnings(
                 .transformIntensity(s, fun=function(x)return(-x)))),
               rep(0, 10))
})

test_that(".transformIntensity works with list of AbstractMassObject objects", {
  expect_error(.transformIntensity(list(x=1, y=1)),
               "no list of MALDIquant::AbstractMassObject objects")
  expect_error(.transformIntensity(list(s, createMassPeaks(1, 1)),
               "no list of MALDIquant::AbstractMassObject objects"))
  r <- createMassSpectrum(mass=1:10, intensity=1:10)
  expect_equal(.transformIntensity(list(s, s), fun=sqrt), list(r, r))
})

context("trim")

s <- createMassSpectrum(mass=1:10, intensity=11:20)

test_that("trim throws errors", {
  expect_error(trim(s, range=1:10), "has to be a vector of length 2")
  expect_error(trim(s, range=1), "has to be a vector of length 2")
  expect_error(trim(c(s, createMassSpectrum(mass=21:30, intensity=1:10))),
               "No overlap")
})

test_that("trim throws warnings", {
  expect_warning(trim(s, range=c(20, 30)), "No data points left")
})

test_that("trim", {
  expect_equal(trim(s, c(2, 9)),
         createMassSpectrum(mass=2:9, intensity=12:19))
})

test_that("trim,list throws errors", {
  expect_error(trim(list(x=1, y=2)),
               "no list of MALDIquant::AbstractMassObject objects")
})

test_that("trim works with list of AbstractMassObject objects", {
  r <- createMassSpectrum(mass=2:9, intensity=12:19)
  expect_equal(trim(list(s, s), c(2, 9)), list(r, r))
  expect_equal(trim(list(s, r)), list(r, r))
})

context("constructors")

meta <- list(name="test", file="test_constructor-functions.R")

test_that("a MassSpectrum object is constructed", {
  expect_error(createMassSpectrum(mass=1:5, intensity=1:10),
               "have to be equal")
  expect_error(createMassSpectrum(mass=1:10, intensity=1:5),
               "have to be equal")
  expect_error(createMassSpectrum(mass=1:10, intensity=1:10, metaData=1:10),
               "invalid object for slot \"metaData\"")
  expect_error(createMassSpectrum(mass=LETTERS[1:5], intensity=1:5),
               "invalid object for slot \"mass\"")
  expect_error(createMassSpectrum(mass=1:5, intensity=LETTERS[1:5]),
               "invalid object for slot \"intensity\"")

  expect_warning(createMassSpectrum(mass=-(1:5), intensity=1:5),
                 "Negative mass values found.")
  expect_warning(createMassSpectrum(mass=1:5, intensity=-(1:5)),
                 "Negative intensity values found.")

  expect_equal(createMassSpectrum(mass=1:10, intensity=1:10),
               new("MassSpectrum", mass=1:10, intensity=1:10, metaData=list()))
  expect_equal(createMassSpectrum(1:10, 1:10, meta),
               new("MassSpectrum", mass=1:10, intensity=1:10, metaData=meta))
  expect_equal(suppressWarnings(createMassSpectrum(10:1, 10:1, meta)),
               new("MassSpectrum", mass=1:10, intensity=1:10, metaData=meta))
})

test_that("a MassPeaks object is constructed", {
  expect_error(createMassPeaks(mass=1:5, intensity=1:10),
               "have to be equal")
  expect_error(createMassPeaks(mass=1:10, intensity=1:5),
               "have to be equal")
  expect_error(createMassPeaks(mass=1:5, intensity=1:5, snr=1:2),
               "have to be equal")
  expect_error(createMassPeaks(mass=1:10, intensity=1:10, metaData=1:10),
               "invalid object for slot \"metaData\"")

  expect_equal(createMassPeaks(mass=1:10, intensity=1:10),
               new("MassPeaks", mass=1:10, intensity=1:10,
                   snr=as.double(rep(NA, 10)), metaData=list()))
  expect_equal(createMassPeaks(1:10, 1:10, snr=1:10, meta),
               new("MassPeaks", mass=1:10, intensity=1:10, snr=1:10,
                   metaData=meta))
})

context("isEmpty")

s <- c(createMassSpectrum(mass=1:10, intensity=11:20),
       createMassSpectrum(mass=1:2, intensity=rep.int(.Machine$integer.max, 2)))
e <- c(createMassSpectrum(mass=double(), intensity=double(),
                          metaData=list(file="fid")),
       createMassSpectrum(mass=1:10, intensity=rep(0, 10)),
       createMassSpectrum(mass=1:10, intensity=as.double(rep(NA, 10))))

test_that("isEmpty", {
  for (i in seq(along=e)) {
    expect_true(isEmpty(e[[i]]))
  }
  for (i in seq(along=s)) {
    expect_false(isEmpty(s[[i]]))
  }
})

test_that("isEmptyWarning", {
  expect_warning(MALDIquant:::.isEmptyWarning(e[[1]]),
                 "MassSpectrum object \\(file: fid\\) is empty")
  expect_warning(MALDIquant:::.isEmptyWarning(e[[2]]),
                 "MassSpectrum object is empty")
  expect_true(suppressWarnings(MALDIquant:::.isEmptyWarning(e[[1]])))
  expect_false(MALDIquant:::.isEmptyWarning(s[[1]]))
})

context("length")

test_that("length", {
  expect_identical(length(createMassSpectrum(mass=1:10, intensity=1:10)), 10L)
  expect_identical(length(createMassSpectrum(mass=double(),
                                             intensity=double())), 0L)
})

## .mapply
##  wrapper for mapply with different defaults
##
## params:
##  see ?mapply
##
## returns:
##  see ?mapply
##
.mapply <- function(FUN, ..., MoreArgs=NULL, SIMPLIFY=FALSE, USE.NAMES=FALSE,
                    mc.cores=1L) {
  parallel::mcmapply(FUN=FUN, ..., MoreArgs=MoreArgs, SIMPLIFY=SIMPLIFY,
                     USE.NAMES=USE.NAMES, mc.cores=mc.cores)
}

context(".mapply")

test_that(".mapply", {
  a <- setNames(1L:10L, LETTERS[1L:10L])
  b <- 1L:2L
  f <- function(x, y)x+y

  expect_identical(MALDIquant:::.mapply(f, a, b),
                   mapply(f, a, b, USE.NAMES=FALSE, SIMPLIFY=FALSE))
})

## AbstractMassObject
setMethod(f="coordinates",
          signature=signature(object="AbstractMassObject"),
          definition=function(object, ...) {

  object@metaData$imaging$pos
})

## AbstractMassObject
setReplaceMethod(f="coordinates",
                 signature=signature(object="AbstractMassObject",
                                     value="numeric"),
                 definition=function(object, value) {

  if (!length(value) %in% c(2L, 3L)) {
    stop("2 or 3 coordinates are needed!")
  }
  object@metaData$imaging$pos <- value
  object
})

setReplaceMethod(f="coordinates",
                 signature=signature(object="AbstractMassObject",
                                     value="matrix"),
                 definition=function(object, value) {
  if (!ncol(value) %in% c(2L, 3L)) {
    stop("2 or 3 coordinates are needed!")
  }
  if (nrow(value) != 1L) {
    warning("all rows but the first are ignored!")
  }
  object@metaData$imaging$pos <- value[1L, ]
  object
})

## list
setMethod(f="coordinates",
          signature=signature(object="list"),
          definition=function(object, adjust=FALSE) {

  ## test arguments
  .stopIfNotIsMassObjectList(object)

  m <- do.call(rbind, lapply(object, coordinates))

  if (!is.null(m)) {
    colnames(m) <- c("x", "y", "z")[seq_len(ncol(m))]

    if (adjust) {
        m <- apply(m, MARGIN=2L, function(x)x - min(x) + 1L)
    }
  }
  m
})

## list
setReplaceMethod(f="coordinates",
                 signature=signature(object="list",
                                     value="matrix"),
                 definition=function(object, value) {
  ## test arguments
  .stopIfNotIsMassObjectList(object)

  .mapply("coordinates<-", object, split(value, row(value)))
})

## AbstractMassObject
setMethod(f="metaData",
          signature=signature(object="AbstractMassObject"),
          definition=function(object) {

  object@metaData
})

## AbstractMassObject
setReplaceMethod(f="metaData",
                  signature=signature(object="AbstractMassObject"),
                  definition=function(object, value) {

  object@metaData <- value
  object
})

## dilation
##  calculates moving max (dilation filter)
##
##  M. van Herk. "A Fast Algorithm for Local Minimum and Maximum Filters on
##  Rectangular and Octagonal Kernels."
##  Pattern Recognition Letters 13.7 (1992): 517-521.
##
##  J. Y. Gil and M. Werman. "Computing 2-Dimensional Min, Median and Max
##  Filters." IEEE Transactions (1996): 504-507.
##
## params:
##  x: vector of x values
##  halfWindowSize: size of local window
##
## returns:
##  moving max
##
.dilation <- function(x, halfWindowSize) {
  .Call(C_dilation, x, halfWindowSize)
}

## erosion
##  calculates moving min (erosion filter)
##
##  M. van Herk. "A Fast Algorithm for Local Minimum and Maximum Filters on
##  Rectangular and Octagonal Kernels."
##  Pattern Recognition Letters 13.7 (1992): 517-521.
##
##  J. Y. Gil and M. Werman. "Computing 2-Dimensional Min, Median and Max
##  Filters." IEEE Transactions (1996): 504-507.
##
## params:
##  x: vector of x values
##  halfWindowSize: size of local window
##
## returns:
##  moving min
##
.erosion <- function(x, halfWindowSize) {
  .Call(C_erosion, x, halfWindowSize)
}

## MassSpectrum
setMethod(f="isRegular",
          signature=signature(object="MassSpectrum"),
          definition=function(object, threshold=1e-3) {

  s <- .irregularScore(object@mass) <= threshold
  !is.na(s) & s
})

## MassSpectrum
setMethod(f="calibrateIntensity",
          signature=signature(object="MassSpectrum"),
          definition=function(object,
                              method=c("TIC", "PQN", "median"),
                              range, ...) {

  method <- match.arg(method)

  switch(method,
    "TIC" = ,
    "median" = {
      .transformIntensity(object, fun=.calibrateIntensitySimple,
                          offset=0L,
                          scaling=.scalingFactor(object, method=method,
                                                 range=range))
    },
    "PQN" = {
      stop(dQuote("PQN"),
           " is not supported for a single MassSpectrum object.")
    })
})

## list
setMethod(f="calibrateIntensity",
          signature=signature(object="list"),
          definition=function(object,
                              method=c("TIC", "PQN", "median"), range, ...) {

  ## test arguments
  .stopIfNotIsMassSpectrumList(object)

  method <- match.arg(method)

  switch(method,
    "TIC" = ,
    "median" = {
      .lapply(object, calibrateIntensity, method=method, range=range, ...)
    },
    "PQN" = {
      .calibrateProbabilisticQuotientNormalization(object, range=range)
    }
  )
})

## .doByLabels
##  run a specific function labelwise
##
## params:
##  l: list of AbstractMassObject objects
##  labels: factor, labels for samples
##  fun: function
##
## returns:
##  list of modified AbstractMassObject objects
##
.doByLabels <- function(l, labels, FUN, ..., mc.cores=1L) {

  ## test parameters
  .stopIfNotIsMassObjectList(l)

  FUN <- match.fun(FUN)

  if (!missing(labels)) {
    ## drop unused levels and turn argument into factor
    if (is.factor(labels)) {
      labels <- droplevels(labels)
    } else {
      ## preserve order in labels
      labels <- factor(labels, levels=unique(labels))
    }

    if (length(labels) != length(l)) {
      stop("For each item in ", sQuote("l"), " there must be a label in ",
           sQuote("labels"), "!")
    }

    ## replace tapply by split to preserve order
    tmp <- .lapply(split(unlist(l), labels), FUN=FUN, ..., mc.cores=mc.cores)

    k <- unlist(tmp)

    if (length(k) != length(tmp)) {
      k <- unsplit(tmp, labels)
    }
  } else {
    k <- FUN(l, ...)
  }

  k
}

## determineWarpingFunctions
##  calculates warping functions for each MassPeaks object
##
## params:
##  l: list, list of MassPeaks objects
##  reference: MassPeaks, a reference MassPeaks object to which all other
##             MassPeaks objects should be aligned
##  tolerance: double, maximal deviation of a peak position to be
##             considered as same peak
##  method: choose type of base warping function
##  allowNoMatches: logical, don't throw an error if a single MassPeaks object
##                can't match to the reference.
##  plot: logical, plots warping function
##
## returns:
##  a list of warping functions
##
determineWarpingFunctions <- function(l, reference, tolerance=0.002,
                                      method=c("lowess", "linear", "quadratic",
                                               "cubic"),
                                      allowNoMatches=FALSE,
                                      plot=FALSE, plotInteractive=FALSE, ...) {

  ## test arguments
  if (!isMassPeaksList(l) && !isMassPeaks(l)) {
    stop(sQuote("l"), " is no list of MALDIquant::MassPeaks objects!")
  }

  method <- match.arg(method)

  if (allowNoMatches) {
      nomatchReporter <- warning
  } else {
      nomatchReporter <- stop
  }

  warpingFunction <- switch(method,
    "lowess" = {
      .warpingFunctionLowess
    },
    "linear" = {
      .warpingFunctionLinear
    },
    "quadratic" = {
      .warpingFunctionQuadratic
    },
    "cubic" = {
      .warpingFunctionCubic
    }
  )

  minPeaks <- switch(method,
    "lowess" = 2L,
    "linear" = 2L,
    "quadratic" = 3L,
    "cubic" = 4L
  )

  optArgs <- list(...)

  ## find reference peaks
  if (missing(reference)) {
    arguments <- list(l=l, tolerance=tolerance)
    argumentNames <- c("method", "minFrequency")

    arguments <- modifyList(arguments, optArgs[argumentNames])
    optArgs[argumentNames] <- NULL

    reference <- do.call(referencePeaks, arguments)
  }

  if (isEmpty(reference)) {
    stop("Reference MassPeaks object contains no peaks!")
  }

  if (length(reference) < 10L) {
    warning("Reference MassPeaks object contains very few peaks (n == ",
            length(reference), "). The warping could be instable. ",
            "Consider to reduce ", sQuote("minFrequency"),
            " or/and to increase ", sQuote("tolerance"), ".")
  }

  ## fetch plot.default arguments (debug plot)
  if (plot) {
    plotNames <- c("xlim", "ylim", "xlab", "ylab", "type", "lwd", "col",
                   "col.sub", "cex.main", "cex.sub", "main", "sub")

    givenPlotArgs <- optArgs[plotNames]
    optArgs[plotNames] <- NULL
  }

  ## reference has to become sample no 1
  tmpPeakList <- c(reference, l)

  ## same procedure as in binPeaks
  ## fetch all mass
  mass <- unname(.unlist(lapply(tmpPeakList, function(x)x@mass)))

  ## fetch all intensities
  intensities <- .unlist(lapply(tmpPeakList, function(x)x@intensity))

  ## store original mass sample number/id
  samples <- rep.int(seq_along(tmpPeakList), lengths(tmpPeakList))

  ## sort values by mass
  s <- sort.int(mass, index.return=TRUE)

  mass <- s$x
  intensities <- intensities[s$ix]
  samples <- samples[s$ix]

  ## run peak binning and use relaxed grouper which choose the highest test
  ## sample peaks
  binnedMass <- .binPeaks(mass=mass, intensities=intensities, samples=samples,
                          tolerance=tolerance,
                          grouper=.grouperRelaxedHighestAtReference, nomatch=0L)

  ## group mass/intensities by sample ids
  lIdx <- split(seq_along(binnedMass), samples)

  ## calculate differences
  binnedMass[binnedMass == 0L] <- NA_real_
  d <- binnedMass - mass

  ## each function which determines a warping function uses these 3 arguments
  arguments <- list(x=NULL, d=NULL)
  if (length(optArgs)) {
    arguments <- c(arguments, optArgs)
  }

  ## determine warping functions
  warpingFunctions <- lapply(lIdx[-1L], function(i) {
    ## fetch changed mass == aligned peaks
    notNA <- !is.na(binnedMass[i])

    arguments$x <- mass[i][notNA] ## original mass
    arguments$d <- d[i][notNA]  ## difference to reference

    if (!length(arguments$x)) {
      nomatchReporter("Could not match any peak in spectrum ",
                      samples[i[1L]] - 1L, " to a reference peak.")
      NA
    } else if (length(arguments$x) < minPeaks) {
      nomatchReporter("Could not match enough peaks in spectrum ",
                      samples[i[1L]] - 1L, " to the reference peaks.\n",
                      minPeaks, " matches required, just ",
                      length(arguments$x), " found.")
      NA
    } else {
      do.call(warpingFunction, arguments)
    }
  })

  ## clean misleading names (names == idx+1 because reference is idx == 1)
  names(warpingFunctions) <- NULL

  ## number of matched peaks
  nmatch <-  vapply(split(d, samples)[-1L], function(dd) {
      sum(!is.na(dd), na.rm=TRUE)
  }, NA_real_, USE.NAMES=FALSE)
  names(nmatch) <- names(l)
  attr(warpingFunctions, "nmatch") <- nmatch

  ## debug plot
  if (plot) {
    ## non interactive device (pdf, png, ...) available?
    isNonInteractivePlot <- dev.cur() != 1L && !dev.interactive()

    if (!isNonInteractivePlot && !plotInteractive) {
      warning(sQuote("plot"), " is ", sQuote("TRUE"),
              " but no non-interactive device is available. ",
              "Using pdf() to create a default one.")
      pdf(paper="a4r", width=12)
    } else if (dev.cur() == 1L && plotInteractive) {
      warning(sQuote("plot"), " is ", sQuote("TRUE"),
              " but no interactive device is available. ",
              "Using dev.new() to create a default one.")
      dev.new()
    }

    ## set default plot arguments
    plotArgsDefaults <- list(xlim=range(mass),
                             ylim=range(d, na.rm=TRUE),
                             xlab="mass",
                             ylab="difference",
                             type="p",
                             lwd=1L,
                             col=1L,
                             cex.main=0.8,
                             cex.sub=0.75,
                             col.sub="#808080")

    plotArgs <- modifyList(plotArgsDefaults, givenPlotArgs)

    nReference <- length(reference)
    x <- plotArgs$xlim[1L]:plotArgs$xlim[2L]

    ## workaround to avoid error:
    ## Error in l[[i]] : this S4 class is not subsettable
    if (!is.list(l)) {
      l <- list(l)
    }

    for (i in seq_along(l)) {
      ## fetch changed mass == aligned peaks
      notNA <- !is.na(binnedMass[lIdx[[i + 1L]]])

      if (is.null(givenPlotArgs$main)) {
        plotArgs$main <- paste0("sample ", i, " vs reference\n",
                                "(matched peaks: ", sum(notNA), "/",
                                nReference, ")")
      }

      if (is.null(givenPlotArgs$sub)) {
        plotArgs$sub <- l[[i]]@metaData$file
      }

      ## plot reference vs sample
      plotArgs$x <- l[[i]]@mass[notNA]
      plotArgs$y <- d[lIdx[[i + 1L]]][notNA]
      do.call(plot.default, plotArgs)

      ## draw warping function
      lines(x, warpingFunctions[[i]](x), lwd=plotArgs$lwd, col=plotArgs$col)
    }

    if (!isNonInteractivePlot && !plotInteractive) {
      dev.off()
    }
  }

  warpingFunctions
}

context("alignSpectra")

test_that("alignSpectra throws errors", {
  expect_error(alignSpectra(list()),
               ".*spectra.* is no list of MALDIquant::MassSpectrum objects")
})

test_that("alignSpectra", {
  data(fiedler2009subset)
  spectra <- fiedler2009subset[1:8]

  peaks <- detectPeaks(spectra, halfWindowSize=20, method="MAD", SNR=2)
  wf <- determineWarpingFunctions(peaks, tolerance=0.002, method="lowess")
  expect_equal(alignSpectra(spectra), warpMassSpectra(spectra, wf))
})

## MassSpectrum
setMethod(f=".findLocalMaxima",
          signature=signature(object="MassSpectrum"),
          definition=function(object, halfWindowSize=20L) {

  if (.isEmptyWarning(object)) {
    return(matrix(ncol=2L, dimnames=list(list(), list("mass", "intensity"))))
  }

  localMaxima <- .findLocalMaximaLogical(object, halfWindowSize=halfWindowSize)

  cbind(mass=object@mass, intensity=object@intensity)[localMaxima,]
})

setMethod(f=".findLocalMaximaLogical",
          signature=signature(object="MassSpectrum"),
          definition=function(object, halfWindowSize=20L) {

  if (.isEmptyWarning(object)) {
    return(logical())
  }

  .stopIfNotIsValidHalfWindowSize(halfWindowSize=halfWindowSize,
                                  n=length(object))

  .localMaxima(object@intensity, halfWindowSize=halfWindowSize)
})

## filterPeaks
##  filter peaks which are not frequently represented in different samples
##
## params:
##  l: list of MassPeaks objects
##  minFrequency: double, minimal frequency of a peak to be not removed
##  minNumber: double, minimal (absolute) number of peaks to be not removed
##  labels: factor, labelwise filtering
##  mergeWhitelists: logical, apply whitelists local (FALSE) or global (TRUE)
##
## returns:
##  a list of adjusted MassPeaks objects
##
filterPeaks <- function(l, minFrequency, minNumber, labels,
                        mergeWhitelists=FALSE) {

  ## test arguments
  .stopIfNotIsMassPeaksList(l)

  ## labels
  if (missing(labels)) {
    labels <- rep.int(0L, length(l))
  }

  ## drop unused levels and turn argument into factor
  if (is.factor(labels)) {
    labels <- droplevels(labels)
  } else {
    ## preserve order in labels
    labels <- factor(labels, levels=unique(labels))
  }

  if (missing(minFrequency)) {
    minFrequency <- NA
  }

  if (missing(minNumber)) {
    minNumber <- NA
  }

  ll <- levels(labels)
  nl <- length(ll)

  if (length(labels) != length(l)) {
    stop("For each item in ", sQuote("l"), " there must be a label in ",
         sQuote("labels"), "!")
  }

  ## recycle arguments if needed
  minFrequency <- rep_len(minFrequency, nl)
  minNumber <- rep_len(minNumber, nl)
  mergeWhitelists <- isTRUE(mergeWhitelists)

  ## use peaks occurrence list
  o <- .as.occurrence.list.MassObjectList(l)

 # group indices by labels
  idx <- lapply(ll, function(x)which(labels == x))

  ## collect whitelists
  w <- matrix(FALSE, nrow = nl, ncol = length(o$mass))

  for (i in seq_along(idx)) {
    wl <- .whitelist(
        o, idx[[i]], minFrequency = minFrequency[i], minNumber = minNumber[i]
    )

    if (sum(wl)) {
      if (mergeWhitelists) {
        ## R uses columnwise recycling
        w <- t(t(w) | wl)
      } else {
        ## R uses columnwise recycling
        w[i, ] <- t(t(w[i, , drop=FALSE]) | wl)
      }
    } else {
      warning("Empty peak whitelist for level ", sQuote(ll[i]), ".")
    }

  }

  ## turn matrix back into MassPeaks objects
  for (i in seq_along(idx)) {
    for (j in idx[[i]]) {
      wmask <- w[i, o$i[o$sample == j]]
      l[[j]]@mass <- l[[j]]@mass[wmask]
      l[[j]]@intensity <- l[[j]]@intensity[wmask]
      l[[j]]@snr <- l[[j]]@snr[wmask]
    }
  }

  l
}

## .whitelist
##  helper function to create whitelists for filtering
##
## params:
##  l: list of occurrences, generated by .as.occurrence.list.MassObjectList
##  rows: index of rows which should filtered
##  minFrequency: double, minimal frequency of a peak to be not removed
##  minNumber: double, minimal (absolute) number of peaks to be not removed
##
## returns:
##  a logical vector representing the whitelist
##
.whitelist <- function(l, rows, minFrequency, minNumber) {
  ## test arguments
  if (is.na(minFrequency) && is.na(minNumber)) {
    stop(sQuote(minFrequency), " or ", sQuote(minNumber),
         " has to be a meaningful number!")
  }

  if (!is.na(minFrequency) && minFrequency < 0L) {
    minFrequency <- 0L
    warning(sQuote("minFrequency"),
            " < 0 does not make sense! Using 0 instead.")
  }

  if (!is.na(minNumber) && minNumber < 0L) {
    minNumber <- 0L
    warning(sQuote("minNumber"), " < 0 does not make sense! Using 0 instead.")
  }

  if (!is.na(minFrequency) && !is.na(minNumber)) {
    warning(sQuote("minFrequency"), " and ", sQuote("minNumber"),
            " arguments are given. Choosing the higher one.")
  }

  ## calculate minimal number of peaks
  minPeakNumber <-
      max(minFrequency * length(rows), minNumber, na.rm=TRUE)

  tabulate(l$i[l$sample %in% rows], nbins = length(l$mass)) >= minPeakNumber
}

## .memoryUsageStr.object_size
##  pretty string of memory usage
##
## params:
##  x: object_size
##
## returns:
##  character
##
.memoryUsageStr <- function(x) {
  os <- object.size(x)
  iec <- c("B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB")
  l <- trunc(log(os) / log(1024L))
  i <- pmin(l + 1L, 9L)

  paste(round(os / (1024L^l), digits=3L), iec[i])
}

## .as.matrix.MassObjectList
##  internal function to convert a list of AbstractMassObject objects into a
##  matrix
##
## params:
##  l: list of AbstractMassObject objects
##
## returns:
##  a matrix
.as.matrix.MassObjectList <- function(l) {
    .stopIfNotIsMassObjectList(l)

    intensity <- .unlist(lapply(l, function(x)x@intensity))
    o <- .as.occurrence.list.MassObjectList(l)

    m <- matrix(
        NA_real_, nrow=length(l), ncol=length(o$mass),
        dimnames=list(NULL, o$mass)
    )
    m[cbind(o$sample, o$i)] <- intensity
    attr(m, "mass") <- o$mass
    m
}

## .as.binary.matrix
##  internal function to convert a matrix with NA to a binary one
##
## params:
##  m: matrix
##
## returns:
##  a binary matrix
.as.binary.matrix <- function(m) {
    if (!is.matrix(m))
        stop("'x' has to be a matrix!")
    m[] <- !is.na(m)
    mode(m) <- "integer"
    m
}

.colMedians <- function(x, na.rm=FALSE) {
  stopifnot(is.matrix(x), is.logical(na.rm))
  .Call(C_colMedians, x, na.rm)
}

#' .colMaxs
#'
#' Similar to apply(x, 2, max).
#'
#' @param x matrix/data.frame
#' @return double
#' @author Sebastian Gibb <mail@@sebastiangibb.de>
#' @noRd
.colMaxs <- function(x) {
  x[max.col(t(x), ties.method="first") + 0L:(ncol(x) - 1L) * nrow(x)]
}

#' .colCors
#'
#' Calculate the correlation for two matrices columnwise.
#'
#' @param x matrix/data.frame
#' @return double
#' @author Sebastian Gibb <mail@@sebastiangibb.de>
#' @noRd
.colCors <- function(x, y, na.rm=FALSE) {
  stopifnot(is.matrix(x) && is.matrix(y))
  stopifnot(all(dim(x) == dim(y)))

  if (na.rm) {
    isNA <- is.na(x) | is.na(y)
    x[isNA] <- NA_real_
    y[isNA] <- NA_real_
  }

  cmX <- colMeans(x, na.rm=na.rm)
  cmY <- colMeans(y, na.rm=na.rm)

  (colMeans(x * y, na.rm=na.rm) - (cmX * cmY)) /
    (sqrt(colMeans(x * x, na.rm=na.rm) - cmX * cmX) *
     sqrt(colMeans(y * y, na.rm=na.rm) - cmY * cmY))
}

context(".colMedians")

test_that(".colMedians throws errors", {
  expect_error(MALDIquant:::.colMedians(list()))
  expect_error(MALDIquant:::.colMedians(matrix(1:10), na.rm="foobar"))
})

test_that(".colMedians", {
  colMedians <- function(x, na.rm=FALSE)apply(x, 2, median, na.rm=na.rm)
  set.seed(1234)
  ## even nrow
  m <- matrix(rnorm(1e5), ncol=1e2)
  expect_equal(MALDIquant:::.colMedians(m), colMedians(m))
  ## odd nrow
  nr <- nrow(m)
  m <- m[1:(nr-1), ]
  expect_equal(MALDIquant:::.colMedians(m), colMedians(m))
  ## NA
  na <- sample(1:length(m), size=1e2)
  m[na] <- NA
  expect_equal(MALDIquant:::.colMedians(m), colMedians(m))
  expect_equal(MALDIquant:::.colMedians(m, TRUE), colMedians(m, TRUE))
})

test_that(".colMaxs", {
  colMaxs <- function(x)apply(x, 2, max)
  set.seed(1234)
  ## even nrow
  m <- matrix(rnorm(1e5), ncol=1e2)
  expect_equal(MALDIquant:::.colMaxs(m), colMaxs(m))
})

test_that(".colCors", {
  colCors <- function(x, y, use="everything") {
    z <- double(ncol(x))
    for (i in 1:ncol(x)) {
      z[i] <- stats::cor(x[,i], y[,i], use=use)
    }
    z
  }
  set.seed(1234)
  m <- matrix(rnorm(1e5), ncol=1e2)
  n <- matrix(rnorm(1e5), ncol=1e2)
  mna <- m
  nna <- n
  mna[sample(1e5, 1e3)] <- NA
  nna[sample(1e5, 1e3)] <- NA
  expect_equal(MALDIquant:::.colCors(m, n), colCors(m, n))
  expect_equal(MALDIquant:::.colCors(mna, nna), colCors(mna, nna))
  expect_equal(MALDIquant:::.colCors(mna, nna, na.rm=TRUE),
               colCors(mna, nna, use="na.or.complete"))
})

## MassSpectrum
setMethod(f="totalIonCurrent",
          signature=signature(object="MassSpectrum"),
          definition=function(object) {

  left <- as.double(head(object@intensity, -1L))
  right <- as.double(tail(object@intensity, -1L))

  as.double(sum((left + right) / 2L * diff(object@mass), na.rm=TRUE))
})

setMethod(f="plotMsiSlice",
          signature=signature(x="list"),
          definition=function(x, center, tolerance,
                              colRamp=colorRamp(c("black", "blue", "green",
                                                   "yellow", "red")),
                              interpolate=FALSE, legend=TRUE, alignLabels=FALSE,
                              combine=FALSE, ...) {
  .stopIfNotIsMassObjectList(x)
  slides <- msiSlices(x, center=center, tolerance=tolerance)
  plotMsiSlice(slides, colRamp=colRamp, interpolate=interpolate, legend=legend,
               alignLabels=alignLabels, combine=combine, ...)
})

setMethod(f="plotMsiSlice",
          signature=signature(x="array"),
          definition=function(x, colRamp=colorRamp(c("black", "blue", "green",
                                                     "yellow", "red")),
                              interpolate=FALSE, legend=TRUE, alignLabels=FALSE,
                              combine=FALSE, plotInteractive=FALSE, ...) {
  n <- dim(x)[3L]

  if (!is.list(colRamp)) {
    colRamp <- rep_len(list(colRamp), n)
  }

  if (n != length(colRamp)) {
    stop(sQuote("dim(x)[3L]"), " (number of centers) has to be the same as ",
         "the length of the list ", sQuote("colRamp"), "!\n",
         "See ", sQuote("?plotMsiSlice"), " for details.")
  }

  if (combine) {
   .plotMsiSlice(x, colRampList=colRamp, interpolate=interpolate,
                  legend=legend, alignLabels=alignLabels, ...)

  } else {
    isNonInteractivePlot <- dev.cur() != 1L && !dev.interactive()
    if (n > 1L && !isNonInteractivePlot && !plotInteractive) {
      warning(sQuote("plotMsiSlice"), " was called for multiple slices on an ",
              "interactive device. Only the first slice is plotted. Use ",
              sQuote("pdf"), " or a similar device to plot all slices at once.",
              " Alternatively use ", dQuote("combine=TRUE"), " to plot ",
              "multiple centers in one plot.\n",
              "See ", sQuote("?plotMsiSlice"), " for details.")
      n <- 1L
    }

    tolerance <- rep_len(attr(x, "tolerance"), n)

    for (i in seq_len(n)) {
      .plotMsiSlice(x[,, i, drop=FALSE],
                    center=attr(x, "center")[i],
                    tolerance=tolerance[i],
                    colRampList=colRamp[i], interpolate=interpolate,
                    legend=legend, ...)
    }
  }
})

setMethod(f="plotMsiSlice",
          signature=signature(x="matrix"),
          definition=function(x, colRamp=colorRamp(c("black", "blue", "green",
                                                     "yellow", "red")),
                                interpolate=FALSE, scale=TRUE, legend=scale,
                                ...) {
  if (!is.list(colRamp)) {
    colRamp <- list(colRamp)
  }

  dim(x) <- c(dim(x), 1L)

  .plotMsiSlice(x, colRampList=colRamp, interpolate=interpolate, scale=scale,
                legend=legend, ...)
})

## AbstractMassObject
setMethod(f="show",
          signature=signature(object="AbstractMassObject"),
          definition=function(object) {

  l <- .prepareShow(object)

  isFilename <- grepl(pattern="^File.*", x=l$groups)

  ## to avoid newlines in other values don't format filenames
  ## (they could be very long)
  l$values[!isFilename] <- format(l$values[!isFilename], justify="left")

  l$groups <- format(l$groups, justify="left")

  cat(paste0(l$groups, ": ", l$values, collapse="\n"), sep="\n")
})

setMethod(f=".prepareShow",
          signature=signature(object="AbstractMassObject"),
          definition=function(object) {

  groups <- c("S4 class type",
              "Number of m/z values",
              "Range of m/z values",
              "Range of intensity values",
              "Memory usage")

  values <- class(object)[1L]

  if (isEmpty(object)) {
    values <- c(values, 0L, NA, NA)
  } else {
    values <- c(values,
                length(object@mass),
                paste0(round(range(object@mass), digits=3L), collapse=" - "),
                paste0(format(min(object@intensity), digits=4L,
                              scientific=TRUE), " - ",
                       format(max(object@intensity), digits=4L,
                              scientific=TRUE)))
  }
  values <- c(values, .memoryUsageStr(object))

  groups <- c(groups, .prepareShowGroupName(object@metaData$name, "Name"))
  values <- c(values, object@metaData$name)

  groups <- c(groups, .prepareShowGroupName(object@metaData$file, "File"))
  values <- c(values, object@metaData$file)

  list(groups=groups, values=values)
})

setMethod(f=".prepareShow",
          signature=signature(object="MassPeaks"),
          definition=function(object) {

  l <- callNextMethod(object)

  groups <- "Range of snr values"

  if (isEmpty(object)) {
    values <- NA
  } else {
    values <- paste0(round(range(object@snr), digits=3), collapse=" - ")
  }

  ## append snr info after intensity
  l$groups <- append(l$groups, groups, after=4L)
  l$values <- append(l$values, values, after=4L)

  list(groups=l$groups, values=l$values)
})

## .deprecated
##  mark a function as deprecated
##  throw an error if current major version or minor is larger than major, or
##  minor+1; if minor == minor+1L wrote warning; otherwise message
##
## params:
##  version: last working version
##  ...: arguments to be passed to stop, warning, message
##
## returns:
##  nothing
##
.deprecated <- function(version, ...) {
  current <- packageVersion("MALDIquant")
  version <- as.package_version(version)

  if (current$major > version$major || current$minor > version$minor + 1L) {
    stop(..., call.=FALSE)
  } else if (current$minor > version$minor) {
    warning(..., call.=FALSE)
  } else {
    message(...)
  }

  invisible()
}

## .deprecatedFunction
##  mark a function as deprecated and show default message
##
## params:
##  version: last working version
##  old: old function
##  new: new function
##
## returns:
##  nothing
##
.deprecatedFunction <- function(version, old, new) {
  if (missing(old)) {
    old <- sys.call(-1L)[[1]]
  }

  msg <- paste0("\"", old , "\" is deprecated.")

  if (!missing(new)) {
    msg <- paste0(msg, "\nUse \"", new, "\" instead. See help(\"", new ,"\").")
  }
  .deprecated(version, msg)
}

## .deprecatedArgument
##  mark an function as deprecated and show default message
##
## params:
##  version: last working version
##  old: old argument
##  new: new argument
##  help: help file
##
## returns:
##  nothing
##
.deprecatedArgument <- function(version, old, new, help) {
  msg <- paste0("Argument \"", old, "\" is deprecated.")

  if (!missing(new)) {
    if (missing(help)) {
      parentCall <- sys.call(-1L)[[1]]
    } else {
      parentCall <- call(help)[[1]]
    }
    msg <- paste0(msg, "\nUse \"", new, "\" instead. See help(\"",
                  deparse(parentCall),"\").")
  }
  .deprecated(version, msg)
}

context(".deprecated")

test_that(".deprecated", {
  expect_error(.deprecated("0.1", "foo", "bar"), "foobar")

  version <- packageVersion("MALDIquant")
  version <- paste(version$major, version$minor - 1L, sep=".")
  expect_warning(.deprecated(version, "foobar"), "foobar")
  expect_message(.deprecated(packageVersion("MALDIquant"), "foobar", "foobar"))
})

test_that(".deprecatedFunction", {
  foobar <- function() {
    .deprecatedFunction("0.1", new="y")
  }
  expect_error(foobar(),
    "\"foobar\" is deprecated..*Use .*y.* instead. See help(.*y*).")
  expect_error(.deprecatedFunction("0.1", old="foobar",
                                   "\"foobar\" is deprecated."))
  expect_error(.deprecatedFunction("0.1", old="foo", new="bar",
    "\"foo\" is deprecated.\nUse \"bar\" instead. See help(\"bar\")."))
})

test_that(".deprecatedArgument", {
  foobar <- function(x, y) {
    .deprecatedArgument("0.1", old="x", new="y")
  }
  expect_error(foobar(1L, 1L),
    "Argument .*x.* is deprecated..*Use .*y.* instead. See help(.*foobar.*).")

  foobar <- function(x, y) {
    .deprecatedArgument("0.1", old="x")
  }
  expect_error(foobar(1L, 1L),
    "Argument \"x\" is deprecated.")

  foobar <- function(x, y) {
    .deprecatedArgument("0.1", old="x", new="y", help="barfoo")
  }
  expect_error(foobar(1L, 1L),
    "Argument .*x.* is deprecated..*Use .*y.* instead. See help(.*barfoo.*).")

})

## AbstractMassObject
setMethod(f="isEmpty",
          signature=signature(x="AbstractMassObject"),
          definition=function(x) {

  !sum(as.double(x@intensity), na.rm=TRUE)
})

setMethod(f=".isEmptyWarning",
          signature=signature(x="AbstractMassObject"),
          definition=function(x) {

  if (isEmpty(x)) {
    msg <- paste0(class(x)[1L], " object")

    if (!is.null(x@metaData$file)) {
      msg <- paste0(msg, " (file: ", x@metaData$file, ")")
    }

    parentCall <- sys.call(-1L)
    warning("In ", deparse(parentCall), " : ", msg, " is empty!",
            call.=FALSE)
    return(TRUE)
  }

  FALSE
})

#' Relaxed Value Matching
#'
#' \code{match.closest} returns a vector of the positions of (first) matches
#' its first arguments in its second. In contrast to the similar
#' \code{\link{match}} it just accept \code{numeric} arguments but
#' has an additional \code{tolerance} argument that allows relaxed
#' matching.
#'
#' @param x \code{numeric}, the values to be matched.
#' @param table \code{numeric}, the values to be matched against. In contrast to
#' \code{\link{match}} \code{table} has to be sorted in increasing order.
#' @param tolerance \code{numeric}, accepted tolerance. Use \code{Inf} to match without
#' restrictions. Could be of length one or the same length as \code{table}.
#' @param nomatch \code{numeric}, if the difference
#' between the value in \code{x} and \code{table} is larger than
#' \code{tolerance} \code{nomatch} is returned. Has to be of length one.
#'
#' @return An \code{integer} vector of the same length as \code{x} giving the
#' closest position in \code{table} of the first match or \code{nomatch} if
#' there is no match.
#'
#' @seealso \code{\link{match}}
#' @aliases match.closest
#' @export
#' @examples
#' library("MALDIquant")
#' match.closest(c(1.1, 1.4, 9.8), 1:10)
#' # [1]  1  1 10
#' match.closest(c(1.1, 1.4, 9.8), 1:10, tolerance=0.25)
#' # [1]  1 NA 10
#' match.closest(c(1.1, 1.4, 9.8), 1:10, tolerance=0.25, nomatch=0)
#' # [1]  1  0 10
#'
#' ## this function is most useful if you want to subset an intensityMatrix
#' ## by a few (reference) peaks
#'
#' ## create an example intensityMatrix
#' im <- matrix(1:10, nrow=2, dimnames=list(NULL, 1:5))
#' attr(im, "mass") <- 1:5
#' im
#' #      1 2 3 4  5
#' # [1,] 1 3 5 7  9
#' # [2,] 2 4 6 8 10
#' # attr(,"mass")
#' # [1] 1 2 3 4 5
#'
#' ## reference peaks
#' ref <- c(2.2, 4.8)
#'
#' im[, match.closest(ref, attr(im, "mass"), tolerance=0.25, nomatch=0)]
#' #      2  5
#' # [1,] 3  9
#' # [2,] 4 10
#'
match.closest <- function(x, table, tolerance=Inf, nomatch=NA_integer_) {
  lIdx <- findInterval(x, table, rightmost.closed=FALSE, all.inside=TRUE)
  rIdx <- lIdx + 1L

  lIdx[lIdx == 0L] <- 1L

  lDiff <- abs(table[lIdx] - x)
  rDiff <- abs(table[rIdx] - x)

  d <- which(lDiff >= rDiff)

  lIdx[d] <- rIdx[d]

  if (any(is.finite(tolerance))) {
    if (any(tolerance < 0L)) {
      warning(sQuote("tolerance"), " < 0 is meaningless. Set to zero.")
      tolerance[tolerance < 0L] <- 0L
    }

    if (length(nomatch) != 1L) {
      stop("Length of ", sQuote("nomatch"), " has to be one.")
    }

    tolerance <- rep_len(tolerance, length(table))

    lDiff[d] <- rDiff[d]
    lIdx[lDiff > tolerance[lIdx]] <- nomatch
  }

  lIdx
}

context("match.closest")

test_that("match.closest", {
  expect_error(match.closest(1, c(0, -1, 3)), "sorted non-decreasingly")
  expect_equal(match.closest(1.001, 1:10), 1)
  expect_equal(match.closest(1.4, 1:10), 1)
  expect_equal(match.closest(9.8, 1:10), 10)
  expect_equal(match.closest(11.1, 1:10), 10)
  expect_equal(match.closest(4:5, 4.8, tolerance=1), c(1, 1))
  expect_equal(match.closest(c(0.5, 1.5, exp(1), pi), 1:10),
               c(1, 2, 3, 3))
})

test_that("match.closest, length(table) == 1", {
  expect_equal(match.closest(1:3, 0, nomatch=0, tolerance=0), c(0, 0, 0))
  expect_equal(match.closest(1:3, 1, nomatch=0, tolerance=0), c(1, 0, 0))
  expect_equal(match.closest(1:3, 2, nomatch=0, tolerance=0), c(0, 1, 0))
  expect_equal(match.closest(1:3, 3, nomatch=0, tolerance=0), c(0, 0, 1))
  expect_equal(match.closest(1:3, 4, nomatch=0, tolerance=0), c(0, 0, 0))
})

test_that("match.closest, tolerance", {
  expect_error(match.closest(1, 1, tolerance=1, nomatch=1:2), "Length of .*nomatch.* has to be one")
  expect_warning(match.closest(1, 1, tolerance=-1), ".*tolerance.* < 0 is meaningless")
  expect_equal(match.closest(1.001, 1:10, tolerance=0), NA_integer_)
  expect_equal(suppressWarnings(match.closest(c(1,2,3.1), 1:3, tolerance=-1)), c(1, 2, NA_integer_))
  expect_equal(match.closest(1.001, 1:10, tolerance=0, nomatch=-1), -1)
  expect_equal(match.closest(1.4, 1:10, tolerance=0.4), 1)
})

## averageMassSpectra
##  averages MassSpectrum objects
##
## params:
##  l: list of MassSpectrum objects
##  labels: factor, labels for samples
##  method: aggregation method
##
## returns:
##  a new MassSpectrum object or a list of new MassSpectra objects
##
averageMassSpectra <- function(l, labels, method=c("mean", "median", "sum"),
                               ...) {

  ## test parameters
  .stopIfNotIsMassSpectrumList(l)

  method <- match.arg(method)

  fun <- switch(method,
              "mean" = {
                colMeans
              },
              "median" = {
                .colMedians
              },
              "sum" = {
                colSums
              }
  )

  .doByLabels(l=l, labels=labels, FUN=.averageMassSpectra, fun=fun, ...)
}

## .averageMassSpectra
##  average MassSpectrum objects
##
## params:
##  l: list of MassSpectrum objects
##  fun: aggregation function
##  ignore.na: ignore NA/set them to 0
##
## returns:
##  a new MassSpectrum object
##
.averageMassSpectra <- function(l, fun=colMeans, mergeMetaData=TRUE) {

  fun <- match.fun(fun)

  ## merge metaData
  if (mergeMetaData) {
    metaData <- .mergeMetaData(lapply(l, function(x)x@metaData))
  } else {
    metaData <- list()
  }

  ## use the first non empty spectrum as reference
  i <- which(!vapply(l, isEmpty, logical(1L)))[1L]
  if (!is.na(i)) {
    mass <- l[[i]]@mass
  } else {
    mass <- NA_real_
  }

  ## interpolate not existing masses
  approxSpectra <- lapply(l, approxfun)

  ## get interpolated intensities
  intensityList <- lapply(approxSpectra, function(x)x(mass))

  ## create a matrix which could merged
  m <- do.call(rbind, intensityList)

  ## merge intensities
  intensity <- fun(m, na.rm=TRUE)

  ## create an empty spectrum if all intensities are NaN
  if (is.nan(intensity[1L])) {
    intensity <- double()
    mass <- double()
  }

  createMassSpectrum(mass=mass, intensity=intensity, metaData=metaData)
}

.isMassObject <- function(x) {
  inherits(x=x, what="AbstractMassObject")
}

isMassSpectrum <- function(x) {
  is(object=x, class2="MassSpectrum")
}

isMassPeaks <- function(x) {
  is(object=x, class2="MassPeaks")
}

context("isMassObject")

s <- createMassSpectrum(mass=1:10, intensity=1:10)
p <- createMassPeaks(mass=1:10, intensity=1:10)

test_that(".isMassObject", {
  expect_false(MALDIquant:::.isMassObject(double()))
  expect_false(MALDIquant:::.isMassObject(list()))

  expect_true(MALDIquant:::.isMassObject(s))
  expect_true(MALDIquant:::.isMassObject(p))
})

test_that("isMassSpectrum", {
  expect_false(isMassSpectrum(double()))
  expect_false(isMassSpectrum(list()))
  expect_false(isMassSpectrum(p))

  expect_true(isMassSpectrum(s))
})

test_that("isMassPeaks", {
  expect_false(isMassPeaks(double()))
  expect_false(isMassPeaks(list()))
  expect_false(isMassPeaks(s))

  expect_true(isMassPeaks(p))
})

.isMassObjectList <- function(x) {
  if (!is.list(x)) {
    return(FALSE)
  }

  length(x) && all(unname(vapply(x, .isMassObject, logical(1L))))
}

.stopIfNotIsMassObjectList <- function(x) {
  if (!.isMassObjectList(x)) {
    parentCall <- deparse(sys.call(-1L))
    stop(parentCall, " : ", sQuote(deparse(substitute(x))),
         " is no list of MALDIquant::AbstractMassObject objects!", call.=FALSE)
  }
  TRUE
}

isMassSpectrumList <- function(x) {
  if (!is.list(x)) {
    return(FALSE)
  }

  length(x) && all(unname(vapply(x, isMassSpectrum, logical(1L))))
}

.stopIfNotIsMassSpectrumList <- function(x) {
  if (!isMassSpectrumList(x)) {
    parentCall <- deparse(sys.call(-1L))
    stop(parentCall, " : ", sQuote(deparse(substitute(x))),
         " is no list of MALDIquant::MassSpectrum objects!", call.=FALSE)
  }
  TRUE
}

isMassPeaksList <- function(x) {
  if (!is.list(x)) {
    return(FALSE)
  }

  length(x) && all(unname(vapply(x, isMassPeaks, logical(1L))))
}

.stopIfNotIsMassPeaksList <- function(x) {
  if (!isMassPeaksList(x)) {
    parentCall <- deparse(sys.call(-1L))
    stop(parentCall, " : ", sQuote(deparse(substitute(x))),
         " is no list of MALDIquant::MassPeaks objects!", call.=FALSE)
  }
  TRUE
}

context("isMassObjectList")

s <- createMassSpectrum(mass=1:10, intensity=1:10)
p <- createMassPeaks(mass=1:10, intensity=1:10)

test_that(".isMassObjectList", {
  expect_false(MALDIquant:::.isMassObjectList(double(2)))
  expect_false(MALDIquant:::.isMassObjectList(list()))
  expect_false(MALDIquant:::.isMassObjectList(s))
  expect_false(MALDIquant:::.isMassObjectList(p))

  expect_true(MALDIquant:::.isMassObjectList(list(s, s)))
  expect_true(MALDIquant:::.isMassObjectList(list(s, p)))
  expect_true(MALDIquant:::.isMassObjectList(list(p, p)))
})

test_that("isMassSpectrumList", {
  expect_false(isMassSpectrumList(double(2)))
  expect_false(isMassSpectrumList(list()))
  expect_false(isMassSpectrumList(s))
  expect_false(isMassSpectrumList(p))
  expect_false(isMassSpectrumList(list(s, p)))
  expect_false(isMassSpectrumList(list(p, p)))

  expect_true(isMassSpectrumList(list(s, s)))
})

test_that("isMassPeaksList", {
  expect_false(isMassPeaksList(double(2)))
  expect_false(isMassPeaksList(list()))
  expect_false(isMassPeaksList(s))
  expect_false(isMassPeaksList(p))
  expect_false(isMassPeaksList(list(s, s)))
  expect_false(isMassPeaksList(list(s, p)))

  expect_true(isMassPeaksList(list(p, p)))
})

## alignSpectra
##  wrapper around detectPeaks, determineWarpingFunctions and warpMassSpectra
##
## params:
##  spectra: list, list of MassSpectrum objects
##  halfWindowSize: numeric, half window size.
##  noiseMethod: character, noise estimation method
##  SNR: double, signal-to-noise ratio
##  reference: MassPeaks, a reference MassPeaks object to which all other
##             MassPeaks objects should be aligned
##  tolerance: double, maximal deviation of a peak position to be
##             considered as same peak
##  warpingMethod: choose type of base warping function
##  allowNoMatches: logical, don't throw an error if a single MassPeaks object
##                  could not match to the reference.
##  emptyNoMatches: logical, if TRUE mismatches (warping function NA)
##
## returns:
##  a list of aligned MassSpectrum objects
##
alignSpectra <- function(spectra,
                         ## peak detection
                         halfWindowSize=20, noiseMethod="MAD", SNR=2,
                         ## warping
                         reference, tolerance=0.002, warpingMethod="lowess",
                         allowNoMatches=FALSE, emptyNoMatches=FALSE, ...) {

  ## test arguments
  .stopIfNotIsMassSpectrumList(spectra)

  peaks <- detectPeaks(spectra, halfWindowSize=halfWindowSize,
                       method=noiseMethod, SNR=SNR, ...)
  wf <- determineWarpingFunctions(peaks, reference=reference,
                                  tolerance=tolerance, method=warpingMethod,
                                  allowNoMatches=allowNoMatches)
  warpMassSpectra(spectra, wf, emptyNoMatches=emptyNoMatches)
}

## mergeMassPeaks
##  merge MassPeaks objects
##
## params:
##  l: list of MassPeaks objects
##  labels: factor, labels for samples
##  fun: merge function
##
## returns:
##  a new MassPeaks object or a list of new MassPeaks objects
##
mergeMassPeaks <- function(l, labels, method=c("mean", "median", "sum"),
                           ignore.na=TRUE, ...) {

  ## test arguments
  .stopIfNotIsMassPeaksList(l)

  method <- match.arg(method)

  fun <- switch(method,
              "mean" = {
                colMeans
              },
              "median" = {
                .colMedians
              },
              "sum" = {
                colSums
              }
  )

  .doByLabels(l=l, labels=labels, FUN=.mergeMassPeaks, fun=fun,
              ignore.na=ignore.na, ...)
}

## .mergeMassPeaks
##  merge MassPeaks objects
##
## params:
##  l: list of MassPeaks objects
##  fun: merge function
##
## returns:
##  a new MassPeaks object
##
.mergeMassPeaks <- function(l, fun=colMeans, ignore.na=TRUE) {

  fun <- match.fun(fun)

  ## create a matrix which could merged
  m <- .as.matrix.MassObjectList(l)

  mass <- attr(m, "mass")

  ## avoid named intensity/snr slot
  colnames(m) <- NULL

  isNA <- is.na(m)
  if (!ignore.na) {
    m[isNA] <- 0L
  }

  ## merge intensities
  intensity <- fun(m, na.rm=TRUE)

  ## merge snr
  for (i in seq_along(l)) {
    m[i, !isNA[i, ]] <- l[[i]]@snr
  }
  snr <- fun(m, na.rm=TRUE)

  ## merge metaData
  metaData <- .mergeMetaData(lapply(l, function(x)x@metaData))

  createMassPeaks(mass=mass, intensity=intensity, snr=snr, metaData=metaData)
}

## merge different metaData by equal list names
##
## params
##  m: list of metaData
##
## returns:
##  merged list
##
.mergeMetaData <- function(m) {

  .flat <- function(x)unname(unlist(x))

  nm <- names(m[[1L]])
  names(nm) <- nm
  lapply(nm, function(n) {
    cur <- m[[1L]][[n]]
    all <- lapply(m, "[[", n)
    len <- lengths(all)

    if (all(length(cur) == len)) {
        fcur <- .flat(cur)
        fall <- .flat(all)
        if (all(is.na(fcur) == is.na(fall)) && all(fcur == fall, na.rm=TRUE))
            return(cur)
    }

    if (!is.list(cur)) {
        all <- unlist(all)
    }
    unname(all)
  })
}

context("merge")

s <- c(createMassSpectrum(mass=1:3, intensity=1:3, metaData=list(a=1, b=2)),
       createMassSpectrum(mass=1:3, intensity=4:6, metaData=list(a=1, b=3)))

p <- c(createMassPeaks(mass=1:3, intensity=1:3, snr=1:3,
                       metaData=list(a=1, b=2)),
       createMassPeaks(mass=1:3, intensity=4:6, snr=4:6,
                       metaData=list(a=1, b=3)))

pNA <- c(createMassPeaks(mass=1:3, intensity=1:3, snr=1:3,
                         metaData=list(a=1, b=2)),
         createMassPeaks(mass=1:2, intensity=4:5, snr=4:5,
                         metaData=list(a=1, b=3)))

meanP <- createMassPeaks(mass=1:3, intensity=(2:4)+0.5, snr=(2:4)+0.5,
                         metaData=list(a=1, b=2:3))

meanPNA <- createMassPeaks(mass=1:3, intensity=c(2.5, 3.5, 1.5),
                           snr=c(2.5, 3.5, 1.5), metaData=list(a=1, b=2:3))
sumP <- createMassPeaks(mass=1:3, intensity=c(5, 7, 9), snr=c(5, 7, 9),
                        metaData=list(a=1, b=2:3))

test_that("mergeMassPeaks", {
  expect_equal(unname(mergeMassPeaks(list(p[[1]], p[[1]], p[[2]], p[[2]]),
                                     labels=c(1:2, 1:2))),
               list(meanP, meanP))
  expect_equal(unname(mergeMassPeaks(pNA[rep(1:2, each=2)],
                                     labels=rep(1:2, times=2), ignore.na=FALSE)),
               list(meanPNA, meanPNA))
  expect_equal(unname(mergeMassPeaks(list(p[[1]], p[[1]], p[[2]], p[[2]]),
                                     labels=c(1:2, 1:2), method="sum")),
               list(sumP, sumP))
  expect_equal(unname(mergeMassPeaks(pNA[rep(1:2, each=2)], method="median",
                      ignore.na=FALSE)),
               createMassPeaks(mass=1:3, intensity=c(2.5, 3.5, 1.5),
                               snr=c(2.5, 3.5, 1.5),
                               metaData=list(a=1, b=rep(2:3, each=2))))
})

test_that("mergeMassPeaks throws errors", {
  expect_error(mergeMassPeaks(1:3),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(mergeMassPeaks(list()),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(mergeMassPeaks(s),
               "no list of MALDIquant::MassPeaks objects")
})

test_that(".mergeMassPeaks", {
  expect_equal(MALDIquant:::.mergeMassPeaks(p), meanP)
  expect_equal(MALDIquant:::.mergeMassPeaks(p, fun=colSums), sumP)
})

test_that(".mergeMetaData", {
  l <- list(a=list(numbers=1:3, lists=list(a=1, b=2), characters=c("a", "b")),
            b=list(numbers=1:3, lists=list(a=1, b=2), characters=c("b", "c")))
  r <- list(numbers=1:3, lists=list(a=1, b=2),
            characters=c("a", "b", "b", "c"))
  expect_identical(MALDIquant:::.mergeMetaData(l), r)
  l <- list(a=list(numbers=1:2, lists=list(a=1, b=2, d=4),
                   characters=c("a", "b", "c")),
            b=list(numbers=1:3, lists=list(a=1, b=2), characters=c("b", "c")))
  r <- list(numbers=c(1:2, 1:3), lists=list(list(a=1, b=2, d=4),
                                            list(a=1, b=2)),
            characters=c("a", "b", "c", "b", "c"))
  expect_identical(MALDIquant:::.mergeMetaData(l), r)
})

## AbstractMassObject
.validAbstractMassObject <- function(object) {
  if (length(object@mass) != length(object@intensity)) {
    return(paste0("Lengths of mass (", length(object@mass),
                  ") and intensity (", length(object@intensity),
                  ") have to be equal."))
  }
  if (is.numeric(object@mass) &&
      length(object@mass) &&
      any(object@mass < 0L)) {
    warning("Negative mass values found.")
  }
  if (is.numeric(object@intensity) &&
      !isEmpty(object) &&
      any(object@intensity < 0L)) {
    warning("Negative intensity values found.")
  }
  if (is.unsorted(object@mass)) {
    warning("Unsorted mass values found.")
  }
  TRUE
}

setValidity("AbstractMassObject", method=.validAbstractMassObject)

.validMassPeaks <- function(object) {
  if (length(object@intensity) != length(object@snr)) {
    return(paste0("Lengths of intensity (", length(object@intensity),
                  ") and snr (", length(object@snr),
                  ") have to be equal."))
  }
  .validAbstractMassObject(object)
}

setValidity("MassPeaks", method=.validMassPeaks)

## basic class for all spectra based information
setClass("AbstractMassObject",
         slots=list(mass="numeric", intensity="numeric",
                    metaData="list"),
         prototype=list(mass=numeric(), intensity=numeric(),
                        metaData=list()),
         contains="VIRTUAL")

## represent a spectrum
setClass("MassSpectrum",
         contains="AbstractMassObject")

## represent a peak list from a single spectrum
setClass("MassPeaks",
         slots=list(snr="numeric"),
         prototype=list(snr=numeric()),
         contains="AbstractMassObject")

## local maxima function called by .findLocalMaxima
##
## This function looks for local maxima in a numeric vector.
##
## params:
##  y: double, intensity
##  halfWindowSize: numeric, half window size.
##
## returns:
##  logical vector of local maxima
##

.localMaxima <- function(y, halfWindowSize=1L) {
  y <- c(rep.int(0L, halfWindowSize), y, rep.int(0L, halfWindowSize))
  i <- (halfWindowSize + 1L):(length(y) - halfWindowSize)
  .Call(C_localMaxima, y, halfWindowSize)[i]
}

context(".localMaxima")

y <- c(1, 1, 2, 1, 1, 3, 4)
set.seed(1)
b <- rnorm(5e4)

test_that(".localMaxima", {
  expect_identical(MALDIquant:::.localMaxima(y, 1),
                   c(T, F, T, F, F, F, T))
})

.onAttach <- function(libname, pkgname) {
  packageStartupMessage("\nThis is MALDIquant version ",
    utils::packageVersion("MALDIquant"), "\n",
    "Quantitative Analysis of Mass Spectrometry Data\n",
    " See ", sQuote("?MALDIquant"),
    " for more information about this package.\n")
}

## .estimateNoise
##  estimating the noise of a spectrum
##
## params:
##  x: vector of x values (mass)
##  y: vector of y values (intensity)
##  method: method to use
##  ...: further arguments passed to "method"
##
## returns:
##  numeric, estimated noise (y)
##
.estimateNoise <- function(x, y, method=c("MAD", "SuperSmoother"), ...) {

  method <- match.arg(method)

  switch(method,
         "MAD" = {
           .estimateNoiseMad(x, y)
         },
         "SuperSmoother" = {
           .estimateNoiseSuperSmoother(x, y, ...)
         }
  )
}

## estimateNoiseMad
##  estimate noise by calculating mad over intensity values
##
## params:
##  x: vector of x values
##  y: vector of y values
##
## returns:
##  numeric, estimated noise (y)
##
.estimateNoiseMad <- function(x, y) {
  rep.int(stats::mad(y), length(x))
}

## estimateNoiseSuperSmoother
##  estimate noise by using Friedman's super smoother
##
## params:
##  x: vector of x values
##  y: vector of y values
##  ...: further arguments to passed to supsmu
##
## returns:
##  numeric, estimated noise (y)
##
.estimateNoiseSuperSmoother <- function(x, y, ...) {
  stats::supsmu(x=x, y=y, ...)$y
}

context("estimateNoise-methods")

s <- createMassSpectrum(mass=1:20, intensity=rep(10:1, 2))

test_that("estimateNoise throws errors", {
  expect_error(estimateNoise(s, method="foobar"),
               ".*arg.* should be one of .*MAD.*, .*SuperSmoother.*")
})

test_that("estimateNoise shows warnings", {
  expect_warning(estimateNoise(
                      createMassSpectrum(mass=double(), intensity=double())),
                 "empty")
  expect_identical(suppressWarnings(estimateNoise(
                      createMassSpectrum(mass=double(), intensity=double()))),
                   0L)
})

test_that(".estimateNoiseMad", {
  n <- rep(stats::mad(intensity(s)), length(s))
  m <- cbind(mass=mass(s), intensity=n)
  expect_identical(MALDIquant:::.estimateNoiseMad(1:20, intensity(s)), n)
  expect_identical(MALDIquant:::.estimateNoise(mass(s), intensity(s)), n)
  expect_identical(MALDIquant:::.estimateNoise(mass(s), intensity(s),
                                               method="MAD"), n)
  expect_identical(estimateNoise(s), m)
  expect_identical(estimateNoise(s, method="MAD"), m)
})

test_that(".estimateNoiseSuperSmoother", {
  n <- stats::supsmu(x=1:20, y=intensity(s))$y
  m <- cbind(mass=mass(s), intensity=n)
  expect_identical(MALDIquant:::.estimateNoiseSuperSmoother(mass(s),
                                                            intensity(s)), n)
  expect_identical(MALDIquant:::.estimateNoise(mass(s), intensity(s),
                                               method="SuperSmoother"), n)
  expect_identical(estimateNoise(s, method="SuperSmoother"), m)
})

## deprecated since MALDIquant 1.15.2
## We have to keep it until May 2017 because the current MSnbase 2.0.x depends
## on this internal function.
.which.closest <- function(x, vec) {
  match.closest(x, vec, tolerance=Inf, nomatch=NA_integer_)
}

## referencePeaks
##  calculate reference peaks (a wrapper around filterPeaks and binPeaks)
##
## params:
##  l: list of MassPeaks objects
##  method: character, grouper to used (strict: don't allow multiple peaks of
##          the same sample in the same bin, relaxed: allow them)
##  minFrequency: double, minimal frequency of a peak to be not removed
##  tolerance: double, maximal deviation of a peak position to be
##             considered as same peak
##
## returns:
##  a new MassPeaks object
##
referencePeaks <- function(l, method=c("strict", "relaxed"), minFrequency=0.9,
                           tolerance=0.002) {

  .stopIfNotIsMassPeaksList(l)

  method <- match.arg(method)

  ## find reference peaks by binning and filtering
  referencePeaks <- filterPeaks(binPeaks(l, method=method,
                                         tolerance=tolerance),
                                minFrequency=minFrequency)

  m <- .as.binary.matrix(.as.matrix.MassObjectList(referencePeaks))

  ## set peak intensity to number of occurrence
  intensity <- unname(colMeans(m))

  createMassPeaks(mass=attr(m, "mass"), intensity=intensity)
}

## .replaceNegativeIntensityValues
##  replaces negative intensity valus by zeros
##
## params:
##  object AbstractMass Object
##  warn: throw a warning?
##
## returns:
##  an AbstractMass object
##
.replaceNegativeIntensityValues <- function(object, warn=TRUE) {
  if (any(object@intensity < 0L, na.rm=TRUE) && !isEmpty(object)) {
    if (warn) {
      warning("Negative intensity values are replaced by zeros.")
    }
    object@intensity[which(object@intensity < 0L)] <- 0L
  }
  object
}

## msiSlices
##  create intensity matrix for each slice from a list of MassSpectrum/MassPeaks
##  objects
##
## params:
##  x: list of MassSpectrum/MassPeaks objects
##  center: position(s) of interest (in mass values)
##  tolerance: aggregate data around center +/- tolerance
##  method: aggregate method
##  adjust: set coordinates to 1,1 (if there are empty/missing spectra before)
##
## returns:
##  an array (dim: x, y, z=slice nr)
##
msiSlices <- function(x, center, tolerance, method=c("sum", "mean", "median"),
                      adjust=TRUE) {
  x <- suppressWarnings(trim(x, range=range(center) + c(-tolerance, tolerance)))

  .msiSlices(m=.as.matrix.MassObjectList(x),
             coord=coordinates(x, adjust=adjust),
             center=center, tolerance=tolerance, method=method)
}

.msiSlices <- function(m, coord, center, tolerance,
                       method=c("sum", "mean", "median")) {
  method <- match.arg(method)

  fun <- switch(method,
                "sum" = rowSums,
                "mean" = rowMeans,
                "median" = function(x, ...).colMedians(t(x), ...))

  n <- unname(apply(coord, MARGIN=2L, FUN=max))

  l <- pmin(findInterval(center - tolerance - .Machine$double.eps,
                         attr(m, "mass")) + 1L, ncol(m))
  r <- findInterval(center + tolerance + .Machine$double.eps, attr(m, "mass"))

  slices <- array(NA_real_, dim=c(x=n[1L], y=n[2L], z=length(center)))

  for (i in seq_along(center)) {
    slices[cbind(coord, i)] <- fun(m[, l[i]:r[i], drop=FALSE], na.rm=TRUE)
  }

  attr(slices, "center") <- center
  attr(slices, "tolerance") <- tolerance
  attr(slices, "method") <- method
  slices
}

context("msiSlices")

test_that("msiSlices", {
  p <- list(createMassPeaks(mass=1:5, intensity=1:5),
            createMassPeaks(mass=1:5, intensity=2:6),
            createMassPeaks(mass=1:5, intensity=3:7))
  coordinates(p) <- cbind(x=c(2, 2, 3), y=c(2, 3, 2))

  r <- array(c(3, 5, 4, NA), dim=c(x=2, y=2, z=1))
  attr(r, "center") <- 3
  attr(r, "tolerance") <- 0.5
  attr(r, "method") <- "sum"

  expect_equal(msiSlices(p, center=3, tolerance=0.5), r)

  r <- array(c(NA, NA, NA, NA, 3, 5, NA, 4, NA), dim=c(x=3, y=3, z=1))
  attr(r, "center") <- 3
  attr(r, "tolerance") <- 0.5
  attr(r, "method") <- "sum"
  expect_equal(msiSlices(p, center=3, tolerance=0.5, adjust=FALSE), r)
})

test_that(".msiSlices", {
  m <- matrix(c(1:5, 2:6, 3:7), byrow=TRUE, nrow=3)
  attr(m, "mass") <- 1:5
  coord <- cbind(x=c(1, 1, 2), y=c(1, 2, 1))

  r <- array(c(3, 5, 4, NA), dim=c(x=2, y=2, z=1))
  attr(r, "center") <- 3
  attr(r, "tolerance") <- 0.5
  attr(r, "method") <- "sum"
  expect_equal(MALDIquant:::.msiSlices(m, coord, center=3, tolerance=0.5), r)

  r[,,1] <- c(9, 15, 12, NA)
  attr(r, "tolerance") <- 1
  expect_equal(MALDIquant:::.msiSlices(m, coord, center=3, tolerance=1), r)

  r[,,1] <- c(3, 5, 4, NA)
  attr(r, "tolerance") <- 1
  attr(r, "method") <- "mean"
  expect_equal(MALDIquant:::.msiSlices(m, coord, center=3, tolerance=1,
                                       method="mean"), r)

  r[,,1] <- c(3, 5, 4, NA)
  attr(r, "tolerance") <- 1
  attr(r, "method") <- "median"
  expect_equal(MALDIquant:::.msiSlices(m, coord, center=3, tolerance=1,
                                       method="median"), r)

  r <- array(c(6, 12, 9, NA, 9, 15, 12, NA), dim=c(x=2, y=2, z=2))
  attr(r, "center") <- 2:3
  attr(r, "tolerance") <- 1
  attr(r, "method") <- "sum"
  expect_equal(MALDIquant:::.msiSlices(m, coord, center=2:3, tolerance=1), r)

  r[,,2] <- c(15, 25, 20, NA)
  attr(r, "tolerance") <- 1:2
  expect_equal(MALDIquant:::.msiSlices(m, coord, center=2:3, tolerance=1:2), r)
})

## AbstractMassObject
setMethod(f="transformIntensity",
          signature=signature(object="AbstractMassObject"),
          definition=function(object,
                              method=c("sqrt", "log", "log2", "log10")) {

  ## empty spectrum?
  if (.isEmptyWarning(object)) {
    return(object)
  }

  method <- match.arg(method)

  fun <- switch(method,
         "sqrt" = {
           sqrt
         },
         "log" = {
           log
         },
         "log2" = {
           log2
         },
         "log10" = {
           log10
         }
  )

  .transformIntensity(object, fun=fun)
})

## AbstractMassObject
setMethod(f=".transformIntensity",
          signature=signature(object="AbstractMassObject"),
          definition=function(object, fun, na.rm=TRUE, ...) {

  if (!isEmpty(object)) {
    fun <- match.fun(fun)

    object@intensity <- fun(object@intensity, ...)

    if (na.rm) {
      naIdx <- which(!is.na(object@intensity))
      object@intensity <- object@intensity[naIdx]
      object@mass <- object@mass[naIdx]
    }

    object <- .replaceNegativeIntensityValues(object)
  }

  object
})

## list
setMethod(f="transformIntensity",
          signature=signature(object="list"),
          definition=function(object, ...) {


  ## test arguments
  .stopIfNotIsMassObjectList(object)

  .lapply(object, transformIntensity, ...)
})

## list
setMethod(f=".transformIntensity",
          signature=signature(object="list"),
          definition=function(object, ...) {

  ## test arguments
  .stopIfNotIsMassObjectList(object)

  .lapply(object, .transformIntensity, ...)
})

## MassSpectrum
setMethod(f="estimateBaseline",
          signature=signature(object="MassSpectrum"),
          definition=function(object, method=c("SNIP", "TopHat", "ConvexHull",
                                               "median"),
                              ...) {
  if (.isEmptyWarning(object)) {
    return(NA)
  }

  cbind(mass=object@mass,
        intensity=.estimateBaseline(x=object@mass, y=object@intensity,
                                    method=method, ...))
})

## AbstractMassObject
setMethod(f="mz",
          signature=signature(object="AbstractMassObject"),
          definition=function(object, ...) {

  mass(object)
})

## AbstractMassObject
setReplaceMethod(f="mz",
          signature=signature(object="AbstractMassObject", value="numeric"),
          definition=function(object, value) {

  mass(object) <- value
  object
})

## .overlap
## returns largest overlapping mass range of a list of AbstractMassObject
## objects.
##
## params:
## l: list of AbstractMassObject objects
##
## returns:
## double, minimal and maximal mass
##
.overlap <- function(l) {
  ## test argument
  .stopIfNotIsMassObjectList(l)

  ## mass values are already sorted
  leftMass <- .unlist(lapply(l, function(x)x@mass[1L]))
  rightMass <- .unlist(lapply(l, function(x)x@mass[length(x@mass)]))

  if (length(rightMass)) {
    r <- c(max(leftMass, na.rm=TRUE), min(rightMass, na.rm=TRUE))
    if (r[1L] < r[2L]) {
      return(r)
    }
  }

  ## no overlap
  c(0L, 0L)
}

## .reorderRange
## swap range values if needed
##
## params:
## x: range values
##
## returns:
## corrected range values
##
.reorderRange <- function(x) {
  ## sort range
  if (x[1L] > x[2L]) {
    x <- x[2L:1L]
  }

  x
}

context("range")

test_that(".overlap", {
  l1 <- c(createMassSpectrum(1L:10L, 1L:10L),
          createMassSpectrum(5L:15L, 5L:15L))

  l2 <- c(createMassSpectrum(1L:10L, 1L:10L),
          createMassSpectrum(15L:25L, 15L:25L))

  expect_identical(MALDIquant:::.overlap(l1), c(5L, 10L))
  expect_identical(MALDIquant:::.overlap(l2), c(0L, 0L))
})

test_that(".reorderRange", {
  expect_identical(MALDIquant:::.reorderRange(c(5L, 10L)), c(5L, 10L))
  expect_identical(MALDIquant:::.reorderRange(c(10L, 5L)), c(5L, 10L))
})

## AbstractMassObject
setMethod(f="as.matrix",
          signature=signature(x="AbstractMassObject"),
          definition=function(x, index) {

  matrix(c(x@mass[index], x@intensity[index]), ncol=2L, byrow=FALSE,
         dimnames=list(NULL, c("mass", "intensity")))
})

## intensityMatrix
##  converts a list of MassPeaks into an expression matrix
##
## params:
##  peaks: list of MassPeaks objects
##  spectra: list of MassSpectrum objects
##
## returns:
##  a matrix
##
intensityMatrix <- function(peaks, spectra) {

  ## test arguments
  .stopIfNotIsMassPeaksList(peaks)

  m <- .as.matrix.MassObjectList(peaks)

  ## lookup corresponding intensity values in spectra for missing peaks
  if (!missing(spectra)) {
    .stopIfNotIsMassSpectrumList(spectra)

    if (length(peaks) != length(spectra)) {
      stop("Incompatible number of spectra!")
    }

    isNa <- is.na(m)
    uniqueMass <- as.double(colnames(m))

    approxSpectra <- lapply(spectra, approxfun, yleft=0L, yright=0L)

    for (i in seq_along(approxSpectra)) {
      m[i, isNa[i, ]] <- approxSpectra[[i]](uniqueMass[isNa[i, ]])
    }
  }

  m
}

## .stopIfNotIsValidHalfWindowSize
##  test for too small/large half window size
##
## params:
##  halfWindowSize: half window size
##  n: length of data
##
## returns:
##  TRUE if valid
##
.stopIfNotIsValidHalfWindowSize <- function(halfWindowSize, n) {
  parentCall <- deparse(sys.call(-1L))

  if (halfWindowSize < 1L) {
    stop(parentCall, " : ", sQuote("halfWindowSize"),
         " is too small!", call.=FALSE)
  }

  windowSize <- halfWindowSize * 2L + 1L

  if (windowSize > n) {
    stop(parentCall, " : ", sQuote("halfWindowSize"),
         " is too large!", call.=FALSE)
  }

  TRUE
}

## AbstractMassObject
setMethod(f="plot",
          signature=signature(x="AbstractMassObject", y="missing"),
          definition=function(x, col="black",
                              xlab=expression(italic(m/z)), ylab="intensity",
                              type=ifelse(isMassPeaks(x), "h", "l"),
                              xlim=c(ifelse(length(x@mass),
                                            min(x@mass, na.rm=TRUE), 0L),
                                     ifelse(length(x@mass),
                                            max(x@mass, na.rm=TRUE), 1L)),
                              ylim=c(0, ifelse(length(x@intensity),
                                               max(x@intensity, na.rm=TRUE),
                                               1L)),
                              main=x@metaData$name, sub=x@metaData$file,
                              cex.sub=0.75, col.sub="#808080", ...) {

  if (all(sub == x@metaData$file) && length(x@metaData$file) > 1L) {
    sub <- paste0(ifelse(isMassSpectrum(x),
                         "averaged spectrum", "merged peaks"),
                  " composed of ", length(x@metaData$file), " ",
                  ifelse(isMassSpectrum(x), "MassSpectrum", "MassPeaks"),
                  " objects")
  }

  plot(x=x@mass, y=x@intensity, col=col, type=type, xlab=xlab, ylab=ylab,
       xlim=xlim, ylim=ylim, main=main, sub=sub, cex.sub=cex.sub,
       col.sub=col.sub, ...)
})

setMethod(f="labelPeaks",
          signature=signature(object="MassPeaks"),
          definition=function(object, index, mass, labels, digits=3L,
                              underline=TRUE,
                              ## verticalOffset ca. 0.01 of plot height
                              verticalOffset=abs(diff(par("usr")[3L:4L]))*0.01,
                              absoluteVerticalPos, adj=c(0.5, 0L), cex=0.7,
                              srt=0L, avoidOverlap=FALSE,
                              arrowLength=0L, arrowLwd=0.5, arrowCol=1L, ...) {

  ## index
  if (missing(index) && missing(mass)) {
    index <- seq_along(object@mass)
  } else if (!missing(index) && is.logical(index)) {
    index <- which(index)
  }

  if (!missing(mass) && is.numeric(mass)) {
    massIdx <- match.closest(mass, object@mass)

    if (missing(index)) {
      index <- massIdx
    } else {
      index <- c(index, massIdx)
    }

    ## remove duplicated indices
    index <- unique(index)
  }

  isValidIndex <- length(index) &&
                  length(index) <= length(object@mass) &&
                  (min(index) >= 1L && max(index) <= length(object@mass))

  if (!isValidIndex) {
    stop("No valid ", sQuote("index"), " nor ", sQuote("mass"), " given.")
  }

  x <- object@mass[index]

  ## labels
  if (missing(labels)) {
    labels <- round(x=x, digits=digits)
  } else if (!missing(labels) && length(index) != length(labels)) {
    stop("Lenghts of ", sQuote("index"), "/", sQuote("mass"), " and ",
         sQuote("labels"), " have to be equal.")
  }

  if (underline) {
    labels <- as.expression(sapply(labels,
      function(x)substitute(underline(a), list(a=x))))
  }

  if (missing(absoluteVerticalPos)) {
    y <- object@intensity[index] + verticalOffset
  } else {
    y <- absoluteVerticalPos
  }

  if (avoidOverlap) {
    if (srt %% 90L != 0L) {
      stop(sQuote("avoidOverlap = TRUE"), " and ", sQuote("srt != x * 90"),
           " is not supported.")
    }

    ## inspired by Ian Fellows' wordcloud::wordlayout
    p <- .calculateLabelPositions(object, x, y, labels, adj=adj, cex=cex,
                                  srt=srt)

    ## create arrows from label to peak
    arrows(x0=p[, "x"], y0=p[, "y"], x1=x, y1=y, col=arrowCol,
           length=arrowLength, lwd=arrowLwd)
    ## no transparent background
    rect(xleft=p[, "x0"], ybottom=p[, "y0"], xright=p[, "x1"], ytop=p[, "y1"],
         col="white", border=NA, density=-1L)
    x <- p[, "x"]
    y <- p[, "y"]
  }
  text(x=x, y=y, labels=labels, adj=adj, cex=cex, srt=srt, ...)
})

## warpingFunction functions called by determineWarpingFunction
##
## requirements:
##  A function with three arguments: x, difference, ... .
##  This function has to return a function with only an argument (x) which does
##  the warping.

## .warpingFunctionLowess
##  lowess based determination of warping function [default]
##
## params:
##  x: double, original mass
##  d: double, corresponding difference to reference
##  ...: additional arguments possible (see ?lowess for details)
##
## returns:
##  function(x)
##
.warpingFunctionLowess <- function(x, d, ...) {
  lo <- lowess(x=x, y=d, ...)
  approxfun(x=lo$x, y=lo$y, rule=2L)
}

## .warpingFunctionLinear
##  1st order polynomial based determination of warping function
##
## params:
##  x: double, original mass
##  d: double, corresponding difference to reference
##  ...: additional arguments possible (see ?lm for details)
##
## returns:
##  function(x)
##
.warpingFunctionLinear <- function(x, d, ...) {
  l <- lm(y ~ x1, data=list(x1=x, y=d), ...)
  co <- coef(l)
  function(x) { co[1L]+x*co[2L] }
}

## .warpingFunctionQuadratic
##  2nd order polynomial based determination of warping function
##
## params:
##  x: double, original mass
##  d: double, corresponding difference to reference
##  ...: additional arguments possible (see ?lm for details)
##
## returns:
##  function(x)
##
.warpingFunctionQuadratic <- function(x, d, ...) {
  l <- lm(y ~ x1+x2, data=list(x1=x, x2=x*x, y=d), ...)
  co <- coef(l)
  function(x) { co[1L]+x*co[2L]+x*x*co[3L] }
}

## .warpingFunctionCubic
##  3rd order polynomial based determination of warping function
##
## params:
##  x: double, original mass
##  d: double, corresponding difference to reference
##  ...: additional arguments possible (see ?lm for details)
##
## returns:
##  function(x)
##
.warpingFunctionCubic <- function(x, d, ...) {
  l <- lm(y ~ x1+x2+x3, data=list(x1=x, x2=x*x, x3=x*x*x, y=d), ...)
  co <- coef(l)
  function(x) { co[1L]+x*co[2L]+x*x*co[3L]+x*x*x*co[4L] }
}

context("warpingFunctions")

test_that(".warpingFunctionLowess", {
  al <- function(x) {
    l <- lowess(x=x, y=x^2)
    return(approxfun(l$x, l$y))
  }
  expect_equal(MALDIquant:::.warpingFunctionLowess(1:10, (1:10)^2)(1:10),
               al(1:10)(1:10))
})

test_that(".warpingFunctionLinear", {
  expect_equal(MALDIquant:::.warpingFunctionLinear(1:10, rep(1, 10))(1:10),
               rep(1, 10))
  expect_equal(MALDIquant:::.warpingFunctionLinear(1:10, 1:10)(1:10),
               1:10)
})

test_that(".warpingFunctionQuadratic", {
  expect_equal(MALDIquant:::.warpingFunctionQuadratic(1:10, (1:10)^2)(1:10),
               (1:10)^2)
})

test_that(".warpingFunctionCubic", {
  expect_equal(MALDIquant:::.warpingFunctionCubic(1:10, (1:10)^3)(1:10),
               (1:10)^3)
})

## AbstractMassObject
setMethod("trim",
          signature=signature(object="AbstractMassObject", range="numeric"),
          definition=function(object, range) {
  if (length(range) != 2L) {
    stop(sQuote("range"), " has to be a vector of length 2.")
  }

  range <- .reorderRange(range)

  sel <- which(findInterval(object@mass, range, rightmost.closed=TRUE) == 1L)

  if (!length(sel)) {
    warning("The mass range (", paste0(range, collapse=":"),
            ") is outside of the stored mass values. No data points left.")
  }

  object[sel]
})

## list
setMethod("trim",
          signature=signature(object="list", range="numeric"),
          definition=function(object, range, ...) {
  .stopIfNotIsMassObjectList(object)
  .lapply(X=object, FUN=trim, range=range, ...)
})

setMethod("trim",
          signature=signature(object="list", range="missing"),
          definition=function(object, ...) {
  .stopIfNotIsMassObjectList(object)
  range <- .overlap(object)

  if (all(range == 0L)) {
    stop("No overlap found!")
  }

  .lapply(X=object, FUN=trim, range=range, ...)
})

## MassSpectrum
setMethod(f="detectPeaks",
          signature=signature(object="MassSpectrum"),
          definition=function(object, halfWindowSize=20L,
                              method=c("MAD", "SuperSmoother"), SNR=2L, ...) {

  ## empty spectrum?
  if (.isEmptyWarning(object)) {
    return(createMassPeaks(mass=object@mass, intensity=object@intensity,
                           metaData=object@metaData))
  }

  ## estimate noise
  noise <- .estimateNoise(x=object@mass, y=object@intensity, method=method, ...)

  ## find local maxima
  isLocalMaxima <- .findLocalMaximaLogical(object,
                                           halfWindowSize=halfWindowSize)

  ## include only local maxima which are above the noise
  isAboveNoise <- object@intensity > (SNR * noise)

  peakIdx <- which(isAboveNoise & isLocalMaxima)

  createMassPeaks(mass=object@mass[peakIdx],
                  intensity=object@intensity[peakIdx],
                  snr=object@intensity[peakIdx] / noise[peakIdx],
                  metaData=object@metaData)
})

## list
setMethod(f="detectPeaks",
          signature=signature(object="list"),
          definition=function(object, ...) {

  ## test arguments
  .stopIfNotIsMassSpectrumList(object)

  .mapply(detectPeaks, object, ...)
})

## MassPeaks
setMethod(f="monoisotopicPeaks",
          signature=signature(object="MassPeaks"),
          definition=function(object, minCor=0.95, tolerance=1e-4,
                              distance=1.00235, size=3L:10L) {
  object[.monoisotopic(x=mass(object), y=intensity(object),
                       minCor=minCor, tolerance=tolerance,
                       distance=distance, size=size)]
})

## list
setMethod(f="monoisotopicPeaks",
          signature=signature(object="list"),
          definition=function(object, ...) {

  ## test arguments
  .stopIfNotIsMassPeaksList(object)

  .lapply(object, monoisotopicPeaks, ...)
})

.plotMsiSlice <- function(x, center=attr(x, "center"),
                          tolerance=attr(x, "tolerance"),
                          colRampList=list(colorRamp(c("black", "blue", "green",
                                                       "yellow", "red"))),
                          xlab="", ylab="", interpolate=FALSE, scale=TRUE,
                          legend=scale, alignLabels=FALSE, label.cex=0.75,
                          label.col=NULL, ...) {
  stopifnot(is.array(x))

  d <- dim(x)
  tolerance <- rep_len(tolerance, length(center))

  xlim <- c(0L, d[1L] + (2L * d[3L] * legend))
  ylim <- c(0L, d[2L])

  ## prepare plot area
  plot(NA, type="n", xlim=xlim, ylim=ylim,
       axes=FALSE, xlab=xlab, ylab=ylab, asp=1L, ...)

  if (d[3L] > 1L) {
    col <- x

    for (i in seq_len(d[3L])) {
      col[,, i] <- .colorMatrix(.array2matrix(x, z=i), colRamp=colRampList[[i]],
                                scale=scale)
    }

    x <- .combineColorMatrices(x, col)
  } else {
    x <- .colorMatrix(.array2matrix(x), colRamp=colRampList[[1L]], scale=scale)
  }

  ## plot image
  .rasterSlice(x, interpolate=interpolate)

  if (legend) {

    if (!is.null(center)) {
      labels <- .mapply(function(cnt, tol)bquote(.(cnt) %+-% .(tol)),
                        cnt=center, tol=tolerance)
      strh <- max(strheight(labels, cex=label.cex)) * 1.2
    } else {
      labels <- character(d[3L])
      strh <- 0L
    }

    xleft <- xlim[2L] - seq(from=d[3L] * 2L - 1L, to=1L, by=-2L)
    xright <- xleft + 1L
    ybottom <- rep.int(d[3L] * strh, d[3L])
    ytext <- (d[3L] - 1L):0L * strh

    xtext <- if (alignLabels) {
      rep.int(xlim[2L], 3L)
    } else {
      xright
    }

    if (is.null(label.col) && d[3L] == 1L) {
      label.col <- "black"
    } else if (is.null(label.col) && d[3L] > 1L) {
      label.col <- lapply(colRampList, function(x).rgb(x(1L)))
    }

    for (i in seq_len(d[3L])) {
      .msiLegend(xleft=xleft[i], xright=xright[i],
                 ybottom=ybottom[1L], ytop=ylim[2L],
                 colRamp=colRampList[[i]], interpolate=interpolate)
      text(x=xtext[i], y=ytext[i], labels=as.expression(labels[i]),
           col=label.col[[i]], cex=label.cex, adj=c(1L, 0L))
    }
  }
}

.rasterSlice <- function(x, interpolate=FALSE) {
  rasterImage(as.raster(t(x)),
              xleft=0L, xright=nrow(x), ybottom=0L, ytop=ncol(x),
              interpolate=interpolate)
}

.array2matrix <- function(a, z=1L) {
## subset function that preserves a matrix even if x or y 1
## ([,,drop=TRUE]) creates a vector
  d <- dim(a)
  matrix(a[,, z, drop=TRUE], nrow=d[1L], ncol=d[2L])
}

.msiLegend <- function(xleft, xright, ybottom, ytop,
                       colRamp=colorRamp(c("black", "blue", "green", "yellow",
                                           "red")), interpolate=FALSE) {
  gradient <- matrix(.rgb(colRamp(seq.int(1L, 0L, length.out=100L))),
                     ncol=1L)
  rect(xleft=xleft, xright=xright, ybottom=ybottom, ytop=ytop,
       col="black")
  rasterImage(as.raster(gradient),
              xleft=xleft, xright=xright, ybottom=ybottom, ytop=ytop,
              interpolate=interpolate)
}

.colorMatrix <- function(x, colRamp, scale=TRUE) {
  if (scale) {
    x <- x / max(x, na.rm=TRUE)
  }

  notNA <- which(!is.na(x))
  x[notNA] <- .rgb(colRamp(x[notNA]))
  x
}

.combineColorMatrices <- function(x, col) {
  i <- apply(x, 2L, max.col, ties.method="first")
  j <- cbind(x=rep.int(seq_len(nrow(x)), ncol(x)),
             y=rep(seq_len(ncol(x)), each=nrow(x)),
             z=as.vector(i))
  y <- .array2matrix(col)
  y[] <- col[j]
  y
}

.rgb <- function(x) {
  rgb(x, maxColorValue=255L)
}

context(".plotMsiSlice")

test_that(".array2matrix", {
  x1 <- array(1:12, dim=c(x=2, y=3, z=2))
  x2 <- array(1:12, dim=c(1, 12, 1))
  r1 <- matrix(1:6, nrow=2, ncol=3)
  r2 <- matrix(7:12, nrow=2, ncol=3)
  r3 <- matrix(1:12, nrow=1, ncol=12)

  expect_identical(MALDIquant:::.array2matrix(x1), r1)
  expect_identical(MALDIquant:::.array2matrix(x1, z=2), r2)
  expect_identical(MALDIquant:::.array2matrix(x2), r3)
})

test_that(".colorMatrix", {
  x <- matrix(c(NA, 1:8, NA), nrow=2)
  colRamp1 <- colorRamp(c("black", "green"))
  colRamp2 <- function(x)cbind(0, 0, 30*x)
  r1 <- matrix(c(NA, rgb(colRamp1(1:8/8), maxColorValue=255), NA), nrow=2)
  r2 <- matrix(c(NA, rgb(colRamp2(1:8), maxColorValue=255), NA), nrow=2)

  expect_equal(MALDIquant:::.colorMatrix(x, colRamp1), r1)
  expect_equal(MALDIquant:::.colorMatrix(x, colRamp2, scale=FALSE), r2)
})

test_that(".combineColorMatrices", {
  x <- array(c(1:8, 8:1), dim=c(2, 4, 2))
  col <- array(rep(1:2, each=8), dim=c(2, 4, 2))
  r <- matrix(rep(2:1, each=4), nrow=2, ncol=4)

  expect_equal(MALDIquant:::.combineColorMatrices(x, col), r)
})

test_that(".rgb", {
  r <- cbind(1:255, 1:255, 1:255)
  expect_equal(MALDIquant:::.rgb(r), rgb(r, maxColorValue=255))
})

## .reorder
##  reorders mass and intensity values (increasing)
##
## params:
##  object AbstractMass Object
##  warn: throw a warning?
##
## returns:
##  an AbstractMass object
##
.reorder <- function(object, warn=TRUE) {
  if (is.unsorted(object@mass)) {
    if (warn) {
      warning("Mass and intensity values are reordered.")
    }
    i <- sort.int(object@mass, index.return=TRUE)
    object@mass <- i$x
    object@intensity <- object@intensity[i$ix]
  }
  object
}

.isFunctionList <- function(x) {
  if (!is.list(x)) {
    return(FALSE)
  }

  length(x) && all(.unlist(vapply(x, is.function, logical(1L))))
}

.stopIfNotIsFunctionList <- function(x) {
  if (!.isFunctionList(x)) {
    parentCall <- deparse(sys.call(-1L))
    stop(parentCall, " : ", sQuote(deparse(substitute(x))),
         " is no list of functions!", call.=FALSE)
  }
  TRUE
}

## .estimateBaseline
##  estimating the baseline of a spectrum
##
## params:
##  x: vector of x values (mass)
##  y: vector of y values (intensity)
##  method: method to use
##  ...: further arguments passed to "method"
##
## returns:
##  numeric, estimated baseline (y)
##
.estimateBaseline <- function(x, y, method=c("SNIP", "TopHat", "ConvexHull",
                                             "median"), ...) {
  method <- match.arg(method)

  switch(method,
         "SNIP" = {
           .estimateBaselineSnip(x, y, ...)
         },
         "TopHat" = {
           .estimateBaselineTopHat(x, y, ...)
         },
         "ConvexHull" = {
           .estimateBaselineConvexHull(x, y, ...)
         },
         "median" = {
           .estimateBaselineMedian(x, y, ...)
         }
  )
}

## estimateBaselineConvexHull
##  estimate baseline by creating a convex hull
##
##  A. M. Andrew, "Another Efficient Algorithm for Convex Hulls in Two
##  Dimensions", Info. Proc. Letters 9, 216-219 (1979).
##  only calculate lower hull (we don't need the upper one)
##
## params:
##  x: vector of x values
##  y: vector of y values
##
## returns:
##  numeric, estimated baseline (y)
##
.estimateBaselineConvexHull <- function(x, y) {
  .Call(C_lowerConvexHull, x, y)
}

## estimateBaselineMedian
##  estimate baseline by computing moving median
##
## params:
##  x: vector of x values
##  y: vector of y values
##  halfWindowSize: size of local window
##
## returns:
##  numeric, estimated baseline (y)
##
.estimateBaselineMedian <- function(x, y, halfWindowSize=100L) {
  .stopIfNotIsValidHalfWindowSize(halfWindowSize=halfWindowSize, n=length(x))

  as.vector(runmed(y, k=2L * halfWindowSize + 1L))
}

## estimateBaselineSnip
##  estimate baseline by SNIP algorithm
##
##  SNIP algorithm based on:
##  C.G. Ryan, E. Clayton, W.L. Griffin, S.H. Sie, and D.R. Cousens.
##  "Snip, a statistics-sensitive background treatment for the quantitative
##  analysis of pixe spectra in geoscience applications."
##  Nuclear Instruments and Methods in Physics Research Section B:
##  Beam Interactions with Materials and Atoms, 34(3):396-402, 1988.
##  ISSN 0168-583X. doi:10.1016/0168-583X(88)90063-8.
##  URL http://www.sciencedirect.com/science/article/B6TJN-46YSYTJ-30/2/e0d015ceb8ea8a7bc0702a857a19750b
##
##  decreasing clipping window adapted from:
##  M. Morhac. 2009.
##  "An algorithm for determination of peak regions and baseline elimination in
##   spectroscopic data."
##  Nuclear Instruments and Methods in Physics Research Section A:
##  Accelerators, Spectrometers, Detectors and Associated Equipment, 600(2), 478-487.
##  ISSN 0168-9002. doi:10.1016/S0168-9002(97)01023-1.
##  URL http://www.sciencedirect.com/science/article/pii/S0168900297010231
##
## params:
##  x: vector of x values (only needed for create a matrix as return value)
##  y: vector of y values
##  iterations: number of iterations
##  decreasing: use a decreasing clipping window
##
## returns:
##  numeric, estimated baseline (y)
##
.estimateBaselineSnip <- function(x, y, iterations=100L, decreasing=TRUE) {
  .Call(C_snip, y, iterations, decreasing)
}

## estimateBaselineTopHat
##  estimate baseline by TopHat filter (erosion + dilation)
##
## params:
##  x: vector of x values (only needed for create a matrix as return value)
##  y: vector of y values
##  halfWindowSize: size of local window
##
## returns:
##  numeric, estimated baseline (y)
##
.estimateBaselineTopHat <- function(x, y, halfWindowSize=100L) {
  .stopIfNotIsValidHalfWindowSize(halfWindowSize=halfWindowSize, n=length(x))

  .dilation(.erosion(y, halfWindowSize=halfWindowSize),
            halfWindowSize=halfWindowSize)
}

context("estimateBaseline")

s <- createMassSpectrum(mass=1:20, intensity=rep(10:1, 2))

test_that("estimateBaseline throws errors", {
  expect_error(estimateBaseline(s, method="foobar"),
               ".*arg.* should be one of .*SNIP.*, .*ConvexHull.*, .*median.*")
})

test_that("estimateBaseline shows warnings", {
  expect_warning(estimateBaseline(
                   createMassSpectrum(mass=double(), intensity=double()),
                 "empty"))
})

test_that("estimateBaselineConvexHull", {
  b <- c(10:1, rep(1, 10))
  m <- cbind(mass=mass(s), intensity=b)
  expect_identical(MALDIquant:::.estimateBaselineConvexHull(mass(s),
                                                            intensity(s)), b)
  ## user method
  expect_identical(estimateBaseline(s, method="ConvexHull"), m)
})

test_that("estimateBaselineMedian", {
  b <- c(10:4, rep(c(3, 8), each=3), 7:1)
  b1 <- c(10:3, rep(c(2, 9), each=2), 8:1)
  m <- cbind(mass=mass(s), intensity=b)
  expect_identical(MALDIquant:::.estimateBaselineMedian(mass(s), intensity(s),
                                                        1), b1)
  expect_identical(MALDIquant:::.estimateBaselineMedian(mass(s), intensity(s),
                                                        2), b)
  ## user method
  expect_identical(estimateBaseline(s, method="median", halfWindowSize=2), m)

  ## halfWindowSize
  expect_error(MALDIquant:::.estimateBaselineMedian(mass(s), intensity(s), 0),
               "too small")
  expect_error(MALDIquant:::.estimateBaselineMedian(mass(s), intensity(s), 20),
               "too large")
})

test_that("estimateBaselineSnip", {
  b <- list(decreasing=list(b=c(10:1, c(2.5, 4.0, 5.5), 7:1),
                            b100=c(10:1, rep(c(1.25, 1.5, 1.75, 1.375, 1),
                                             times=2))),
            increasing=list(b=c(10:1, 5:4, 5.5, 7:1),
                            b100=c(10:1, 3.75, 3.375, 3:1, 2.5, 4:1)))

  ## test default decreasing argument
  expect_identical(MALDIquant:::.estimateBaselineSnip(mass(s), intensity(s), 2),
                   b$decreasing$b)

  for (j in seq(along=b)) {
    d <- names(b)[j] == "decreasing"

    expect_identical(MALDIquant:::.estimateBaselineSnip(mass(s), intensity(s),
                                                        2, decreasing=d),
                     b[[j]]$b)
    expect_equal(MALDIquant:::.estimateBaselineSnip(mass(s), intensity(s),
                                                    decreasing=d),
                 b[[j]]$b100)

    ## user method
    m <- cbind(mass=mass(s), intensity=b[[j]]$b)
    m100 <- cbind(mass=mass(s), intensity=b[[j]]$b100)
    expect_identical(estimateBaseline(s, method="SNIP", iterations=2,
                                      decreasing=d), m)
    expect_identical(estimateBaseline(s, iterations=2, decreasing=d), m)
    expect_equal(estimateBaseline(s, decreasing=d), m100)
  }
})

test_that("estimateBaselineTopHat", {
  b <- c(rep(8, 3), 7:1, rep(6, 5), 5:1)
  b1 <- c(rep(9, 2), 8:1, rep(8, 3), 7:1)
  m <- cbind(mass=mass(s), intensity=b)
  expect_identical(MALDIquant:::.estimateBaselineTopHat(mass(s), intensity(s),
                                                        1), b1)
  expect_identical(MALDIquant:::.estimateBaselineTopHat(mass(s), intensity(s),
                                                        2), b)

  ## user method
  expect_identical(estimateBaseline(s, method="TopHat", halfWindowSize=2), m)

  ## halfWindowSize
  expect_error(MALDIquant:::.estimateBaselineTopHat(mass(s), intensity(s), 0),
               "too small")
  expect_error(MALDIquant:::.estimateBaselineTopHat(mass(s), intensity(s), 20),
               "too large")

})

## AbstractMassObject
setGeneric("plotMsiSlice", function(x, ...) standardGeneric("plotMsiSlice"))
setGeneric(".prepareShow", function(object) standardGeneric(".prepareShow"))
setGeneric(
    "transformIntensity",
    function(object, ...) standardGeneric("transformIntensity")
)
setGeneric(
    ".transformIntensity",
    function(object, ...) standardGeneric(".transformIntensity")
)
setGeneric("trim", function(object, range, ...) standardGeneric("trim"))

## get/set slots
setGeneric("mass", function(object, ...) standardGeneric("mass"))
setGeneric("mass<-", function(object, value) standardGeneric("mass<-"))

# from ProtGenerics (same as mass)
setGeneric("mz", function(object, ...) standardGeneric("mz"))
setGeneric("mz<-", function(object, value) standardGeneric("mz<-"))

setGeneric("intensity", function(object, ...) standardGeneric("intensity"))
setGeneric(
    "intensity<-",
    function(object, value) standardGeneric("intensity<-")
)
setGeneric("isEmpty", function(x) standardGeneric("isEmpty"))
setGeneric(".isEmptyWarning", function(x) standardGeneric(".isEmptyWarning"))
setGeneric("metaData", function(object) standardGeneric("metaData"))
setGeneric("metaData<-", function(object, value) standardGeneric("metaData<-"))
setGeneric("coordinates", function(object, ...) standardGeneric("coordinates"))
setGeneric(
    "coordinates<-",
    function(object, value) standardGeneric("coordinates<-")
)
## end of AbstractMassObject

## MassSpectrum
setGeneric(
    "calibrateIntensity",
    function(object, ...) standardGeneric("calibrateIntensity")
)
setGeneric("detectPeaks", function(object, ...) standardGeneric("detectPeaks"))
setGeneric(
    "estimateBaseline",
    function(object, method=c("SNIP", "ConvexHull", "Median"), ...)
        standardGeneric("estimateBaseline"))
setGeneric(
    "estimateNoise", function(object, ...) standardGeneric("estimateNoise")
)
setGeneric(
    ".findLocalMaxima",
    function(object, halfWindowSize=20L) standardGeneric(".findLocalMaxima")
)
setGeneric(
    ".findLocalMaximaLogical",
    function(object, halfWindowSize=20L)
        standardGeneric(".findLocalMaximaLogical")
)
setGeneric("isRegular", function(object, ...) standardGeneric("isRegular"))
setGeneric(
    "removeBaseline", function(object, ...) standardGeneric("removeBaseline")
)
setGeneric(
    "smoothIntensity", function(object, ...) standardGeneric("smoothIntensity")
)
setGeneric(
    "totalIonCurrent", function(object) standardGeneric("totalIonCurrent")
)
## end of MassSpectrum

## MassPeaks
setGeneric("labelPeaks", function(object, ...) standardGeneric("labelPeaks"))
setGeneric(
    "monoisotopicPeaks",
    function(object, ...) standardGeneric("monoisotopicPeaks")
)
setGeneric("snr", function(object) standardGeneric("snr"))
## end of MassPeaks

## convert MassPeaks into MassSpectrum objects
##
## returns:
##  a MassSpectrum object

setAs(from="MassPeaks", to="MassSpectrum",
      function (from)createMassSpectrum(mass=from@mass,
                                        intensity=from@intensity,
                                        metaData=from@metaData))

## AbstractMassObject
setMethod(f="mass",
          signature=signature(object="AbstractMassObject"),
          definition=function(object, ...) {

  object@mass
})

## AbstractMassObject
setReplaceMethod(f="mass",
          signature=signature(object="AbstractMassObject", value="numeric"),
          definition=function(object, value) {

  if (length(object@mass) == length(value)) {
    object@mass <- as.double(value)
  } else {
    stop("Lengths of mass (", length(object@mass), ") and value (",
         length(value), ") have to be equal.")
  }
  object
})

## AbstractMassObject
setMethod(f="lines",
    signature=signature(x="AbstractMassObject"),
    definition=function(x,
        type=ifelse(isMassPeaks(x), "h", "l"),
        ...) {

    lines(x=x@mass, y=x@intensity, type, ...)
})

## .irregularScore
##  calculate frequency of irregular data
##
## params:
##  x: double
##
## returns:
##  double, frequency of irregular data
##
.irregularScore <- function(x) {
  d <- diff(x)
  d <- d[-1L] < head(d, -1L)

  mean(d)
}

## binPeaks
##  binning peaks by splitting at the largest gap
##  This is a wrapper function around .binPeaks which prepares the peak list
##  before and recreates a correct peak list after binning.
##
## params:
##  l: list of MassPeaks objects
##  method: character, grouper to used (strict: don't allow multiple peaks of
##          the same sample in the same bin, relaxed: allow them, reference:
##          group to reference sample == the first one)
##  tolerance: double, maximal deviation of a peak position to be
##             considered as same peak
##
## returns:
##  a list of adjusted MassPeaks objects
##
binPeaks <- function(l, method=c("strict", "relaxed", "reference"),
                     tolerance=0.002) {

  ## test arguments
  .stopIfNotIsMassPeaksList(l)

  method <- match.arg(method)

  ## store original mass sample number/id
  nn <- lengths(l)
  nonEmpty <- nn != 0L
  samples <- rep.int(seq_along(l), nn)

  ## fetch all mass
  mass <- unname(.unlist(lapply(l[nonEmpty], function(x)x@mass)))

  ## fetch all intensities
  intensities <- .unlist(lapply(l[nonEmpty], function(x)x@intensity))

  ## fetch all snr
  snr <- .unlist(lapply(l[nonEmpty], function(x)x@snr))

  ## sort values by mass
  s <- sort.int(mass, index.return=TRUE)

  mass <- s$x
  intensities <- intensities[s$ix]
  snr <- snr[s$ix]
  samples <- samples[s$ix]

  ## select grouper
  grouper <- switch(method,
            "strict"  = {
              .grouperStrict
            },
            "relaxed" = {
              .grouperRelaxed
            },
            "reference" = {
              .grouperRelaxedHighestAtReference
            }
  )

  ## binning
  mass <- .binPeaks(mass=mass, intensities=intensities, samples=samples,
                    tolerance=tolerance, grouper=grouper)

  ## resort mass (order could change if "relaxed"/"reference" is used)
  if (method != "strict") {
    s <- sort.int(mass, index.return=TRUE)
    mass <- s$x
    intensities <- intensities[s$ix]
    snr <- snr[s$ix]
    samples <- samples[s$ix]
  }

  ## group mass/intensities/snr by sample ids
  lIdx <- split(seq_along(mass), samples)

  ## create adjusted peak list
  l[nonEmpty] <- .mapply(FUN=function(p, i) {
    p@mass <- mass[i]
    p@intensity <- intensities[i]
    p@snr <- snr[i]
    p
  }, p=l[nonEmpty], i=lIdx)

  l
}

## .binPeaks
##  binning peaks by splitting at the largest gap
##
## params:
##  mass: double, sorted mass
##  intensities: double, corresponding intensities
##  samples: double, corresponding sample id numbers
##  tolerance: double, maximal deviation of a peak position to be
##             considered as same peak
##  grouper: grouping function
##  ...: arguments passed to grouping function
##
## returns:
##  vector (double) of modified mass
##
.binPeaks <- function(mass, intensities, samples, tolerance,
            grouper=.grouperStrict, ...) {
  n <- length(mass)

  ## calculate difference
  d <- diff(mass)

  ## grouper function
  grouper <- match.fun(grouper)

  ## stack based implementation taken from
  ## caMassClass 1.9 R/msc.peaks.clust.R written by
  ## Jarek Tuszynski <jaroslaw.w.tuszynski@saic.com>
  ## it is a lot of faster than recursion

  ## store boundaries in a stack
  nBoundaries <- max(20L, floor(3L * log(n)))
  boundary <- list(left=double(nBoundaries), right=double(nBoundaries))

  currentBoundary <- 1L
  boundary$left[currentBoundary] <- 1L
  boundary$right[currentBoundary] <- n

  ## workhorse loop
  while (currentBoundary > 0L) {
    ## find largest gap
    left <- boundary$left[currentBoundary]
    right <- boundary$right[currentBoundary]
    currentBoundary <- currentBoundary - 1L
    gaps <- d[left:(right - 1L)]

    gapIdx <- which.max(gaps) + left - 1L

    ## left side
    l <- grouper(mass=mass[left:gapIdx],
                 intensities=intensities[left:gapIdx],
                 samples=samples[left:gapIdx],
                 tolerance=tolerance, ...)
    ## further splitting needed?
    if (is.na(l[1L])) {
      currentBoundary <- currentBoundary + 1L
      boundary$left[currentBoundary] <- left
      boundary$right[currentBoundary] <- gapIdx
    } else {
      mass[left:gapIdx] <- l
    }

    ## right side
    r <- grouper(mass=mass[(gapIdx + 1L):right],
                 intensities=intensities[(gapIdx + 1L):right],
                 samples=samples[(gapIdx + 1L):right],
                 tolerance=tolerance, ...)
    ## further splitting needed?
    if (is.na(r[1L])) {
      currentBoundary <- currentBoundary + 1L
      boundary$left[currentBoundary] <- gapIdx + 1L
      boundary$right[currentBoundary] <- right
    } else {
      mass[(gapIdx + 1L):right] <- r
    }

    ## stack size have to be increased?
    ## (should rarely happen because recursion deep is mostly < 20)
    if (currentBoundary == nBoundaries) {
      nBoundaries <- floor(nBoundaries * 1.5)
      boundary$left <- c(boundary$left,
                         double(nBoundaries - currentBoundary))
      boundary$right <- c(boundary$right,
                          double(nBoundaries - currentBoundary))
    }
  }
  mass
}

## .scalingFactor
## calculate scaling factor for TIC/median ranged calibration
##
## params:
##  object: AbstractMassObject
##  method: character
##  range: double
##
## returns:
##  double
##
.scalingFactor <- function(object, method=c("TIC", "median"), range) {

  method <- match.arg(method)

  if (!missing(range)) {
    object <- trim(object, range=range)
  }

  switch(method,
    "TIC" = {
      totalIonCurrent(object)
    },
    "median" = {
      median(object@intensity)
    }
  )
}

## .calibrateIntensitySimple
## calibrate intensity values by offset and scaling factor
##
## params:
##  y: double, intensity values
##  offset: double
##  scaling: double
##
## returns:
##  double, calibrated intensity values
##
.calibrateIntensitySimple <- function(y, offset=0L, scaling=1L) {
  if (scaling == 0L) {
    warning("Scaling factor is zero. No calibration applied.")
    y
  } else {
    (y - offset) / scaling
  }
}

## .calibrateProbabilisticQuotientNormalization
## calibrate intensity values by Probabilistic Quotient Normalization
##
## F. Dieterle, A. Ross, G. Schlotterbeck, and Hans Senn.
## "Probabilistic quotient normalization as robust method to account for
## dilution of complex biological mixtures. Application in 1H NMR
## metabonomics."
## Analytical Chemistry 78, no. 13 (2006): 4281-4290.
##
## 1. Perform an integral normalization (typically a constant
##    integral of 100 is used).
## 2. Choose/calculate the reference spectrum (the best approach
##    is the calculation of the median spectrum of control samples).
## 3. Calculate the quotients of all variables of interest of the test
##    spectrum with those of the reference spectrum.
## 4. Calculate the median of these quotients.
## 5. Divide all variables of the test spectrum by this median.
##
## params:
##  l: list of MassSpectrum objects
##  range: double
##
## returns:
##  list of calibrated MassSpectrum objects
##
.calibrateProbabilisticQuotientNormalization <- function(l, range) {
  ## 1. integral normalization (==TIC)
  l <- calibrateIntensity(l, method="TIC", range=range)
  ## 2. median reference spectrum
  if (missing(range)) {
    reference <- .averageMassSpectra(l, fun=.colMedians, mergeMetaData=FALSE)
  } else {
    reference <- .averageMassSpectra(trim(l, range=range), fun=.colMedians,
                                     mergeMetaData=FALSE)
  }

  lapply(l, function(x) {
    ## 3. quotient calculation
    q <- approxfun(x)(reference@mass) / reference@intensity
    ## 4. median
    m <- median(q, na.rm=TRUE)
    ## 5. divide by median
    .transformIntensity(x, fun=.calibrateIntensitySimple, offset=0L, scaling=m)
  })
}

## MassSpectrum
setMethod(f="removeBaseline",
          signature=signature(object="MassSpectrum"),
          definition=function(object,
                              method=c("SNIP", "TopHat", "ConvexHull",
                                       "median"),
                              ...) {
  ## empty spectrum?
  if (.isEmptyWarning(object)) {
    return(object)
  }

  ## estimate baseline
  baseline <- .estimateBaseline(x=object@mass, y=object@intensity,
                                method=method, ...)

  ## substract baseline
  object@intensity <- object@intensity - baseline

  object
})

## list
setMethod(f="removeBaseline",
          signature=signature(object="list"),
          definition=function(object, ...) {

  ## test arguments
  .stopIfNotIsMassSpectrumList(object)

  .mapply(removeBaseline, object, ...)
})

## .prepareShowGroupName
##  prepares a group name for show (appends numbers to name if needed)
##
## params:
##  x: vector
##  name: group name
##
## returns:
##
.prepareShowGroupName <- function(x, name) {
  if (!is.null(x)) {
    n <- length(x)

    if (n > 1L) {
      name <- paste0(name, seq_len(n))
    }

  } else {
    name <- NULL
  }

  name
}

## grouper functions called by .binPeaks

## .grouperStrict
##  strict grouping function
##  Don't allow peaks of one sample in the same bin.
##
## params:
##  mass: double, sorted mass
##  intensities: double, corresponding intensities
##  samples: double, corresponding sample id numbers
##  tolerance: double, maximal deviation of a peak position to be
##             considered as same peak
##
## returns:
##  NA if further splitting is needed
##  meanMass (double) if all criteria are matched
##
.grouperStrict <- function(mass, intensities, samples, tolerance) {
  ## don't accept two or more peaks of the same sample
  if (anyDuplicated(samples)) {
    return(NA)
  }

  meanMass <- mean(mass)

  ## all peaks in range?
  if (any(abs(mass - meanMass) / meanMass > tolerance)) {
    return(NA)
  }

  meanMass
}

## .grouperRelaxed
##  relaxed grouping function (more than one peak of one sample per bin possible)
##  choose highest peak in range.
##
## params:
##  mass: double, sorted mass
##  intensities: double, corresponding intensities
##  samples: double, corresponding sample id numbers
##  tolerance: double, maximal deviation of a peak position to be
##             considered as same peak
##  meanMass: double, mean of mass (new peak position)
##
## returns:
##  NA if further splitting is needed
##  meanMass (double) if all criteria are matched
##
.grouperRelaxed <- function(mass, intensities, samples, tolerance) {
  meanMass <- mean(mass)

  ## all peaks in range?
  if (any(abs(mass - meanMass) / meanMass > tolerance)) {
    return(NA)
  }

  ## choose highest peak in duplicates
  if (anyDuplicated(samples)) {
    s <- sort.int(intensities, decreasing=TRUE, index.return=TRUE)
    samples <- samples[s$ix]

    noDup <- !duplicated(samples)

    noDup[s$ix] <- noDup

    ## replace mass corresponding to highest intensity
    mass[noDup] <- mean(mass[noDup])

    return(mass)
  }

  meanMass
}

## .grouperRelaxedHighestAtReference
##  relaxed grouping function (more than one peak of one sample per bin possible)
##  Choose highest test sample peaks in range around a reference peak.
##
## params:
##  mass: double, sorted mass
##  intensities: double, corresponding intensities
##  samples: double, corresponding sample id numbers (1==reference)
##  tolerance: double, maximal deviation of a peak position to be
##             considered as same peak
##  nomatch: return value if no reference peak found, mass for original mass, 0L
##           for `determineWarpingFunctions`
##
## returns:
##  NA if further splitting is needed
##  meanMass (double) if all criteria are matched else 0
##
.grouperRelaxedHighestAtReference <- function(mass, intensities, samples,
                                              tolerance, nomatch=mass) {
  ## any reference peaks in current samples?
  ref <- samples == 1L
  nRef <- sum(ref)
  if (nRef == 0L) {
    ## no reference peak
    return(nomatch)
  } else if (nRef > 1L) {
    ## too many reference peaks => further splitting needed
    return(NA)
  }

  ## only one mass should left as reference mass
  meanMass <- mass[ref]

  ## all peaks in range?
  if (any(abs(mass - meanMass) / meanMass > tolerance)) {
    return(NA)
  }

  ## choose highest peak in duplicates
  if (anyDuplicated(samples)) {
    s <- sort.int(intensities, decreasing=TRUE, index.return=TRUE)
    samples <- samples[s$ix]
    noDup <- !duplicated(samples)
    noDup[s$ix] <- noDup

    ## replace mass corresponding to highest intensity
    mass[] <- nomatch
    mass[noDup] <- meanMass

    return(mass)
  }

  meanMass
}

context("grouper")

test_that("grouperStrict", {
  expect_true(is.na(MALDIquant:::.grouperStrict(mass=c(1, 2),
                                                intensities=c(1, 2),
                                                samples=c(1, 1),
                                                tolerance=0.01)))
  expect_true(is.na(MALDIquant:::.grouperStrict(mass=c(1, 2, 2.1, 2.01),
                                                intensities=rep(1, 4),
                                                samples=1:4,
                                                tolerance=0.01)))
  expect_true(MALDIquant:::.grouperStrict(mass=c(2, 2.02, 2.01),
                                          intensities=rep(1, 3),
                                          samples=1:4,
                                          tolerance=0.01) == 2.01)
})

test_that("grouperRelaxed", {
  expect_true(is.na(MALDIquant:::.grouperRelaxed(mass=c(1, 2, 2.1, 2.01),
                                                 intensities=rep(1, 4),
                                                 samples=1:4,
                                                 tolerance=0.01)))
  expect_equal(MALDIquant:::.grouperRelaxed(mass=c(2.01, 2.03, 2.04, 2.01),
                                            intensities=c(2, 1, 1, 1),
                                            samples=c(1, 1, 2, 3),
                                            tolerance=0.1),
               c(2.02, 2.03, 2.02, 2.02))
  expect_equal(MALDIquant:::.grouperRelaxed(mass=c(1.009, 1.01, 1, 1.03),
                                            intensities=c(2, 1, 1, 1),
                                            samples=c(1, 1, 2, 3),
                                            tolerance=0.1),
               c(1.013, 1.01, 1.013, 1.013))
})

test_that("grouperRelaxedHighestAtReference", {
  expect_identical(MALDIquant:::.grouperRelaxedHighestAtReference(
                      mass=1:5, intensities=rep(1, 5), samples=2:6,
                      tolerance=0.01), 1:5)
  expect_identical(MALDIquant:::.grouperRelaxedHighestAtReference(
                      mass=1:5, intensities=rep(1, 5), samples=2:6,
                      tolerance=0.01, nomatch=0L), 0L)
  expect_true(is.na(MALDIquant:::.grouperRelaxedHighestAtReference(
                      mass=1:5, intensities=rep(1, 5), samples=c(1, 1:4),
                      tolerance=0.01)))
  expect_equal(MALDIquant:::.grouperRelaxedHighestAtReference(
                  mass=c(2.01, 2.03, 2.04, 2.01), intensities=c(1, 2, 1, 1),
                  samples=c(1, 2, 2, 3), tolerance=0.1, nomatch=0L),
               c(2.01, 2.01, 0L, 2.01))
  expect_equal(MALDIquant:::.grouperRelaxedHighestAtReference(
                  mass=c(2.01, 2.03, 2.04, 2.01), intensities=c(1, 2, 1, 1),
                  samples=c(1, 2, 2, 3), tolerance=0.1),
               c(2.01, 2.01, 2.04, 2.01))
})

## .movingAverage
##  runs a simple 2-side moving average.
##
## params:
##  y: double, intensity values
##  halfWindowSize: integer, half window size
##  weighted: boolean, if TRUE then applies weighted average,
##      otherwise unweighted average.
## returns:
##  double
##
.movingAverage <- function(y, halfWindowSize=2L, weighted=FALSE) {
  .stopIfNotIsValidHalfWindowSize(halfWindowSize, n=length(y))

  windowSize <- 2L * halfWindowSize + 1L

  if (weighted) {
    weights <- 1 / 2^abs(-halfWindowSize:halfWindowSize)
  } else {
    weights <- rep.int(1L, windowSize)
  }
  .filter(y, hws=halfWindowSize,
          coef=matrix(weights / sum(weights),
                      nrow=windowSize, ncol=windowSize, byrow=TRUE))
}

## .savitzkyGolay
##  runs a savitzky golay filter
##
## Savitzky, A., & Golay, M. J. (1964). Smoothing and differentiation of data
## by simplified least squares procedures. Analytical chemistry, 36(8), 1627-1639.
##
## params:
##  y: double, intensity values
##  halfWindowSize: integer, half window size.
##  polynomialOrder: integer, polynomial order of sg-filter
##
## returns:
##  double
##
.savitzkyGolay <- function(y, halfWindowSize=10L, polynomialOrder=3L) {
  .stopIfNotIsValidHalfWindowSize(halfWindowSize, n=length(y))

  windowSize <- 2L * halfWindowSize + 1L

  if (windowSize < polynomialOrder) {
    stop("The window size has to be larger than the polynomial order.")
  }
  .filter(y, hws=halfWindowSize,
          coef=.savitzkyGolayCoefficients(m=halfWindowSize, k=polynomialOrder))
}

## .savitzkyGolayCoefficients
##
## Savitzky, A., & Golay, M. J. (1964). Smoothing and differentiation of data
## by simplified least squares procedures. Analytical chemistry, 36(8), 1627-1639.
##
## Implementation based on:
## Steinier, J., Termonia, Y., & Deltour, J. (1972). Comments on Smoothing and
## differentiation of data by simplified least square procedure.
## Analytical Chemistry, 44(11), 1906-1909.
##
## Implemention of left/right extrema based on:
## sgolay in signal 0.7-3/R/sgolay.R by Paul Kienzle <pkienzle@users.sf.net>
## modified by Sebastian Gibb <mail@sebastiangibb.de>
##
## params:
##  m: integer, half window size
##  k: integer, polynomial order (k == 0 = moving average)
.savitzkyGolayCoefficients <- function(m, k=3L) {
  k <- 0L:k
  nm <- 2L * m + 1L
  nk <- length(k)
  K <- matrix(k, nrow=nm, ncol=nk, byrow=TRUE)

  ## filter is applied to -m:m around current data point
  ## to avoid removing (NA) of left/right extrema
  ## lhs: 0:2*m
  ## rhs: (n-2m):n

  ## filter matrix contains 2*m+1 rows
  ## row 1:m == lhs coef
  ## row m+1 == typical sg coef
  ## row (n-m-1):n == rhs coef
  F <- matrix(double(), nrow=nm, ncol=nm)
  for (i in seq_len(m + 1L)) {
    M <- matrix(seq_len(nm) - i, nrow=nm, ncol=nk, byrow=FALSE)
    X <- M^K
    T <- solve(t(X) %*% X) %*% t(X)
    F[i, ] <- T[1L, ]
  }
  ## rhs (row (n-m):n) are equal to reversed lhs
  F[(m + 2L):nm, ] <- rev(F[seq_len(m), ])

  F
}

## .filter
##  remove time series attributes and NA at left/right extrema
.filter <- function(x, hws, coef) {
  n <- length(x)
  w <- 2L * hws + 1L
  y <- stats::filter(x=x, filter=coef[hws + 1L, ], sides=2L)
  attributes(y) <- NULL

  ## fix left/right extrema
  y[seq_len(hws)] <- head(coef, hws) %*% head(x, w)
  y[(n - hws + 1L):n] <- tail(coef, hws) %*% tail(x, w)
  y
}

context("smoothingFilters")

test_that(".movingAverage", {
  values <- c(rep(3, 3), 4:7, rep(8, 3))
  expect_equal(MALDIquant:::.movingAverage(1:10, halfWindowSize=2), values)
  expect_equal(MALDIquant:::.movingAverage(1:10, halfWindowSize=2,
                weighted=TRUE), values)

  values <- c(rep(4, 4), 5:6, rep(7, 4))
  expect_equal(MALDIquant:::.movingAverage(1:10, 3), values)
  expect_equal(MALDIquant:::.movingAverage(1:10, 3, weighted=TRUE), values)

  values <- c(rep(1, 4), 6, 20, 6, rep(1, 4))
  result <- c(rep(c(2, 5.8, 6.8, 5.8, 2), c(3, 1, 3, 1, 3)))
  resultWeighted <- c(rep(1.5, 3), 3.9, 7.3, 10.6, 7.3, 3.9, rep(1.5, 3))
  expect_equal(MALDIquant:::.movingAverage(values, halfWindowSize=2,
               weighted=FALSE), result)
  expect_equal(MALDIquant:::.movingAverage(values, halfWindowSize=2,
               weighted=TRUE), resultWeighted)
})

test_that(".movingAverage throws errors", {
  expect_error(MALDIquant:::.movingAverage(1:10, halfWindowSize=0),
               "too small")
  expect_error(MALDIquant:::.movingAverage(1:10, halfWindowSize=100),
               "too large")
})

test_that(".savitzkyGolayCoefficients", {
  values <- list(rep(1, 5)/5,
                 c(-3, 12, 17, 12, -3)/35,
                 c(-2, 3, 6, 7, 6, 3, -2)/21,
                 c(-21, 14, 39, 54, 59, 54, 39, 14, -21)/231,
                 c(5, -30, 75, 131, 75, -30, 5)/231,
                 c(15, -55, 30, 135, 179, 135, 30, -55, 15)/429)
  hws <- c(2, 2, 3, 4, 3, 4)
  order <- c(0, 2, 3, 3:5)

  for (i in seq(along=values)) {
    expect_equal(.savitzkyGolayCoefficients(m=hws[i], k=order[i])[hws[i]+1, ],
                 values[[i]])
  }
})

test_that(".savitzkyGolay", {
  expect_equal(MALDIquant:::.savitzkyGolay(1:10, halfWindowSize=2,
                                           polynomialOrder=0),
               MALDIquant:::.movingAverage(1:10, halfWindowSize=2))

  values <- c(8L, 1L, 7L, 6L, 3L, 13L, 5L, 2L, 19L, 11L, 15L, 18L, 9L, 10L,
              20L, 12L, 17L, 14L, 16L, 4L)

  result <- c(7.64285714285716, 2.42857142857143, 4.85714285714286,
              5.14285714285714, 6.94285714285714, 8.37142857142857,
              5.68571428571428, 7.14285714285714, 11.9714285714286,
              15.2857142857143, 14.8285714285714, 15.1714285714286,
              10.9714285714286, 12.2285714285714, 15.0285714285714,
              16.4571428571429, 14.0857142857143, 16.7428571428571,
              14.1714285714286, 4.45714285714283)
  expect_equal(MALDIquant:::.savitzkyGolay(values, halfWindowSize=2,
                                           polynomialOrder=3), result)
})

test_that(".savitzkyGolay throws errors", {
  expect_error(MALDIquant:::.savitzkyGolay(1:10, halfWindowSize=0,
                                           polynomialOrder=3), "too small")
  expect_error(MALDIquant:::.savitzkyGolay(1:10, halfWindowSize=100,
                                           polynomialOrder=3), "too large")
  expect_error(MALDIquant:::.savitzkyGolay(1:10, halfWindowSize=2,
                                           polynomialOrder=10),
               "The window size has to be larger than the polynomial order.")
})

## MassSpectrum
setMethod(f="approxfun",
          signature=signature(x="MassSpectrum"),
          definition=function(x, y=NULL, method="linear", yleft, yright,
                              rule=1L,  f=0L, ties=mean) {
  if (isEmpty(x)) {
    function(x)rep.int(NA, length(x))
  } else {
    approxfun(x=x@mass, y=x@intensity, method=method,
              yleft=yleft, yright=yright, rule=rule, f=f, ties=ties)
  }
})

## findEmptyMassObjects
##  find empty AbstractMassObject objects in a MassObjectsList
##
## params:
##  l: list of AbstractMassObject objects
##
## returns:
##  a vector of indices refer to empty AbstractMassObject objects
##
findEmptyMassObjects <- function(l) {

  .stopIfNotIsMassObjectList(l)

  isEmpty <- vapply(l, isEmpty, logical(1L))

  which(isEmpty)
}

## warpMassSpectra
##  warp MassSpectrum objects
##
## params:
##  l: list of MassSpectrum objects
##  w: list of warping functions determined by determineWarpingFunctions
##  emptyNoMatches: logical, if TRUE mismatches (warping function NA)
##
## returns:
##  a list of warped MassSpectrum objects
##
warpMassSpectra <- function(l, w, emptyNoMatches=FALSE) {
  .stopIfNotIsMassSpectrumList(l)

  .warp(l, w, emptyNoMatches=emptyNoMatches)
}

## warpMassPeaks
##  warp MassPeaks objects
##
## params:
##  l: list of MassPeaks objects
##  w: list of warping functions determined by determineWarpingFunctions
##  emptyNoMatches: logical, if TRUE mismatches (warping function NA)
##
## returns:
##  a list of warped MassPeaks objects
##
warpMassPeaks <- function(l, w, emptyNoMatches=FALSE) {
  .stopIfNotIsMassPeaksList(l)

  .warp(l, w, emptyNoMatches=emptyNoMatches)
}


## .warp
##  .warp bstractMassObject objects
##
## params:
##  l: list of AbstractMassObject objects
##  w: list of warping functions determined by determineWarpingFunctions
##  emptyNoMatches: logical, if TRUE mismatches (warping function NA)
##
## returns:
##  a list of warped AbstractMassObject objects
##
.warp <- function(l, w, emptyNoMatches=FALSE) {
  notNa <- !is.na(w)
  wl <- w[notNa]
  ml <- l[notNa]

  if (length(wl)) {
    .stopIfNotIsFunctionList(wl)
  }

  l[notNa] <- .mapply(function(m, wf) {
           m@mass <- m@mass + wf(m@mass)
           m
  }, m=ml, wf=wl)
  if (emptyNoMatches) {
      l[!notNa] <- lapply(l[!notNa], function(m) { m@intensity[] <- 0; m })
  }
  l
}

context("warp")

s <- createMassSpectrum(mass=1:10, intensity=1:10)
p <- createMassPeaks(mass=1:10, intensity=1:10, snr=1:10)

test_that("warpMassSpectra throws errors", {
  expect_error(warpMassSpectra(p, function(x)1),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(warpMassSpectra(s, function(x)1),
               "no list of MALDIquant::MassSpectrum objects")
  expect_error(warpMassSpectra(list(s), 1),
               "no list of functions")
})

test_that("warpMassSpectra", {
  expect_equal(warpMassSpectra(list(s, s), list(function(x)1, function(x)x+1)),
               list(createMassSpectrum(mass=2:11, intensity=1:10),
                    createMassSpectrum(mass=seq(3, 21, by=2), intensity=1:10)))
})

test_that("warpMassPeaks throws errors", {
  expect_error(warpMassPeaks(p, function(x)1),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(warpMassPeaks(s, function(x)1),
               "no list of MALDIquant::MassPeaks objects")
  expect_error(warpMassPeaks(list(p), 1),
               "no list of functions")
})

test_that("warpMassPeaks", {
  expect_equal(warpMassPeaks(list(p, p), list(function(x)1, function(x)x+1)),
               list(createMassPeaks(mass=2:11, intensity=1:10, snr=1:10),
                    createMassPeaks(mass=seq(3, 21, by=2), intensity=1:10,
                                    snr=1:10)))
  expect_equal(
    warpMassPeaks(
      list(p, p, p), list(function(x)1, NA, function(x)x+1), emptyNoMatches=TRUE
    ),
    list(
      createMassPeaks(mass=2:11, intensity=1:10, snr=1:10),
      createMassPeaks(mass=1:10, intensity=rep(0, 10), snr=1:10),
      createMassPeaks(mass=seq(3, 21, by=2), intensity=1:10, snr=1:10)
    )
  )
})

test_that(".warp", {
  expect_equal(MALDIquant:::.warp(list(s, s),
                                  list(function(x)1, function(x)x+1)),
               list(createMassSpectrum(mass=2:11, intensity=1:10),
                    createMassSpectrum(mass=seq(3, 21, by=2), intensity=1:10)))
  expect_equal(
    MALDIquant:::.warp(
      list(p, p, p), list(function(x)1, NA, function(x)x+1), emptyNoMatches=TRUE
    ),
    list(
      createMassPeaks(mass=2:11, intensity=1:10, snr=1:10),
      createMassPeaks(mass=1:10, intensity=rep(0, 10), snr=1:10),
      createMassPeaks(mass=seq(3, 21, by=2), intensity=1:10, snr=1:10)
    )
  )
})

## .calculateLabelPositions
##  calculate label positions to avoid collisions
##
## params:
##  object: a single MassPeaks object
##  x: original x coordinates for labels
##  y: original y coordinates for labels
##  peakLabels: peak labels
##  adj: text alignment ?par("adj")
##  cex: font size ?par("cex")
##  srt: rotation in degree
##  maxSteps: max tries to avoid collisons
##
## returns:
##  a matrix of coordinates
##
.calculateLabelPositions <- function(object, x, y, peakLabels, adj, cex, srt,
                                     maxSteps=100L) {
  ## start with smallest peak
  i <- sort.int(y, index.return=TRUE)$ix

  ## calculate label rectangles
  rects <- .textLabelRects(x[i], y[i], peakLabels[i], adj=adj, cex=cex, srt=srt)

  ## move rectangles around to avoid collisons
  for (j in seq_along(x)) {
    rects[j, ] <- .testLabelOverlap(object, rects, currentIndex=j,
                                    maxSteps=maxSteps)
  }

  ## undo sorting
  rects[i, ] <- rects

  rects
}

## .testLabelOverlap
##  try to avoid overlap/collisions
##
## params:
##  object: a single MassPeaks object
##  rects: a matrix of coordinates (created by .textRects)
##  currentIndex: which coordinates should moved around
##  maxSteps: max tries to avoid collisons
##
## returns:
##  a vector of coordinates
##
.testLabelOverlap <- function(object, rects, currentIndex, maxSteps) {
  r <- pi / 180L * c(90, as.vector(rbind(seq(80L, 40L, by=-10L),
                                         seq(100L, 140L, by=10L))))

  testrects <- rbind(matrix(c(object@mass, rep(0L, length(object)),
                              object@mass, object@intensity), ncol=4L),
                       rects[seq_len(currentIndex - 1L),
                             c("x0", "y0", "x1", "y1")])

  for (k in 0L:maxSteps) {
    ## move up
    s <- k / 4L
    cur <- rects[currentIndex, ]
    cur[c("y0", "y1", "y")] <- cur[c("y0", "y1", "y")] + s * cur["h"]

    for (l in r) {
        ## move in curve
        oldcur <- cur
        cur[c("y0", "y1", "y")] <- cur[c("y0", "y1", "y")] +
                                   sin(l) * s * cur["h"]
        cur[c("x0", "x1", "x")] <- cur[c("x0", "x1", "x")] +
                                   cos(l) * s * cur["w"]
      ## success
      if (!.overlaps(cur, testrects)) {
        return(cur)
      }
      cur <- oldcur
    }
  }
  ## no success, return original pos
  rects[currentIndex, ]
}

## .textLabelRects
##  calculate label rectangles
##
## params:
##  x: original x coordinates for labels
##  y: original y coordinates for labels
##  text: peak labels
##  adj: text alignment ?par("adj")
##  cex: font size ?par("cex")
##  srt: rotation in degree
##  offset: offset
##
## returns:
##  a matrix of coordinates
##
.textLabelRects <- function(x, y, text, adj=c(0.5, 0L), cex=0.7, srt=0,
                            offset=c(0.0, 0.2)) {
  wh <- .strWH(text, srt=srt, cex=cex)
  ## extra calculation of offsets to be independend of adj
  offset <- t(t(wh) * offset)

  m <- matrix(c(x - wh[, "w"] * adj[1L] - offset[, "w"],
                y - wh[, "h"] * adj[2L] - offset[, "h"],
                x + wh[, "w"] * (1L - adj[1L]) + offset[, "w"],
                y + wh[, "h"] * (1L - adj[2L]) + offset[, "h"],
                x, y, wh[, "w"], wh[, "h"]),
              ncol = 8L, dimnames = list(c(), c("x0", "y0", "x1", "y1",
                                                "x", "y", "w", "h")))
  if (srt) {
    m[, c("x0", "x1", "y0", "y1")] <-
      .rotate(rbind(m[, c("x0", "y0")], m[, c("x1", "y1")]),
              center=m[, c("x", "y")], srt=srt)

    if (m[1L, "x0"] > m[1L,  "x1"]) {
      m[, c("x1", "x0")] <- m[, c("x0", "x1")]
    }
    if (m[1L, "y0"] > m[1L,  "y1"]) {
      m[, c("y1", "y0")] <- m[, c("y0", "y1")]
    }
    m[, c("w", "h")] <- m[, c("x1", "y1")] - m[, c("x0", "y0")]
  }
  m
}

## .overlaps
## does rectangles overlap?
##
## HINT: doesn't work for rotated rectangles (!= 0, 90, 180, 270, 360)
##
## params:
##  a: vector, length 4, c(x0, y0, x1, y1); c(x0, y0) bottom left
##  b: matrix, at least 4 columns, c(x0, y0, x1, y1)
##
## returns:
##  TRUE/FALSE
.overlaps <- function(a, b) {
  if (is.vector(b)) {
    b <- t(b)
  }
  any(# rectangles on left/right of each other?
      !((a[1L] >= b[, 3L] | a[3L] <= b[, 1L]) |
      # rectangles on top/bottom of each other?
      (a[2L] >= b[, 4L] | a[4L] <= b[, 2L])))
}

## .scaleFactor
## get height to width scale factor of a graphic device
##
## returns:
##  double, scale factor
##
.scaleFactor <- function() {
  usr <- par("usr")
  pin <- par("pin")

  dx <- usr[2L] - usr[1L]
  dy <- usr[4L] - usr[3L]

  (dy * pin[1L]) / (dx * pin[2L])
}

## .rotate
## rotate rectangle
##
## params:
##  x: matrix, 2 columns, x,y coordinates
##  center: matrix, 2 columns, rotation center x,y coordinate
##  srt: rotation in degree
## returns:
##  double, length 2, x, y, rotated
##
.rotate <- function(x, center, srt) {
  stopifnot(all(dim(x)) == all(dim(center)))
  a <- pi * srt / 180L
  cosa <- cos(a)
  sina <- sin(a)

  dx <- x[, 1L] - center[, 1L]
  dy <- x[, 2L] - center[, 2L]

  x[, 1L] <- center[, 1L] + dx * cosa - dy * sina
  x[, 2L] <- center[, 2L] + dx * sina + dy * cosa

  x
}

## .strWH
## string width and height scaled to device scale and rotatet
##
## TODO: doesn't work for srt != 0, 90, 180, ... (.overlaps doesn't work for
## rotated rectangles)
##
## params:
##  text: text
##  cex: character scaling factor (see ?par)
##  srt: rotation in degree
##  scale: scale factor
##
## returns:
##  double, length 2, w, h
##
.strWH <- function(text, srt, cex=0.7, scale=.scaleFactor()) {
  a <- pi * srt / 180L

  scale <- abs(cos(a) + sin(a) * scale)

  matrix(c(strwidth(text, cex=cex) * scale,
           strheight(text, cex=cex) / scale), ncol = 2L,
         dimnames = list(c(), c("w", "h")))
}

context("calculateLabelPositions")

test_that(".textLabelRects", {
  pdf(NULL)
  on.exit(dev.off())
  plot.new()

  words <- c("foo", "foobar")
  cex <- 1

  r <- matrix(c(1:4, strwidth(words, cex=cex), strheight(words, cex=cex)),
              nrow = 2, dimnames = list(c(), c("x", "y", "w", "h")))
  r <- cbind(x0=r[, "x"] - r[, "w"]/2, y0=r[, "y"],
             x1=r[, "x"] + r[, "w"]/2, y1=r[, "y"] + r[, "h"],
             r)
  s <- MALDIquant:::.scaleFactor()
  r90 <- r
  r90[, "x1"] <- r90[, "x"]
  r90[, "x0"] <- r90[, "x"] - r[, "h"] / s
  r90[, "y0"] <- r90[, "y"] - r[, "w"]/2 * s
  r90[, "y1"] <- r90[, "y"] + r[, "w"]/2 * s
  r90[, c("w", "h")] <- r90[, c("x1", "y1")] - r90[, c("x0", "y0")]

  expect_equal(MALDIquant:::.textLabelRects(1:2, 3:4, c("foo", "foobar"),
                                            offset = c(0, 0), cex=1), r)
  expect_equal(MALDIquant:::.textLabelRects(1:2, 3:4, c("foo", "foobar"),
                                            offset = c(0, 0), cex=1, srt=90),
               r90)
})

test_that(".overlaps", {
  target <- c(2, 2, 4, 3)              # x1,y1 == bottom left
  dest <- list(a=target,               # identical
               b=c(1, 1, 3, 2.5),      # overlap
               c=c(3, 1, 6, 4),        # overlap
               d=c(2, 4, 3, 5),        # no overlap
               e=c(6, 0, 8, 1))        # no overlap
  dest2 <- list(rbind(dest$a, dest$b), # overlap
                rbind(dest$b, dest$e), # overlap
                rbind(dest$d, dest$e)) # no overlap

  r <- c(TRUE, TRUE, TRUE, FALSE, FALSE)
  r2 <- c(TRUE, TRUE, FALSE)

  for (i in seq(along=dest)) {
    expect_identical(MALDIquant:::.overlaps(target, dest[[i]]), r[i])
  }
  for (i in seq(along=dest2)) {
    expect_identical(MALDIquant:::.overlaps(target, dest2[[i]]), r2[i])
  }

})

test_that(".scaleFactor", {
  pdf(NULL)
  on.exit(dev.off())
  plot.new()

  expect_equal(MALDIquant:::.scaleFactor(), 1.116279, tolerance = 1e-5)
})

test_that(".strWH", {
  pdf(NULL)
  on.exit(dev.off())
  plot.new()

  expect_equal(as.vector(MALDIquant:::.strWH("foo", srt=0, cex=1,
                                             scale=0.5)),
               c(strwidth("foo", cex=1), strheight("foo", cex=1)))
  expect_equal(as.vector(MALDIquant:::.strWH("foo", srt=90, cex=1,
                                             scale=0.5)),
               c(strwidth("foo", cex=1)/2, strheight("foo", cex=1)*2))
})

test_that(".rotate", {
  .m <- function(x)matrix(x, ncol=2, byrow=TRUE)
  p <- .m(c(1, 2,
            4, 4,
            3, 2))
  center <- p - 1
  srt <- c(0, 90, 180, 270, 360)
  r <- list("0" = p,
            "90" = .m(c(-1, 2,
                        2, 4,
                        1, 2)),
            "180" = .m(c(-1, 0,
                         2, 2,
                         1, 0)),
            "270" = .m(c(1, 0,
                         4, 2,
                         3, 0)),
            "360" = p)

  for (i in seq(along=srt)) {
    expect_equal(MALDIquant:::.rotate(p, center, srt=srt[i]),
                 r[[i]])
  }
})

context(".unlist")

test_that(".unlist doesn't work recursive", {
  expect_identical(MALDIquant:::.unlist(list(a=list(a=1:3, b=4:6),
                                             b=list(c=7:9))),
                   list(1:3, 4:6, 7:9))
})

%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{MALDIquant: Quantitative Analysis of Mass Spectrometry Data}
%\VignetteKeywords{Bioinformatics, Proteomics, Mass Spectrometry}
%\VignettePackage{MALDIquant}

\documentclass[12pt]{article}

\usepackage{natbib}
\usepackage{hyperref}
\usepackage{tikz}
\usepackage{bibentry}       % inline bibentries
\nobibliography*            % no special bibliography for bibentry

\newcommand{\R}{\texttt{R}}
\newcommand{\CRAN}{\texttt{CRAN}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\texttt{#1}}}
\newcommand{\Mq}{\Rpackage{MALDIquant}}
\newcommand{\email}[1]{\href{mailto:#1}{\normalfont\texttt{#1}}}

\newcommand{\workflow}{
  \tikzstyle{line} = [draw, -latex]
  \tikzstyle{mqbox}=[rectangle, draw, text width=8.5em, text centered,
             minimum height=2.0em, fill=white]

  \begin{tikzpicture}[node distance=3.2em]
  % nodes
  \node [mqbox] (di) {Data Import};
  \node [mqbox, below of=di] (qc) {Quality Control};
  \node [mqbox, below of=qc] (vs) {Transformation \& Smoothing};
  \node [mqbox, below of=vs] (bc) {Baseline Correction};
  \node [mqbox, below of=bc] (cb) {Intensity Calibration};
  \node [mqbox, below of=cb] (sa) {Spectra Alignment};
  \node [mqbox, below of=sa] (pd) {Peak Detection};
  \node [mqbox, below of=pd] (pb) {Peak Binning};
  \node [mqbox, below of=pb] (fm) {Feature Matrix};

  % edges
  \path [line] (di) -- (qc);
  \path [line] (qc) -- (vs);
  \path [line] (vs) -- (bc);
  \path [line] (bc) -- (cb);
  \path [line] (cb) -- (sa);
  \path [line] (sa) -- (pd);
  \path [line] (pd) -- (pb);
  \path [line] (pb) -- (fm);

  \end{tikzpicture}
}


\title{\Mq{}: Quantitative Analysis of Mass Spectrometry Data}

\author{
  Sebastian Gibb%
  \thanks{\email{mail@sebastiangibb.de}}
}
\date{\today}

\begin{document}

<<setup, include=FALSE, cache=FALSE>>=
library("knitr")
opts_chunk$set(tidy.opts=list(width.cutoff=45), tidy=FALSE, fig.align="center",
               fig.height=4.25, comment=NA, prompt=TRUE)
@
<<env, echo=FALSE>>=
suppressPackageStartupMessages(library("MALDIquant"))
@

\maketitle

\begin{abstract}
  \Mq{} provides a complete analysis pipeline for MALDI-TOF and other
  2D mass spectrometry data.\\
  This vignette describes the usage of the \Mq{} package and guides the user
  through a typical preprocessing workflow.
\end{abstract}

\clearpage

\tableofcontents

\section*{Foreword}

\Mq{} is free and open source software for the \R{} \citep{RPROJECT}
environment and under active development.
If you use it, please support the project by citing it in publications:

\begin{quote}
  \bibentry{MALDIquant}
\end{quote}

If you have any questions, bugs, or suggestions do not hesitate to contact
me (\email{mail@sebastiangibb.de}). \\
Please visit \url{http://strimmerlab.github.io/software/maldiquant/}.

\section{Introduction}

\Mq{} comprising all steps from importing of raw data, preprocessing
(e.g. baseline removal), peak detection and non-linear peak alignment to
calibration of mass spectra.

\Mq{} was initially developed for clinical proteomics using
Matrix-Assisted Laser Desorption/Ionization (MALDI) technology.
However, the algorithms implemented in \Mq{} are generic and may be equally
applied to other 2D mass spectrometry data.

\Mq{} was carefully designed to be independent of any specific mass spectrometry
hardware. Nonetheless, a lot of open and native file formates, e.g. binary data
files from Bruker flex series instruments, mzXML, mzML, etc. are supported
through the associated \R{} package \Rpackage{MALDIquantForeign}.

\section{Setup}

After starting \R{} we could install \Mq{} and \Rpackage{MALDIquantForeign}
directly from \CRAN{} using \Rfunction{install.packages}:
<<mqsetup, eval=FALSE>>=
install.packages(c("MALDIquant", "MALDIquantForeign"))
@
Before we can use \Mq{} we have to load the package.
<<mqlibrary, eval=FALSE>>=
library("MALDIquant")
@

\section{MALDIquant objects}
\Mq{} is written in an object-oriented programming approach and uses
\R{}'s S4 objects. A spectrum is represented by an \Robject{MassSpectrum}
and a list of peaks by an \Robject{MassPeaks} instance. To create such objects
manually we could use \Rfunction{createMassSpectrum} and
\Rfunction{createMassPeaks}. In general we do not need these functions because
\Rpackage{MALDIquantForeign's} import routines will generate the
\Robject{MassSpectrum}/\Robject{MassPeaks} objects.
<<mqobjects>>=
s <- createMassSpectrum(mass=1:10, intensity=1:10,
                        metaData=list(name="Spectrum1"))
s
@
Each \Robject{MassSpectrum}/\Robject{MassPeaks} stores the mass and intensity
values of a spectrum respective of the peaks. Additionally they contain a list
of metadata. To access these information we use \Rfunction{mass},
\Rfunction{intensity} and \Rfunction{metaData}.
<<mqaccess>>=
mass(s)
intensity(s)
metaData(s)
@

\section{Workflow}

A Mass Spectrometry Analysis often follows the same workflow (see also Fig.
\ref{fig:workflow}). After importing the raw data
(see also the \Rpackage{MALDIquantForeign} package) we control the quality of
the spectra and draw some plots. We apply a variance-stabilizing transformation
and smoothing filter. Next we remove the chemical background using a
Baseline Correction method. To compare the intensities across spectra
we calibrate the intensity values (often called normalization) and the mass
values (warping, alignment). Subsequently we perfom a Peak Detection and do some
post processing like filtering etc.

\begin{figure}[htbp]
  \centering
    \begin{small}
    \workflow{}
  \caption{MS Analysis Workflow}
  \label{fig:workflow}
    \end{small}
\end{figure}

\clearpage
\subsection{Data Import}

Normally we will use some of the import methods provided by
\Rpackage{MALDIquantForeign}, e.g. \Rfunction{importBrukerFlex},
\Rfunction{importMzMl}, etc.
But in this vignette we will use a small example dataset
shipped with \Mq{}. This dataset is a
subset of MALDI-TOF data described in \cite{Fiedler2009}.
<<mqdataimport>>=
data(fiedler2009subset)
@
\Robject{fiedler2009subset} is a \Robject{list} of 16 \Robject{MassSpectrum}
objects. The 16 spectra are 8 biological samples with 2 technical replicates.
<<mqdataimport2>>=
length(fiedler2009subset)
fiedler2009subset[1:2]
@

\subsection{Quality Control}

For a basic quality control we test whether all spectra contain the same number
of data points and are not empty.
<<mqqclength>>=
any(sapply(fiedler2009subset, isEmpty))
table(sapply(fiedler2009subset, length))
@

Subsequently we control the mass difference between each data point
(should be equal or monotonically increasing) because
\Mq{} is designed for profile data and not for centroided data.
<<mqqcregular>>=
all(sapply(fiedler2009subset, isRegular))
@
Finally we draw some plots and inspect the spectra visually.
<<mqqcplots>>=
plot(fiedler2009subset[[1]])
plot(fiedler2009subset[[16]])
@

\subsection{Variance Stabilization}
We use the square root transformation to simplify graphical visualization and
to overcome the potential dependency of the variance from the mean.
<<mqvs>>=
spectra <- transformIntensity(fiedler2009subset,
                              method="sqrt")
@
\subsection{Smoothing}
Next we use a 21 point Savitzky-Golay-Filter \citep{Savitzky1964}
to smooth the spectra.
<<mqsm>>=
spectra <- smoothIntensity(spectra, method="SavitzkyGolay",
                           halfWindowSize=10)
@

\subsection{Baseline Correction}
Before we correct the baseline we visualize it. Here we use the SNIP algorithm
\citep{Ryan1988}.
<<mqve>>=
baseline <- estimateBaseline(spectra[[16]], method="SNIP",
                             iterations=100)
plot(spectra[[16]])
lines(baseline, col="red", lwd=2)
@
If we are satisfied with our estimated baseline we remove it.
<<mqbc>>=
spectra <- removeBaseline(spectra, method="SNIP",
                          iterations=100)
plot(spectra[[1]])
@

\subsection{Intensity Calibration/Normalization}
For better comparison and to overcome (very) small batch effects we equalize the
intensity values using the Total-Ion-Current-Calibration (often called
normalization).
<<mqcb>>=
spectra <- calibrateIntensity(spectra, method="TIC")
@

\subsection{Warping/Alignment}
Now we (re)calibrate the mass values. Our
alignment procedure is a peak based warping algorithm. If you need a finer
control or want to investigate the impact of different parameters please use
\Rfunction{determineWarpingFunctions} instead of the easier
\Rfunction{alignSpectra}.
<<mqpa>>=
spectra <- alignSpectra(spectra,
                        halfWindowSize=20,
                        SNR=2,
                        tolerance=0.002,
                        warpingMethod="lowess")
@
Before we call the Peak Detection we want to average the technical replicates.
Therefore we look for the sample name that is stored in the metadata because
each technical replicate has the same sample name.
<<mqav1>>=
samples <- factor(sapply(spectra,
                         function(x)metaData(x)$sampleName))
@
Next we use \Rfunction{averageMassSpectra} to create a mean spectrum for each
biological sample.
<<mqav2>>=
avgSpectra <- averageMassSpectra(spectra, labels=samples,
                                 method="mean")
@

\subsection{Peak Detection}
The next crucial step is the Peak Detection. Before we perform the peak
detection algorithm we estimate the noise of the spectra to get a feeling for
the signal-to-noise ratio.
<<mqpd1>>=
noise <- estimateNoise(avgSpectra[[1]])
plot(avgSpectra[[1]], xlim=c(4000, 5000), ylim=c(0, 0.002))
lines(noise, col="red")
lines(noise[,1], noise[, 2]*2, col="blue")
@
We decide to use a signal-to-noise ratio of 2 (blue line).
<<mqpd2>>=
peaks <- detectPeaks(avgSpectra, method="MAD",
                     halfWindowSize=20, SNR=2)
plot(avgSpectra[[1]], xlim=c(4000, 5000), ylim=c(0, 0.002))
points(peaks[[1]], col="red", pch=4)
@
\subsection{Peak Binning}
After the alignment the peak positions (mass) are very similar but not
identical. The binning is needed to make similar peak mass values identical.
<<mqpb>>=
peaks <- binPeaks(peaks, tolerance=0.002)
@
\subsection{Feature Matrix}
We choose a very low signal-to-noise ratio to keep as much features as possible.
To remove some false positive peaks we remove less frequent peaks.
<<mqfp>>=
peaks <- filterPeaks(peaks, minFrequency=0.25)
@
At the end of the analysis we create a feature matrix that could be used in
further statistical analysis. Please note that missing values (not detected
peaks) are imputed/interpolated from the corresponding spectrum.
<<mqim>>=
featureMatrix <- intensityMatrix(peaks, avgSpectra)
head(featureMatrix[, 1:3])
@

\section{Summary}
We shortly described a complete example workflow of a mass spectrometry data
analysis. Please note that this workflow is only an example and could not cover
every use case.\\
\Mq{} provides a lot of more functions than we mentioned in this vignette. The
described functions are the most used ones but they have a lot of more
parameters which could/need adjust to your data (e.g. \Robject{halfWindowSize},
\Robject{SNR}, \Robject{tolerance}, etc.). That's why we suggest the
user to read the manual pages of theses functions carefully.\\
We also provide more examples in the demo directory and at:
\begin{quote}
\url{http://strimmerlab.github.io/software/maldiquant/}
\end{quote}
Please do not hesitate to contact me (\email{mail@sebastiangibb.de}) if you have
any questions.

\section{Session Information}
<<sessioninfo, echo=FALSE, results="asis">>=
toLatex(sessionInfo(), locale=FALSE)
@

\bibliographystyle{apalike}
\bibliography{bibliography}

\end{document}

\name{determineWarpingFunctions}
\alias{determineWarpingFunctions}
\title{
Determine warping functions of MassPeaks objects.
}
\description{
This function determines a warping function for a list of
\code{\linkS4class{AbstractMassObject}} objects
(warping is also known as \emph{phase correction/spectra alignment}).
}
\usage{
determineWarpingFunctions(l, reference, tolerance=0.002,
                          method=c("lowess", "linear", "quadratic", "cubic"),
                          allowNoMatches=FALSE,
                          plot=FALSE, plotInteractive=FALSE, \dots)
}
\arguments{
  \item{l}{
    \code{list}, list of \code{\linkS4class{MassPeaks}} objects.
  }
  \item{reference}{
    \code{\linkS4class{MassPeaks}}, reference object to which the samples
    (\code{l}) should be aligned. If missing
    \code{\link[MALDIquant]{referencePeaks}} is used.
  }
  \item{tolerance}{
    \code{double}, maximal relative deviation of a peak position (mass) to be
    considered as identical. Must be multiplied by \code{10^-6} for ppm,
    e.g. use \code{tolerance=5e-6} for 5 ppm.
  }
  \item{method}{
    used basic warping function.
  }
  \item{allowNoMatches}{
    \code{logical}, don't throw an error if an \code{\linkS4class{MassPeaks}}
    object could not match to the \code{reference}.
  }
  \item{plot}{
    \code{logical}, if \code{TRUE} a warping plot is drawn for each sample.
  }
  \item{plotInteractive}{
    \code{logical}, if \code{FALSE} a non-interactive device (e.g.
    \code{pdf}) is used for warping plots.
  }
  \item{\dots}{
    arguments to be passed to \code{warpingFunction}
  }
}
\details{
  \code{warpingFunction}: \code{determineWarpingFunctions} estimates a warping
  function to overcome the difference between mass in \code{reference} and in
  the current sample. To calculate the differences each reference peak would
  match with the highest sample peak in the nearer neighborhood
  (defined by \code{mass of reference peak*tolerance}). \cr
  \code{allowNoMatches}: If \code{allowNoMatches} is \code{TRUE} a warning instead
  of an error is thrown if an \code{\linkS4class{MassPeaks}}
  object could not match to the \code{reference}. The returned list of warping
  functions will contain \code{NA} for this object (same index in the list).
  \code{plotInteractive}: If \code{plot} is \code{TRUE} a lot of output is
  created (each sample in \code{l} gets its own plot).
  That's why an non-interactive devices is recommended:
  \preformatted{
## create a device
pdf()
## calculate warping functions
w <- determineWarpingFunctions(p, plot=TRUE)
## close device
dev.off()
  }
}
\value{
Returns a \code{list} of individual warping functions. The attribute
\code{nmatch} contains the number of matches of each
\code{\linkS4class{MassPeaks}} element in \code{l} against \code{reference}.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{referencePeaks}},
\code{\link[MALDIquant]{warpMassPeaks}},
\code{\link[MALDIquant]{warpMassSpectra}},
\code{\linkS4class{MassPeaks}}

\code{demo("warping")}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create a reference MassPeaks object
r <- createMassPeaks(mass=1:5, intensity=1:5)

## create test samples
p <- list(createMassPeaks(mass=((1:5)*1.01), intensity=1:5),
          createMassPeaks(mass=((1:5)*0.99), intensity=1:5))

## create an interactive device with 2 rows
par(mfrow=c(2, 1))
## calculate warping function
## (using a linear function as basic warping function)
## and show warping plot
w <- determineWarpingFunctions(p, tolerance=0.02, method="linear",
                               plot=TRUE, plotInteractive=TRUE)
par(mfrow=c(1, 1))

## access number of matches
attr(w, "nmatch")

## w contains the individual warping functions
warpedPeaks <- warpMassPeaks(p, w)

## compare results
all(mass(r) == mass(warpedPeaks[[1]])) # TRUE
all(mass(r) == mass(warpedPeaks[[2]])) # TRUE



## realistic example

## load example data
data("fiedler2009subset", package="MALDIquant")

## running typical workflow

## use only four spectra of the subset
spectra <- fiedler2009subset[1:4]

## transform intensities
spectra <- transformIntensity(spectra, method="sqrt")

## smooth spectra
spectra <- smoothIntensity(spectra, method="MovingAverage")

## baseline correction
spectra <- removeBaseline(spectra)

## detect peaks
peaks <- detectPeaks(spectra)

## create an interactive device with 2 rows
par(mfrow=c(4, 1))
## calculate warping functions (using LOWESS based basic function [default])
w <- determineWarpingFunctions(peaks, plot=TRUE, plotInteractive=TRUE)
par(mfrow=c(1, 1))



## realistic example with user defined reference/calibration peaks

## use the workflow above for fiedler2009subset

## create reference peaks
refPeaks <- createMassPeaks(mass=c(1207, 1264, 1351, 1466, 1616, 2769, 2932,
                                   3191, 3262, 4091, 4209, 5904, 7762, 9285),
                            intensity=rep(1, 14))

## create an interactive device with 2 rows
par(mfrow=c(4, 1))
## calculate warping functions (using a quadratic function as basic function)
w <- determineWarpingFunctions(peaks, reference=refPeaks, method="quadratic",
                               plot=TRUE, plotInteractive=TRUE)
par(mfrow=c(1, 1))
}
\keyword{methods}

\name{findEmptyMassObjects}
\alias{findEmptyMassObjects}
\alias{removeEmptyMassObjects}
\title{
Finds or removes empty AbstractMassObject objects in a list.
}
\description{
These functions looks for empty \code{\linkS4class{AbstractMassObject}} objects
in a \code{\link{list}}.
}
\usage{
findEmptyMassObjects(l)

removeEmptyMassObjects(l)
}
\arguments{
  \item{l}{
    \code{list}, list of \code{\linkS4class{AbstractMassObject}} where empty
    objects should be found or removed.
  }
}
\value{
\code{findEmptyMassObjects}: Returns a \code{\link{vector}} of indices referring to empty
\code{\linkS4class{AbstractMassObject}} objects.

\code{removeEmptyMassObjects}: Returns a \code{\link{list}} of
\code{\linkS4class{AbstractMassObject}} objects but without empty ones.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{isEmpty,AbstractMassObject-method}},
\code{\linkS4class{AbstractMassObject}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create list
peakList <- list()

## create two MassPeaks objects and add them to the list
peakList[[1]] <- createMassPeaks(mass=1:100, intensity=1:100,
                                 metaData=list(name="example 1"))
peakList[[2]] <- createMassPeaks(mass=1:100, intensity=1:100,
                                 metaData=list(name="example 2"))

## find empty objects (there should not be any one)
findEmptyMassObjects(peakList)

## add an empty MassPeaks object to the list
peakList[[3]] <- createMassPeaks(mass=double(), intensity=double(),
                                 metaData=list(name="empty MassPeaks object"))

## look for empty objects (isEmptyIdx == 3)
(isEmptyIdx <- findEmptyMassObjects(peakList))

## to remove all empty MassObjects from a list
length(peakList) # 3
peakList <- removeEmptyMassObjects(peakList)
length(peakList) # 2; WARNING: all indices could changed
}
\keyword{methods}

\name{msiSlices}
\title{Turn a list of AbstractMassObjects into a mass spectrometry imaging
  slice.}
\alias{msiSlices}
\description{
This function turns a mass spectrometry imaging dataset represented by a
\code{list} of \code{\linkS4class{AbstractMassObject}} objects into an
\code{\link[MALDIquant]{intensityMatrix}} for each slice (stored in an
\code{array}).
}
\usage{
msiSlices(x, center, tolerance, method=c("sum", "mean", "median"), adjust=TRUE)

}
\arguments{
  \item{x}{a \code{list} of \code{\linkS4class{MassSpectrum}}/
    \code{\linkS4class{MassPeaks}} objects.}
  \item{center}{\code{double}, the \code{center} mass value of each slice.}
  \item{tolerance}{\code{double}, specifies the thickness of the slices
    (\code{center + c(-tolerance, tolerance)}).}
  \item{method}{used aggregation function.}
  \item{adjust}{\code{logical}, if \code{TRUE} the lowest coordinates of the
    mass spectrometry imaging dataset are set to \code{c(x=1, y=1)} to avoid
    \code{NA} values at the borders.}
}
\details{
Each \code{\linkS4class{MassSpectrum}}/\code{\linkS4class{MassPeaks}} object in
\code{x} must contain a \code{list} named \code{imaging} with an element
\code{pos} that stores the \code{x} and \code{y} value of the spectrum, e.g.:
\preformatted{
> metaData(spectra[[1]])$imaging$pos
x y
1 5
}
}
\value{
Returns an \code{\link{array}} of three dimensions. The first and second
dimensions contains the x and y coordinates of the image. The third dimension
represents the index of the \code{center} of each slice. There are two
additional attributes, namely \code{"center"} and \code{"tolerance"} which store
the original \code{center} and \code{tolerance} information.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{AbstractMassObject}},
\code{\linkS4class{MassSpectrum}},
\code{\linkS4class{MassPeaks}},
\code{\link[MALDIquant]{coordinates,AbstractMassObject-method}},
\code{\link[MALDIquant]{plotMsiSlice,list-method}}

Please find real examples on:

Website: \url{https://strimmerlab.github.io/software/maldiquant/}

Vignette: \url{https://github.com/sgibb/MALDIquantExamples/raw/master/inst/doc/nyakas2013.pdf}

Shiny: \url{https://github.com/sgibb/ims-shiny/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## please note: this is NOT a MSI data set
## we just add some coordinates for demonstration
coordinates(fiedler2009subset) <- cbind(x=rep(1:4, 2), y=rep(1:2, each=4))

slices <- msiSlices(fiedler2009subset, center=c(5864.49, 8936.97),
                    tolerance=0.25)

slices
}

\name{removeBaseline-methods}
\docType{methods}
\alias{removeBaseline}
\alias{removeBaseline,MassSpectrum-method}
\alias{removeBaseline,list-method}
\title{Removes the baseline of a MassSpectrum object.}
\description{
This method removes the baseline of mass spectrometry data
(represented by a \code{\linkS4class{MassSpectrum}} object).\cr
The intensity of the mass spectrometry data would be reduced by
\code{baseline}.
}
\usage{
\S4method{removeBaseline}{MassSpectrum}(object,
  method=c("SNIP", "TopHat", "ConvexHull", "median"),
  \dots)
\S4method{removeBaseline}{list}(object, \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{MassSpectrum}} object or a \code{list} of
    \code{\linkS4class{MassSpectrum}} objects.}

  \item{method}{used baseline estimation method, one of
    \code{"SNIP"}, \code{"TopHat"}, \code{"ConvexHull"} or \code{"median"}. See
    \code{\link[MALDIquant]{estimateBaseline,MassSpectrum-method}} for details.
  }
  \item{\dots}{arguments to be passed to
    \code{\link[MALDIquant]{estimateBaseline,MassSpectrum-method}}. If
    \code{object} is a \code{list} \code{mc.cores} is also supported.}
}
\value{
  Returns a modified \code{\linkS4class{MassSpectrum}} object with reduced
  intensities.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassSpectrum}},
\code{\link[MALDIquant]{estimateBaseline,MassSpectrum-method}}

\code{demo("baseline")}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## choose only the first mass spectrum
s <- fiedler2009subset[[1]]

## plot spectrum
plot(s)

## subtract baseline
b <- removeBaseline(s, method="SNIP")

## draw modified spectrum on the plot
lines(b, col="blue")
}
\keyword{methods}

\name{detectPeaks-methods}
\docType{methods}
\alias{detectPeaks}
\alias{detectPeaks,MassSpectrum-method}
\alias{detectPeaks,list-method}
\title{Detects peaks in a MassSpectrum object.}
\description{
This method looks for peaks in mass spectrometry data
(represented by a \code{\linkS4class{MassSpectrum}} object).\cr
A peak is a local maximum above a user defined noise threshold.
}
\usage{
\S4method{detectPeaks}{MassSpectrum}(object,
  halfWindowSize=20, method=c("MAD", "SuperSmoother"), SNR=2,
  \dots)
\S4method{detectPeaks}{list}(object, \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{MassSpectrum}} object or a \code{list} of
    \code{\linkS4class{MassSpectrum}} objects.}
  \item{halfWindowSize}{\code{numeric}, half window size. \cr
    The resulting window reaches from \code{mass[currentIndex-halfWindowSize]}
    to \code{mass[currentIndex+halfWindowSize]}. A local maximum have to be
    the highest one in the given window to be recognized as peak.}
  \item{method}{a noise estimation function; see
    \code{\link[MALDIquant]{estimateNoise,MassSpectrum-method}}.
  }
  \item{SNR}{single numeric value. \code{SNR} is an abbreviation for
    \emph{s}ignal-to-\emph{n}oise-\emph{r}atio. A local maximum has to
    be higher than \code{SNR*noise} to be recognize as peak.}
  \item{\dots}{arguments to be passed to
    \code{\link[MALDIquant]{estimateNoise,MassSpectrum-method}}. If
    \code{object} is a \code{list} \code{mc.cores} is also supported.}
}
\value{
Returns a \code{\linkS4class{MassPeaks}} object.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassPeaks}},
\code{\linkS4class{MassSpectrum}},
\code{\link[MALDIquant]{estimateNoise,MassSpectrum-method}}

\code{demo("peaks")}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## choose only the first mass spectrum
s <- fiedler2009subset[[1]]

## transform intensities
s <- transformIntensity(s, method="sqrt")

## smoothing spectrum
s <- smoothIntensity(s, method="MovingAverage")

## remove baseline
s <- removeBaseline(s)

## plot spectrum
plot(s)

## call peak detection
p <- detectPeaks(s)

## draw peaks on the plot
points(p)

## label 10 highest peaks
top10 <- intensity(p) \%in\% sort(intensity(p), decreasing=TRUE)[1:10]
labelPeaks(p, index=top10)
}
\keyword{methods}

\name{AbstractMassObject-class}
\Rdversion{1.1}
\docType{class}
\alias{AbstractMassObject-class}
\alias{[,AbstractMassObject,logical,missing-method}
\alias{[,AbstractMassObject,numeric,missing-method}
\alias{[,MassPeaks,logical,missing-method}
\alias{[,MassPeaks,numeric,missing-method}
\alias{as.matrix,AbstractMassObject-method}
\alias{intensity}
\alias{intensity<-}
\alias{intensity,AbstractMassObject-method}
\alias{intensity<-,AbstractMassObject,numeric-method}
\alias{isEmpty}
\alias{isEmpty,AbstractMassObject-method}
\alias{length,AbstractMassObject-method}
\alias{lines,AbstractMassObject-method}
\alias{mass}
\alias{mass<-}
\alias{mass,AbstractMassObject-method}
\alias{mass<-,AbstractMassObject,numeric-method}
\alias{mz}
\alias{mz<-}
\alias{mz,AbstractMassObject-method}
\alias{mz<-,AbstractMassObject,numeric-method}
\alias{metaData}
\alias{metaData<-}
\alias{metaData,AbstractMassObject-method}
\alias{metaData<-,AbstractMassObject-method}
\alias{coordinates}
\alias{coordinates<-}
\alias{coordinates,AbstractMassObject-method}
\alias{coordinates<-,AbstractMassObject,numeric-method}
\alias{coordinates<-,AbstractMassObject,matrix-method}
\alias{coordinates,list-method}
\alias{coordinates<-,list,matrix-method}
\alias{points,AbstractMassObject-method}

\title{Class "AbstractMassObject"}
\description{
\code{\linkS4class{AbstractMassObject}} is an abstract (means pure virtual)
class. It is the parent class of \code{\linkS4class{MassSpectrum}} and
\code{\linkS4class{MassPeaks}}.
It shouldn't create or handle by the user because it is for internal use only.
}
\section{Derived classes}{
\code{\linkS4class{MassPeaks}},
\code{\linkS4class{MassSpectrum}}
}
\section{Slots}{
\describe{
  \item{\code{mass}:}{\code{numeric}, mass or mass-to-charge ratio}
  \item{\code{intensity}:}{\code{numeric}, intensities for measured
    mass-to-charge ratios}
  \item{\code{metaData}:}{\code{list}, some metadata to describe the
    spectrum}
}
}
\section{Methods}{
\describe{
  \item{[}{\code{signature(x = "AbstractMassObject", i = "numeric")}:
    Extracts a range of an \code{\linkS4class{AbstractMassObject}} object
    and returns a new one.}
  \item{as.matrix}{\code{signature(x = "AbstractMassObject")}:
    Converts an \code{\linkS4class{AbstractMassObject}} object to a
    matrix with 2 columns (\code{mass}, \code{intensity}).}
  \item{coordinates}{\code{signature(object = "AbstractMassObject")}:
    Accessor function for coordinates stored in object generated from imaging
    mass spectrometry data.}
  \item{coordinates<-}{\code{signature(object = "AbstractMassObject",
      value = "numeric|matrix")}
    Replacement function for coordinates used in imaging mass spectrometry
    datasets.}
  \item{intensity}{\code{signature(object = "AbstractMassObject")}:
    Accessor function for slot \code{intensity}.}
  \item{intensity<-}{\code{signature(object = "AbstractMassObject",
      value = "numeric")}
    Replacement function for slot \code{intensity}.}
  \item{isEmpty}{\code{signature(object = "AbstractMassObject")}:
    Returns \code{TRUE} if length of \code{intensity} is 0 or all
    \code{intensity} values are 0.}
  \item{length}{\code{signature(x = "AbstractMassObject")}:
    Returns length of slot \code{intensity}.}
  \item{lines}{\code{signature(x = "AbstractMassObject")}:
    Extented function for adding \code{AbstractMassObject} object as a
    line to a specific plot.
    See \code{\link{lines}} for details.}
  \item{mass}{\code{signature(object = "AbstractMassObject")}:
    Accessor function for slot \code{mass}.}
  \item{mass<-}{\code{signature(object = "AbstractMassObject",
      value = "numeric")}
    Replacement function for slot \code{mass}.}
  \item{mz}{\code{signature(object = "AbstractMassObject")}:
    Accessor function for slot \code{mass}.}
  \item{mz<-}{\code{signature(object = "AbstractMassObject",
      value = "numeric")}
    Replacement function for slot \code{mass}.}
  \item{metaData}{\code{signature(object = "AbstractMassObject")}:
    Accessor function for slot \code{metaData}.}
  \item{metaData<-}{\code{signature(object = "AbstractMassObject")}:
    Replacement function for slot \code{metaData}.}
  \item{plot}{\code{signature(x = "AbstractMassObject", y = "missing")}:
    Extented function for plotting an \code{AbstractMassObject} object.
    See \code{\link{plot,AbstractMassObject,missing-method}} for
    details.}
  \item{points}{\code{signature(x = "AbstractMassObject")}:
    Extented function for adding
    \code{\linkS4class{AbstractMassObject}} object as points
    to a specific plot.
    See \code{\link{points}} for details.}
  \item{trim}{\code{signature(object = "AbstractMassObject",
      range = "numeric")}: Trim an \code{AbstractMassObject}
    object. See \code{\link[MALDIquant]{trim,AbstractMassObject,numeric-method}}
    for details.}
  \item{transformIntensity}{\code{signature(object = "AbstractMassObject")}:
    Transforms the intensities of an \code{AbstractMassObject} object.
    See \code{\link[MALDIquant]{transformIntensity,AbstractMassObject-method}}
    for details.}
}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassPeaks}},
\code{\linkS4class{MassSpectrum}},
\code{\link[MALDIquant]{plot,AbstractMassObject,missing-method}},
\code{\link[MALDIquant]{transformIntensity,AbstractMassObject-method}},
\code{\link[MALDIquant]{trim,AbstractMassObject,numeric-method}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create example spectrum
s <- createMassSpectrum(mass=1:10, intensity=11:20,
                        metaData=list(name="Example Spectrum"))

## get intensity
intensity(s)

## get mass
mass(s)

## get metaData
metaData(s)

## replace metaData
metaData(s) <- list(name="Spectrum")

## trim spectrum
trim(s, c(2, 9))

## select a range
s[3:6]
}
\keyword{classes}

\name{isMassSpectrum}
\alias{isMassSpectrum}
\alias{isMassPeaks}
\title{
Tests for MassSpectrum or MassPeaks object.
}
\description{
These functions test for a \code{\linkS4class{MassSpectrum}} or
\code{\linkS4class{MassPeaks}} object.
}
\usage{
isMassSpectrum(x)

isMassPeaks(x)
}
\arguments{
  \item{x}{
    object to be tested.
  }
}
\value{
Returns \code{\link{TRUE}} or \code{\link{FALSE}} depending on whether its
argument is an \code{\linkS4class{MassSpectrum}} or
\code{\linkS4class{MassPeaks}} object.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassPeaks}},
\code{\linkS4class{MassSpectrum}},
\code{\linkS4class{AbstractMassObject}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create a MassPeaks object
peaks <- createMassPeaks(mass=1:100, intensity=1:100,
                         metaData=list(name="example 1"))

## test
isMassPeaks(peaks)      # returns TRUE
isMassSpectrum(peaks)   # returns FALSE
isMassPeaks(double())   # returns FALSE
}
\keyword{methods}

\name{labelPeaks-methods}
\docType{methods}
\alias{labelPeaks}
\alias{labelPeaks,MassPeaks-method}
\title{Draws peak labels to plot.}
\description{
\code{\link[MALDIquant]{labelPeaks}} draws the corresponding mass values on top
of the peaks stored in a \code{\linkS4class{MassPeaks}} object to a plot.
}
\usage{
\S4method{labelPeaks}{MassPeaks}(object,
  index,
  mass,
  labels,
  digits=3, underline=TRUE,
  verticalOffset=abs(diff(par("usr")[3:4]))*0.01,
  absoluteVerticalPos,
  adj=c(0.5, 0), cex=0.7, srt=0,
  avoidOverlap=FALSE,
  arrowLength=0, arrowLwd=0.5, arrowCol=1,
  \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{MassPeaks}} object.}
  \item{index}{\code{integer}/\code{logical}, indices of peaks to label.}
  \item{mass}{\code{numeric}, mass of peaks to label.}
  \item{labels}{\code{character}, use \code{labels} instead of mass values as
    peak label.}
  \item{digits}{\code{integer}, number of decimal places.}
  \item{underline}{logical, underline peak values?}
  \item{verticalOffset}{\code{numeric}, move label vertically (relative to
    peak height).}
  \item{absoluteVerticalPos}{\code{numeric}, absolute \code{y} value for the
    label. If missing \code{verticalOffset} is used.}
  \item{adj}{\code{numeric}, adjust text to the left, center, right and top,
    center, bottom; see \code{\link[graphics]{text}}.}
  \item{cex}{\code{numeric}, font size, see \code{\link[graphics]{par}}.}
  \item{srt}{\code{numeric}, the label rotation in degrees.}
  \item{avoidOverlap}{\code{logical}, try to find label coordinates to avoid
    overlap.}
  \item{arrowLength, arrowLwd, arrowCol}{arrow parameters, possible vectors.
    \code{NA} values in \code{arrowCol} cause the arrow to be omitted, see
    \code{\link[graphics]{arrows}}.}
  \item{...}{arguments to be passed to \code{\link[graphics]{text}}.}
}
\details{
  Please note that \code{avoidOverlap = TRUE} is just supported for
  \code{srt \%\% 90 == 0} (means \code{srt} has to be a multiple of 90 degree).
}
\author{Sebastian Gibb}
\seealso{
  \code{\linkS4class{MassPeaks}},
  \code{\link[MALDIquant]{plot,AbstractMassObject,missing-method}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create a MassPeaks object from scratch
p <- createMassPeaks(mass=1:20, intensity=sample(x=100:10000, size=20),
                     metaData=list(name="example"))

## plot peaks
plot(p)

## label the first 5 peaks
labelPeaks(p, index=1:5)

## label all peaks in mass range 15 to 20
labelPeaks(p, mass=15:20, underline=FALSE)

## label highest peaks (top 5)
top5 <- intensity(p) \%in\% sort(intensity(p), decreasing=TRUE)[1:5]
labelPeaks(p, index=top5, col="red")


## real example
data("fiedler2009subset")

## a simplified preprocessing
r <- removeBaseline(fiedler2009subset[[1]])
p <- detectPeaks(r)
plot(p)

## label highest peaks (top 10) and avoid label overlap
top10 <- sort(intensity(p), decreasing=TRUE, index.return=TRUE)$ix[1:10]
labelPeaks(p, index=top10, avoidOverlap=TRUE, digits=1)

## use own labels and rotate by 90 degree
plot(p)
labelPeaks(p, index=top10, labels=paste("TOP", 1:10), underline=FALSE,
           srt=90, adj=c(0, 0.5), col=2)
}
\keyword{methods}
\keyword{Graphics|hplot}

\name{estimateNoise-methods}
\docType{methods}
\alias{estimateNoise}
\alias{estimateNoise,MassSpectrum-method}
\title{Estimates the noise of a MassSpectrum object.}
\description{
This method estimates the noise of mass spectrometry data
(represented by a \code{\linkS4class{MassSpectrum}} object).\cr
}
\usage{
\S4method{estimateNoise}{MassSpectrum}(object,
  method=c("MAD", "SuperSmoother"),
  \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{MassSpectrum}} object}
  \item{method}{used noise estimation method, one of
    \code{"MAD"} or \code{"SuperSmoother"}.
  }
  \item{\dots}{arguments to be passed to \code{method}.}
}
\details{
  \describe{
    \item{\code{"MAD"}:}{
      This function estimates the noise of mass spectrometry data
      by calculating the median absolute deviation, see also
      \code{\link[stats]{mad}}.
    }
    \item{\code{"SuperSmoother"}:}{
      This function estimates the noise of mass spectrometry data using
      Friedman's Super Smoother. Please refer
      \code{\link[stats]{supsmu}} for details and additional arguments.
    }
  }
}
\value{
Returns a two column matrix (first column: mass, second column: intensity)
of the estimated noise.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassSpectrum}},
\code{\link[MALDIquant]{detectPeaks,MassSpectrum-method}},
\code{\link[stats]{mad}},
\code{\link[stats]{supsmu}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## choose only the first mass spectrum
s <- fiedler2009subset[[1]]

## transform intensities
s <- transformIntensity(s, method="sqrt")

## remove baseline
s <- removeBaseline(s)

## plot spectrum
plot(s)

## estimate noise
nm <- estimateNoise(s, method="MAD")
nss <- estimateNoise(s, method="SuperSmoother")

## draw noise on the plot
lines(nm, col=2)
lines(nss, col=4)

## draw legend
legend(x="topright", lwd=1, legend=c("MAD", "SuperSmoother"),
       col=c(2, 4))
}
\keyword{methods}

\name{averageMassSpectra}
\alias{averageMassSpectra}
\title{
Averages \code{\linkS4class{MassSpectrum}} objects.
}
\description{
This function averages \code{\linkS4class{MassSpectrum}} objects.
}
\usage{
averageMassSpectra(l, labels, method=c("mean", "median", "sum"), \dots)
}
\arguments{
  \item{l}{
    \code{list}, list of \code{\linkS4class{MassSpectrum}} objects.
  }
  \item{labels}{
    \code{list}, list of \code{\link{factor}}s (one for each
    \code{\linkS4class{MassSpectrum}} object) to do groupwise averaging.
  }
  \item{method}{used aggregation function.}
  \item{\dots}{arguments to be passed to underlying functions (currently only
    \code{mc.cores} is supported).}
}
\details{
The mass of the averaged \code{\linkS4class{MassSpectrum}} object
will be the mass of the first non-empty
\code{\linkS4class{MassSpectrum}} object (of each group).
}
\value{
Returns a single (no \code{labels} given) or a \code{\link{list}}
(\code{labels} given) of averaged \code{\linkS4class{MassSpectrum}} objects.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassSpectrum}},
\code{\link[MALDIquant]{mergeMassPeaks}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create four MassSpectrum objects and add them to a list
s <- list(createMassSpectrum(mass=1:5, intensity=1:5),
          createMassSpectrum(mass=1:5, intensity=1:5),
          createMassSpectrum(mass=1:5, intensity=6:10),
          createMassSpectrum(mass=1:5, intensity=6:10))

## average all four MassSpectrum objects into a single new one
## by sum their intensities
## (no labels, returns only one new MassSpectrum object)
summedSpectra <- averageMassSpectra(s, method="sum")

## only average MassSpectrum objects in a group
## (e.g. useful for technical replicates)
## (two different labels, returns a list of two new MassPeaks objects)
groups <- factor(c("a", "a", "b", "b"), levels=c("a", "b"))
averagedSpectra <- averageMassSpectra(s, labels=groups, method="mean")
}
\keyword{methods}

\name{warpMassSpectra}
\alias{warpMassSpectra}
\alias{warpMassPeaks}
\title{
Run warping functions on AbstractMassObject objects.
}
\description{
These functions run warping functions on
\code{\linkS4class{AbstractMassObject}} objects
(warping is also known as \emph{phase correction}).
}
\usage{
warpMassPeaks(l, w, emptyNoMatches=FALSE)

warpMassSpectra(l, w, emptyNoMatches=FALSE)
}
\arguments{
  \item{l}{
    \code{list}, list of \code{\linkS4class{MassPeaks}} or
    \code{\linkS4class{MassSpectrum}} objects.
  }
  \item{w}{
    a \code{list} of warping functions determined by
    \code{\link[MALDIquant]{determineWarpingFunctions}}. Has to be of the
    same length as \code{l}.
  }
  \item{emptyNoMatches}{
    \code{logical}, if \code{TRUE} (default: \code{FALSE})
    the intensity values of
    \code{\linkS4class{MassSpectrum}} or
    \code{\linkS4class{MassPeaks}} objects with missing (\code{NA})
    warping functions are set to zero.
  }
}
\details{
The warping function \code{w} is called in the following way:
\deqn{newMass = oldMass + w(oldMass)}
}
\value{
Returns a \code{list} of warped \code{\linkS4class{MassPeaks}} or
\code{\linkS4class{MassSpectrum}} objects.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{determineWarpingFunctions}},
\code{\linkS4class{MassPeaks}},
\code{\linkS4class{MassSpectrum}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create a MassPeaks object
p <- createMassPeaks(mass=1:5, intensity=1:5)

## stupid warping function for demonstration
## (please use determineWarpingFunctions in real life applications)
simpleWarp <- function(x) { return(1) }

## run warping function
w <- warpMassPeaks(list(p), list(simpleWarp))[[1]]

## compare results
all(mass(w) == mass(p)+1) # TRUE

## no warping (MassPeaks object is not changed)
warpMassPeaks(list(p), list(NA))

## no warping (intensity values of MassPeaks object are set to zero)
warpMassPeaks(list(p), list(NA), emptyNoMatches=TRUE)
}
\keyword{methods}

\name{MALDIquant-deprecated}
\alias{MALDIquant-deprecated}
%------ PLEASE: put \alias{.} here for EACH !
%------ NOTE: ../R/Deprecated.R must be synchronized with this!
\title{Deprecated Functions/Methods in Package \pkg{MALDIquant}}
\description{
  These functions/methods are provided for compatibility with older versions
  \code{\link{MALDIquant}} only, and may be defunct as soon as the next release.
}
%\details{
%\describe{
%  % since MALDIquant x.y.z
%  %\item{function}{explanation}
%  % since MALDIquant 1.11.12
%  }
%}
\seealso{
  \code{\link{Deprecated}}
}
\keyword{internal}
\keyword{misc}

\name{calibrateIntensity-methods}
\docType{methods}
\alias{calibrateIntensity}
\alias{calibrateIntensity,MassSpectrum-method}
\alias{calibrateIntensity,list-method}
\title{Calibrates intensities of a MassSpectrum object.}
\description{
This function calibrates (normalize) intensities of
\code{\linkS4class{MassSpectrum}} objects.
}

\usage{
\S4method{calibrateIntensity}{MassSpectrum}(object,
  method=c("TIC", "PQN", "median"), range, \dots)
\S4method{calibrateIntensity}{list}(object,
  method=c("TIC", "PQN", "median"), range, \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{MassSpectrum}} object or a \code{list} of
    \code{\linkS4class{MassSpectrum}} objects.}
  \item{method}{the calibration method to be used. This should be one of
    \code{"TIC"}, \code{"PQN"} or \code{"median"}.
    See \sQuote{Details} section.}
  \item{range}{\code{numeric} of length 2, if given the scaling factor is
    calculated on the mass range from \code{range[1L]} to \code{range[2L]} and
    applied to the whole spectrum.}
  \item{\dots}{arguments to be passed to other functions. Currently only
    \code{mc.cores} is supported if \code{object} is a \code{list}.}
}
\details{
  A number of different calibration methods are provided:
  \describe{
    \item{\code{"TIC"}:}{The TIC (\emph{T}otal \emph{I}on \emph{C}urrent) of a
      \code{\linkS4class{MassSpectrum}} object is set to one. If \code{range} is
      given the \emph{TIC} is only calculated for the intensities in the
      specified mass range.}
    \item{\code{"PQN"}:}{The PQN (\emph{P}robabilistic \emph{Q}uotient
      \emph{N}ormalization) is described in \emph{Dieterle et al 2006}.
      \code{calibrateIntensity} uses the following algorithm:
      \enumerate{
        \item{Calibrate all spectra using the \code{"TIC"} calibration.}
        \item{Calculate a median reference spectrum.}
        \item{Calculate the quotients of all intensities of the spectra with
          those of the reference spectrum.}
        \item{Calculate the median of these quotients for each spectrum.}
        \item{Divide all intensities of each spectrum by its median of
          quotients.}
      }
    }
    \item{\code{"median"}:}{The median of intensities of a
      \code{\linkS4class{MassSpectrum}} object is set to one.}
  }
}
\value{
Returns a modified \code{\linkS4class{MassSpectrum}} object with calibrated
intensities.
}
\references{
F. Dieterle, A. Ross, G. Schlotterbeck, and Hans Senn. 2006.
Probabilistic quotient normalization as robust method to account for
dilution of complex biological mixtures. Application in 1H NMR metabonomics.
Analytical Chemistry 78(13): 4281-4290.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassSpectrum}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## baseline correction
b <- removeBaseline(fiedler2009subset)

## calibrate intensity values
calibrateIntensity(b, method="TIC")

## calibrate intensity values using TIC for a specific mass range
calibrateIntensity(b, method="TIC", range=c(3000, 5000))
}
\keyword{methods}

\name{mergeMassPeaks}
\alias{mergeMassPeaks}
\title{
Merges \code{\linkS4class{MassPeaks}} objects.
}
\description{
This function merges \code{\linkS4class{MassPeaks}} objects.
}
\usage{
mergeMassPeaks(l, labels, method=c("mean", "median", "sum"), ignore.na=TRUE,
               \dots)
}
\arguments{
  \item{l}{
    \code{list}, list of \code{\linkS4class{MassPeaks}} objects.
  }
  \item{labels}{
    \code{list}, list of \code{\link{factor}}s (one for each
    \code{\linkS4class{MassPeaks}} object) to do groupwise merging.
  }
  \item{method}{used merge method.}
  \item{ignore.na}{Should \code{NA} (positions where a peak is missing)
    ignored (\code{ignore.na=TRUE}) or treated as zero
    (\code{ignore.na=FALSE})?}
  \item{\dots}{arguments to be passed to underlying functions (currently only
    \code{mc.cores} is supported).}
}
\value{
Returns a single (no \code{labels} given) or a \code{\link{list}}
(\code{labels} given) of merged \code{\linkS4class{MassPeaks}} objects.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassPeaks}},
\code{\link[MALDIquant]{averageMassSpectra}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create four MassPeaks objects and add them to the list
p <- list(createMassPeaks(mass=1:2, intensity=1:2),
          createMassPeaks(mass=1:3, intensity=1:3),
          createMassPeaks(mass=1:4, intensity=1:4),
          createMassPeaks(mass=1:5, intensity=1:5))

## merge all four MassPeaks objects into a single new one
## by sum their intensities
## (no labels, returns only one new MassPeaks object)
mergedPeaks <- mergeMassPeaks(p, method="sum")

## only merge MassPeaks objects in a group
## (two different labels, returns a list of two new MassPeaks objects)
groups <- factor(c("a", "a", "b", "b"), levels=c("a", "b"))
mergedPeaks <- mergeMassPeaks(p, labels=groups, method="mean")

## the same, but treat NA as zero
mergedPeaks <- mergeMassPeaks(p, labels=groups, method="mean", ignore.na=FALSE)
}
\keyword{methods}

\name{intensityMatrix}
\alias{intensityMatrix}
\title{
Converts a list of MassPeaks objects into a matrix.
}
\description{
This function converts a \code{\link{list}} of
\code{\linkS4class{MassPeaks}} objects into a \code{\link{matrix}}.
}
\usage{
intensityMatrix(peaks, spectra)
}
\arguments{
  \item{peaks}{
    \code{list}, list of \code{\linkS4class{MassPeaks}} objects.
  }
  \item{spectra}{
    \code{list}, list of \code{\linkS4class{MassSpectrum}} objects. If a peak
    is missing the corresponding intensity value of the spectrum is used. If
    \code{spectra} is missing \code{NA} is used instead.
  }
}
\details{
\code{peaks} have to be binned by \code{\link[MALDIquant]{binPeaks}} before
calling \code{\link[MALDIquant]{intensityMatrix}}.
}
\value{
Returns a \code{\link{matrix}} containing intensities of all
\code{\linkS4class{MassPeaks}} objects of \code{peaks} and interpolated
intensity values for missing peaks if \code{spectra} was given or \code{NA}
otherwise. \cr
The \code{\link{matrix}} has \code{length(peaks)} rows
(one row for each sample) and \code{length(unique(mass))} columns.
There is an additional attribute \code{"mass"} that stores the mass values.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{binPeaks}},
\code{\linkS4class{MassPeaks}},
\code{\linkS4class{MassSpectrum}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create example MassPeaks objects
p <- list(createMassPeaks(mass=1:4,
                          intensity=11:14,
                          metaData=list(name="test mass peaks 1")),
          createMassPeaks(mass=2:5,
                          intensity=22:25,
                          metaData=list(name="test mass peaks 2")))

## converts MassPeaks objects into a matrix
intensityMatrix(p)


## realistic example

## load example data
data("fiedler2009subset", package="MALDIquant")

## transform intensities
s <- transformIntensity(fiedler2009subset, method="sqrt")

## smoothing spectrum
s <- smoothIntensity(s, method="MovingAverage")

## remove baseline
s <- removeBaseline(s)

## call peak detection
p <- detectPeaks(s)

## bin peaks
p <- binPeaks(p)

## convert MassPeaks objects into a matrix with missing intensity
## values
intensityMatrix(p)

## convert MassPeaks and MassSpectrum objects into a matrix without
## missing intensity values
intensityMatrix(p, s)
}
\keyword{methods}

\name{smoothIntensity-methods}
\docType{methods}
\alias{smoothIntensity}
\alias{smoothIntensity,MassSpectrum-method}
\alias{smoothIntensity,list-method}
\title{Smoothes intensities of a MassSpectrum object.}
\description{
This method smoothes the intensity values of a
\code{\linkS4class{MassSpectrum}} object.
}
\usage{
\S4method{smoothIntensity}{MassSpectrum}(object,
  method=c("SavitzkyGolay", "MovingAverage"), halfWindowSize,
  \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{AbstractMassObject}} object or a
    \code{list} of \code{\linkS4class{AbstractMassObject}} objects.}
  \item{method}{used smoothing method, one of \code{"SavitzkyGolay"} or
    \code{"MovingAverage"}.}
  \item{halfWindowSize}{half window size. The resulting
    window reaches from \code{mass[currentIndex-halfWindowSize]} to
    \code{mass[currentIndex+halfWindowSize]} (window size is
    \code{2*halfWindowSize+1}).
    The best size differs depending on the selected \code{method}.}
  \item{\dots}{arguments to be passed to \code{method}. \code{SavitzkyGolay} has
    an additional \code{polynomialOrder} argument (default: \code{3}) to control
    the order of the filter.
    \code{MovingAverage} has an additional \code{weighted} argument (default:
    \code{FALSE}) to indicate if the average should be equal weight (default) or
    if it should have weights depending on the distance from the center as
    calculated as \code{1/2^abs(-halfWindowSize:halfWindowSize)} with the sum
    of all weigths normalized to 1.}
}
\details{
  \code{halfWindowSize}: Depends on the selected \code{method}.
  For the \code{SavitzkyGolay} the \code{halfWindowSize} should be smaller than
  \emph{FWHM} of the peaks (full width at half maximum; please find details in
  Bromba and Ziegler 1981).
  In general the \code{halfWindowSize} for the \code{MovingAverage} has to be
  much smaller than for \code{SavitzkyGolay} to conserve the peak shape.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}

Weighted moving average: Sigurdur Smarason
}
\references{
A. Savitzky and M. J. Golay. 1964.
Smoothing and differentiation of data by simplified least squares procedures.
Analytical chemistry, 36(8), 1627-1639.

M. U. Bromba and H. Ziegler. 1981.
Application hints for Savitzky-Golay digital smoothing filters.
Analytical Chemistry, 53(11), 1583-1586.
}
\seealso{
\code{\linkS4class{MassSpectrum}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## smooth spectra
s <- smoothIntensity(fiedler2009subset, method="MovingAverage",
                     halfWindowSize=2)
## or
s <- smoothIntensity(fiedler2009subset, method="MovingAverage",
                     halfWindowSize=2, weighted=TRUE)
## or
s <- smoothIntensity(fiedler2009subset, method="SavitzkyGolay",
                     halfWindowSize=10)
}
\keyword{methods}

\name{MALDIquant-parallel}
\alias{MALDIquant-parallel}
\title{Parallel Support in Package \pkg{MALDIquant}}
\description{
  \code{\link{MALDIquant}} offers multi-core support using
  \code{\link[parallel]{mclapply}} and \code{\link[parallel]{mcmapply}}. This
  approach is limited to unix-based platforms.

  Please note that not all functions benfit from parallelisation. Often the
  overhead to create/copy objects outrun the time saving of parallel runs. This
  is true for functions that are very fast to compute (e.g.
  \code{sqrt}-transformation). That's why the default value for the
  \code{mc.cores} argument in all functions is \code{1L}.
  It depends on the size of the dataset which step (often only
  \code{\link[MALDIquant]{removeBaseline}} and
  \code{\link[MALDIquant]{detectPeaks}}) benefits from parallelisation. \cr
  In general it is faster to encapsulate the complete workflow into a function
  and parallelise it using \code{\link[parallel]{mclapply}} instead of using the
  \code{mc.cores} argument of each method. The reason is the reduced overhead
  for object management (only one split/combine is needed instead of doing these
  operations in each function again and again).
}
\details{
\describe{
  The following functions/methods support the \code{mc.cores} argument:
  \itemize{
    \item{\code{\link[MALDIquant]{trim,list,numeric-method}}}
    \item{\code{\link[MALDIquant]{transformIntensity,list-method}}}
    \item{\code{\link[MALDIquant]{smoothIntensity,list-method}}}
    \item{\code{\link[MALDIquant]{removeBaseline,list-method}}}
    \item{\code{\link[MALDIquant]{calibrateIntensity,list-method}}}
    \item{\code{\link[MALDIquant]{detectPeaks,list-method}}}
    \item{\code{\link[MALDIquant]{alignSpectra}}}
    \item{\code{\link[MALDIquant]{averageMassSpectra}}}
    \item{\code{\link[MALDIquant]{mergeMassPeaks}}}
}
}
}
\seealso{
  \code{\link[parallel]{mclapply}},
  \code{\link[parallel]{mcmapply}}
}
\examples{
\dontrun{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## run single-core baseline correction
print(system.time(
  b1 <- removeBaseline(fiedler2009subset, method="SNIP")
))

if(.Platform$OS.type == "unix") {
  ## run multi-core baseline correction
  print(system.time(
    b2 <- removeBaseline(fiedler2009subset, method="SNIP", mc.cores=2)
  ))
  stopifnot(all.equal(b1, b2))
}

## parallelise complete workflow
workflow <- function(spectra, cores) {
  s <- transformIntensity(spectra, method="sqrt", mc.cores=cores)
  s <- smoothIntensity(s, method="SavitzkyGolay", halfWindowSize=10,
                       mc.cores=cores)
  s <- removeBaseline(s, method="SNIP", iterations=100, mc.cores=cores)
  s <- calibrateIntensity(s, method="TIC", mc.cores=cores)
  detectPeaks(s, method="MAD", halfWindowSize=20, SNR=2, mc.cores=cores)
}

if(.Platform$OS.type == "unix") {
  ## parallelise the complete workflow is often faster because the overhead is
  ## reduced
  print(system.time(
    p1 <- unlist(parallel::mclapply(fiedler2009subset,
                                    function(x)workflow(list(x), cores=1),
                                    mc.cores=2), use.names=FALSE)
  ))
  print(system.time(
    p2 <- workflow(fiedler2009subset, cores=2)
  ))
  stopifnot(all.equal(p1, p2))
}
}
}
\keyword{misc}

\name{createMassPeaks}
\alias{createMassPeaks}
\title{
Creates a MassPeaks object.
}
\description{
This function creates a \code{\linkS4class{MassPeaks}} object. Normally it
shouldn't called by the user. Try
\code{\link[MALDIquant]{detectPeaks,MassSpectrum-method}} instead.
}
\usage{
createMassPeaks(mass, intensity, snr=rep.int(NA_real_, length(intensity)),
                metaData=list())
}
\arguments{
  \item{mass}{
    \code{vector}, mass or mass-to-charge ratio.
  }
  \item{intensity}{
    \code{vector}, intensities for measured mass-to-charge ratios.
  }
  \item{snr}{
    \code{vector}, signal-to-noise ratios for intensity values.
  }
  \item{metaData}{
    \code{list}, some metadata to describe the peaks.
  }
}
\value{
Returns a \code{\linkS4class{MassPeaks}} object.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{detectPeaks,MassSpectrum-method}},
\code{\linkS4class{MassPeaks}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create a MassPeaks object by default constructor
s <- createMassPeaks(mass=1:100, intensity=rnorm(100)^2,
                     metaData=list(name="example peaks"))

## show some details
s
}
\keyword{methods}

\name{referencePeaks}
\alias{referencePeaks}
\title{
Creates a reference \code{\linkS4class{MassPeaks}} object.
}
\description{
This function creates a reference \code{\linkS4class{MassPeaks}} object (also
called \emph{Anchor Peaks}) from a list of \code{\linkS4class{MassPeaks}}
objects.\cr
Generally it is a combination of \code{\link[MALDIquant]{binPeaks}} and
\code{\link[MALDIquant]{filterPeaks}}
}
\usage{
referencePeaks(l, method=c("strict", "relaxed"), minFrequency=0.9,
               tolerance=0.002)
}
\arguments{
  \item{l}{
    \code{list}, list of \code{\linkS4class{MassPeaks}} objects.
  }
  \item{method}{
    bin creation rule (see \code{\link[MALDIquant]{binPeaks}}).
  }
  \item{minFrequency}{
    \code{double}, remove all peaks which occur in less than
    \code{minFrequency*length(l)} \code{\linkS4class{MassPeaks}}
    objects.
  }
  \item{tolerance}{
    \code{double}, maximal relative deviation of a peak position (mass) to be
    considered as identical. Must be multiplied by \code{10^-6} for ppm,
    e.g. use \code{tolerance=5e-6} for 5 ppm.
  }
}
\value{
Returns a new \code{\linkS4class{MassPeaks}} objects. \cr
The \code{\link[MALDIquant]{intensity}} slot of the returned
\code{\linkS4class{MassPeaks}} represents the frequency of this mass position
in all samples.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{binPeaks}},
\code{\link[MALDIquant]{filterPeaks}},
\code{\linkS4class{MassPeaks}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create four MassPeaks objects and add them to the list
p<- list(createMassPeaks(mass=1:2, intensity=1:2),
         createMassPeaks(mass=1:3, intensity=1:3),
         createMassPeaks(mass=1:4, intensity=1:4),
         createMassPeaks(mass=1:5, intensity=1:5))

## only use peaks which occur in all MassPeaks objects as reference peaks
refPeaks <- referencePeaks(p, minFrequency=1)

mass(refPeaks)      # 1:2
intensity(refPeaks) # c(1, 1)
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match.closest-functions.R
\name{match.closest}
\alias{match.closest}
\title{Relaxed Value Matching}
\usage{
match.closest(x, table, tolerance = Inf, nomatch = NA_integer_)
}
\arguments{
\item{x}{\code{numeric}, the values to be matched.}

\item{table}{\code{numeric}, the values to be matched against. In contrast to
\code{\link{match}} \code{table} has to be sorted in increasing order.}

\item{tolerance}{\code{numeric}, accepted tolerance. Use \code{Inf} to match without
restrictions. Could be of length one or the same length as \code{table}.}

\item{nomatch}{\code{numeric}, if the difference
between the value in \code{x} and \code{table} is larger than
\code{tolerance} \code{nomatch} is returned. Has to be of length one.}
}
\value{
An \code{integer} vector of the same length as \code{x} giving the
closest position in \code{table} of the first match or \code{nomatch} if
there is no match.
}
\description{
\code{match.closest} returns a vector of the positions of (first) matches
its first arguments in its second. In contrast to the similar
\code{\link{match}} it just accept \code{numeric} arguments but
has an additional \code{tolerance} argument that allows relaxed
matching.
}
\examples{
library("MALDIquant")
match.closest(c(1.1, 1.4, 9.8), 1:10)
# [1]  1  1 10
match.closest(c(1.1, 1.4, 9.8), 1:10, tolerance=0.25)
# [1]  1 NA 10
match.closest(c(1.1, 1.4, 9.8), 1:10, tolerance=0.25, nomatch=0)
# [1]  1  0 10

## this function is most useful if you want to subset an intensityMatrix
## by a few (reference) peaks

## create an example intensityMatrix
im <- matrix(1:10, nrow=2, dimnames=list(NULL, 1:5))
attr(im, "mass") <- 1:5
im
#      1 2 3 4  5
# [1,] 1 3 5 7  9
# [2,] 2 4 6 8 10
# attr(,"mass")
# [1] 1 2 3 4 5

## reference peaks
ref <- c(2.2, 4.8)

im[, match.closest(ref, attr(im, "mass"), tolerance=0.25, nomatch=0)]
#      2  5
# [1,] 3  9
# [2,] 4 10

}
\seealso{
\code{\link{match}}
}

\name{estimateBaseline-methods}
\docType{methods}
\alias{estimateBaseline}
\alias{estimateBaseline,MassSpectrum-method}
\title{Estimates the baseline of a MassSpectrum object.}
\description{
This method estimates the baseline of mass spectrometry data
(represented by a \code{\linkS4class{MassSpectrum}} object).\cr
}
\usage{
\S4method{estimateBaseline}{MassSpectrum}(object,
  method=c("SNIP", "TopHat", "ConvexHull", "median"),
  \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{MassSpectrum}} object}
  \item{method}{used baseline estimation method, one of
    \code{"SNIP"}, \code{"TopHat"}, \code{"ConvexHull"} or \code{"median"}.
  }
  \item{\dots}{arguments to be passed to \code{method}}
}
\details{
  \describe{
    \item{\code{"SNIP"}:}{
      This baseline estimation is based on the Statistics-sensitive Non-linear
      Iterative Peak-clipping algorithm (SNIP) described in Ryan et al 1988.\cr

      The algorithm based on the following equation:
      \deqn{y_i(k) = \min \{ y_i, \frac{(y_{i-k}+y_{i+k})}{2} \}}{y_i(k) = min \{ y_i, (y_{i-k}+y_{i+k})/2 \}}

      It has two additional arguments namely \code{iterations} and
      \code{decreasing}. \code{iterations} controls the window size (\emph{k};
      similar to \code{halfWindowSize} in \code{"TopHat"}, \code{"Median"}) of
      the algorithm. The resulting window reaches from
      \code{mass[cur_index-iterations]} to \code{mass[cur_index+iterations]}.

      \code{decreasing}: In Morhac 2009 a decreasing clipping window is
      suggested to get a smoother baseline.  For \code{decreasing = TRUE}
      (\code{decreasing = FALSE}) \emph{k}=\code{iterations} is decreased
      (increased) by one until zero (\code{iterations}) is reached. The default
      setting is \code{decreasing = TRUE}.
    }
    \item{\code{"TopHat"}:}{
      This algorithm applies a moving minimum (erosion filter) and subsequently
      a moving maximum (dilation filter) filter on the intensity values. The
      implementation is based on van Herk 1996.
      It has an additional \code{halfWindowSize} argument determining the half
      size of the moving window for the TopHat filter. The resulting window
      reaches from \code{mass[cur_index-halfWindowSize]} to
      \code{mass[cur_index+halfWindowSize]}.
    }
    \item{\code{"ConvexHull"}:}{
      The baseline estimation is based on a convex hull constructed below the
      spectrum.
    }
    \item{\code{"median"}:}{
      This baseline estimation uses a moving median. It is based on
      \code{\link[stats]{runmed}}. The additional argument \code{halfWindowSize}
      corresponds to the \code{k} argument in \code{\link[stats]{runmed}}
      (\code{k = 2 * halfWindowSize + 1}) and controls the half size of the
      moving window. The resulting window reaches from
      \code{mass[cur_index-halfWindowSize]} to
      \code{mass[cur_index+halfWindowSize]}.
    }
  }
}
\value{
Returns a two column matrix (first column: mass, second column: intensity)
of the estimated baseline.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\references{
\code{"SNIP"}:\cr
C.G. Ryan, E. Clayton, W.L. Griffin, S.H. Sie, and D.R. Cousens. 1988.
Snip, a statistics-sensitive background treatment for the quantitative analysis
of pixe spectra in geoscience applications.
Nuclear Instruments and Methods in Physics Research Section B:
Beam Interactions with Materials and Atoms, 34(3): 396-402.

M. Morhac. 2009.
An algorithm for determination of peak regions and baseline elimination in
spectroscopic data.
Nuclear Instruments and Methods in Physics Research Section A:
Accelerators, Spectrometers, Detectors and Associated Equipment, 600(2),
478-487.

\code{"TopHat"}:\cr
M. van Herk. 1992.
A Fast Algorithm for Local Minimum and Maximum Filters on Rectangular and
Octagonal Kernels.
Pattern Recognition Letters 13.7: 517-521.

J. Y. Gil and M. Werman. 1996.
Computing 2-Dimensional Min, Median and Max Filters.
IEEE Transactions: 504-507.

\code{"ConvexHull"}:\cr
Andrew, A. M. 1979.
Another efficient algorithm for convex hulls in two dimensions.
Information Processing Letters, 9(5), 216-219.
}
\seealso{
\code{\linkS4class{MassSpectrum}},
\code{\link[MALDIquant]{removeBaseline,MassSpectrum-method}}

\code{demo("baseline")}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## choose only the first mass spectrum
s <- fiedler2009subset[[1]]


## SNIP
plot(s)

## estimate baseline
b <- estimateBaseline(s, method="SNIP", iterations=100)

## draw baseline on the plot
lines(b, col="red")


## TopHat
plot(s)

## estimate baseline (try different parameters)
b1 <- estimateBaseline(s, method="TopHat", halfWindowSize=75)
b2 <- estimateBaseline(s, method="TopHat", halfWindowSize=150)

## draw baselines on the plot
lines(b1, col=2)
lines(b2, col=3)

## draw legend
legend(x="topright", lwd=1, legend=paste0("halfWindowSize=", c(75, 150)),
       col=c(2, 3))


## ConvexHull
plot(s)

## estimate baseline
b <- estimateBaseline(s, method="ConvexHull")

## draw baseline on the plot
lines(b, col="red")


## Median
plot(s)

## estimate baseline
b <- estimateBaseline(s, method="median")

## draw baseline on the plot
lines(b, col="red")
}
\keyword{methods}

\encoding{utf8}
\name{fiedler2009subset}
\docType{data}
\alias{fiedler2009subset}
\title{Example Mass Spectra (raw)}
\description{
This dataset contains 16 example mass spectra.
It is used to demonstrate the usage of \code{\link{MALDIquant-package}}.
}
\usage{data(fiedler2009subset)}
\format{
A list containing 16 \code{\link[MALDIquant]{MassSpectrum-class}}
objects.}
\details{
The dataset is a subset of data used in \emph{Fiedler et al 2009}. \cr
It contains spectra of 8 different patients (each one has 2 technical
replicates).
\tabular{rlllll}{
list_index  \tab laboratory \tab patient_id \tab sex    \tab age \tab type   \cr
1           \tab Leipzig    \tab LC77       \tab male   \tab 37  \tab control\cr
2           \tab Leipzig    \tab LC77       \tab male   \tab 37  \tab control\cr
3           \tab Leipzig    \tab LC213      \tab female \tab 51  \tab control\cr
4           \tab Leipzig    \tab LC213      \tab female \tab 51  \tab control\cr
5           \tab Leipzig    \tab LT178      \tab male   \tab 58  \tab cancer \cr
6           \tab Leipzig    \tab LT178      \tab male   \tab 58  \tab cancer \cr
7           \tab Leipzig    \tab LT157      \tab male   \tab 60  \tab cancer \cr
8           \tab Leipzig    \tab LT157      \tab male   \tab 60  \tab cancer \cr
9           \tab Heidelberg \tab HC49       \tab male   \tab 43  \tab control\cr
10          \tab Heidelberg \tab HC49       \tab male   \tab 43  \tab control\cr
11          \tab Heidelberg \tab HC54       \tab female \tab 71  \tab control\cr
12          \tab Heidelberg \tab HC54       \tab female \tab 71  \tab control\cr
13          \tab Heidelberg \tab HT151      \tab male   \tab 53  \tab cancer \cr
14          \tab Heidelberg \tab HT151      \tab male   \tab 53  \tab cancer \cr
15          \tab Heidelberg \tab HT429      \tab female \tab 58  \tab cancer \cr
16          \tab Heidelberg \tab HT429      \tab female \tab 58  \tab cancer \cr
}
}
\references{
G.M. Fiedler, A.B. Leichtle, J. Kase, S. Baumann, U. Ceglarek, K. Felix,
T. Conrad, H. Witzigmann, A. Weimann, C. Schütte, J. Hauss, M. Büchler
and J. Thiery
\cr
\dQuote{Serum Peptidome Profiling Revealed Platelet Factor 4 as a Potential
Discriminating Peptide Associated with Pancreatic Cancer} \cr
Clinical Cancer Research, 11(15): 3812-3819, 2009 \cr
ISSN 1557-3265; \doi{10.1158/1078-0432.CCR-08-2701}
}
\seealso{
\code{\link[MALDIquant]{MassSpectrum-class}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\keyword{datasets}

\name{plot-methods}
\docType{methods}
\alias{plot,AbstractMassObject,missing-method}
\alias{plot,MassSpectrum,missing-method}
\title{Plots an AbstractMassObject object.}
\description{
This is an overloaded method to allow plotting of an
\code{\linkS4class{AbstractMassObject}} object.
}
\usage{
\S4method{plot}{AbstractMassObject,missing}(x, col="black",
   xlab=expression(italic(m/z)), ylab="intensity",
   type=ifelse(isMassPeaks(x), "h", "l"),
   xlim=c(ifelse(length(x@mass), min(x@mass, na.rm=TRUE), 0),
      ifelse(length(x@mass), max(x@mass, na.rm=TRUE), 1)),
   ylim=c(0, ifelse(length(x@intensity), max(x@intensity, na.rm=TRUE), 1)),
   main=x@metaData$name, sub=x@metaData$file,
   cex.sub=0.75, col.sub="#808080", \dots)
}
\arguments{
  \item{x}{\code{\linkS4class{MassSpectrum}} object.}
  \item{col}{line colour, see \code{\link[graphics]{par}}.}
  \item{xlab}{title for the x-axis, see \code{\link[graphics]{title}}.}
  \item{ylab}{title for the y-axis, see \code{\link[graphics]{title}}.}
  \item{type}{type of plot: see \code{\link[graphics]{plot}}.}
  \item{xlim}{the x limits (x1, x2) of the plot, see
    \code{\link[graphics]{plot.default}}.}
  \item{ylim}{the y limits (y1, y2) of the plot, see
    \code{\link[graphics]{plot.default}}.}
  \item{main}{title for the plot, see \code{\link[graphics]{title}}.}
  \item{sub}{sub title for the plot, see \code{\link[graphics]{title}}.}
  \item{cex.sub}{sub title font size, see \code{\link[graphics]{par}}.}
  \item{col.sub}{sub title color, see \code{\link[graphics]{par}}.}
  \item{\dots}{arguments to be passed to \code{\link[graphics]{plot}}.}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{AbstractMassObject}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create a MassSpectrum object by default constructor
s <- createMassSpectrum(mass=1:100, intensity=rnorm(100)^2,
                        metaData=list(name="example"))

## show some details
s

## plot spectrum
plot(s)
}
\keyword{methods}
\keyword{Graphics|hplot}

\name{isMassSpectrumList}
\alias{isMassSpectrumList}
\alias{isMassPeaksList}
\title{
Tests a list of MassSpectrum or MassPeaks objects.
}
\description{
These functions test a \code{\link{list}} whether containing
\code{\linkS4class{MassSpectrum}} or \code{\linkS4class{MassSpectrum}}
objects.
}
\usage{
isMassSpectrumList(x)

isMassPeaksList(x)
}
\arguments{
  \item{x}{
    object to be tested.
  }
}
\value{
Returns \code{\link{TRUE}} or \code{\link{FALSE}} depending on whether its
argument is a \code{\link{list}} of \code{\linkS4class{MassSpectrum}}
or \code{\linkS4class{MassPeaks}} objects.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassPeaks}},
\code{\linkS4class{MassSpectrum}},
\code{\linkS4class{AbstractMassObject}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create list
p <- list()

## test list
isMassPeaksList(p) # returns FALSE

## create two MassPeaks objects and add them to the list
p <- createMassPeaks(mass=1:100, intensity=1:100,
                     metaData=list(name="example 1"))
p <- createMassPeaks(mass=1:100, intensity=1:100,
                     metaData=list(name="example 2"))

## test list
isMassPeaksList(p)    # returns TRUE
isMassSpectrumList(p) # returns FALSE
}
\keyword{methods}

\name{MassSpectrum-class}
\Rdversion{1.1}
\docType{class}
\alias{MassSpectrum}
\alias{MassSpectrum-class}
\alias{isRegular}
\alias{isRegular,MassSpectrum-method}
\alias{totalIonCurrent}
\alias{totalIonCurrent,MassSpectrum-method}

\title{Class "MassSpectrum"}
\description{
\code{\linkS4class{MassSpectrum}} represents a single spectrum of a MALDI-TOF
mass spectrometry measurement. It provides an easy framework for doing
some preprocessing steps like peak detection, baseline correction and
much more. \cr
}
\section{Objects from the Class}{
\code{\link[MALDIquant]{createMassSpectrum}}: Creates a
\code{\linkS4class{MassSpectrum}} object.
}
\section{Extends}{
Class \code{\linkS4class{AbstractMassObject}}, directly.
}
\section{Methods}{
\describe{
  \item{calibrateIntensity}{\code{signature(x = "MassSpectrum")}:
    Calibrates the intensity of a
    \code{\linkS4class{MassSpectrum}} object.
    See \code{\link[MALDIquant]{calibrateIntensity,MassSpectrum-method}} for
    details.}
  \item{detectPeaks}{\code{signature(x = "MassSpectrum")}:
    Look for local maxima and estimate noise to extract peaks out of a
    \code{\linkS4class{MassSpectrum}} object.
    See \code{\link[MALDIquant]{detectPeaks,MassSpectrum-method}} for
    details.}
  \item{estimateBaseline}{\code{signature(x = "MassSpectrum")}:
    Estimates the baseline of a
    \code{\linkS4class{MassSpectrum}} object.
    See \code{\link[MALDIquant]{estimateBaseline,MassSpectrum-method}} for
    details.}
  \item{estimateNoise}{\code{signature(x = "MassSpectrum")}:
    Estimates the noise of a
    \code{\linkS4class{MassSpectrum}} object.
    See \code{\link[MALDIquant]{estimateNoise,MassSpectrum-method}} for
    details.}
  \item{isRegular}{\code{signature(object = "MassSpectrum")}:
    Returns \code{FALSE} if the frequency of mass values with irregular
    intervals is greater than \code{threshold} (because \code{object}
    was measured in \emph{centroid} mode or some \code{intensity}
    values were filtered).}
  \item{removeBaseline}{\code{signature(x = "MassSpectrum")}:
    Estimates and removes the baseline of a
    \code{\linkS4class{MassSpectrum}} object.
    See \code{\link[MALDIquant]{removeBaseline,MassSpectrum-method}} for
    details.}
  \item{smoothIntensity}{\code{signature(object = "MassSpectrum")}:
    Smoothes the intensities of an \code{MassSpectrum} object.
    See \code{\link[MALDIquant]{smoothIntensity,MassSpectrum-method}} for
    details.}
  \item{totalIonCurrent}{\code{signature(object = "MassSpectrum")}:
    Accessor function for Total Ion Current (TIC, area under the curve).}
}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{createMassSpectrum}},
\code{\link[MALDIquant]{calibrateIntensity,MassSpectrum-method}},
\code{\link[MALDIquant]{detectPeaks,MassSpectrum-method}},
\code{\link[MALDIquant]{estimateBaseline,MassSpectrum-method}},
\code{\link[MALDIquant]{estimateNoise,MassSpectrum-method}},
\code{\link[MALDIquant]{removeBaseline,MassSpectrum-method}},
\code{\link[MALDIquant]{smoothIntensity,MassSpectrum-method}},
\code{\linkS4class{AbstractMassObject}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create a MassSpectrum object by default constructor
s <- createMassSpectrum(mass=1:100, intensity=rnorm(100)^2,
                        metaData=list(name="example"))

## show some details
s

## plot spectrum
plot(s)

## get TIC
totalIonCurrent(s)

## modify intensity and metaData
intensity(s)[1:50] <- 0
metaData(s) <- list(name="modified example")

## plot again
plot(s)
}
\keyword{classes}

\name{MALDIquant-defunct}
\alias{MALDIquant-defunct}
\title{Removed Functions/Methods in Package \pkg{MALDIquant}}
\description{
  These functions/methods are removed.
}
\details{
\describe{
  \item{\code{plotImsSlice}}{use \code{\link[MALDIquant]{plotMsiSlice}}
    instead.}
  \item{totalIonCurrent}{\code{signature(object = "MassPeaks")}: removed.}
  \item{totalIonCurrent<-}{\code{signature(object = "AbstractMassObject",
      value = "numeric")}:
    use \code{\link[MALDIquant]{calibrateIntensity}} instead.}
  \item{calibrate}{\code{signature(x = "matrix")}:
    use \code{\link[MALDIquant]{calibrateIntensity}} instead.}
  \item{standardizeTotalIonCurrent}{\code{signature(object = "list")}:
    use \code{\link[MALDIquant]{calibrateIntensity}} instead.}
  \item{ltrim}{
    \code{signature(object = "AbstractMassObject", minMass = "numeric")}:
    use \code{\link[MALDIquant]{trim}} instead.}
  \item{rtrim}{
    use \code{\link[MALDIquant]{trim}} instead.}
  \item{mergeMassSpectra}{use \code{\link[MALDIquant]{averageMassSpectra}}
    instead.}
  \item{movingAverage}{use \code{\link[MALDIquant]{smoothIntensity}}
    instead.}
  \item{savitzkyGolay}{use \code{\link[MALDIquant]{smoothIntensity}}
    instead.}
  \item{isMassObject}{removed.}
  \item{isMassObjectList}{removed.}
  \item{intensityMatrix}{
    does not support \code{\linkS4class{MassSpectrum}} objects anymore.}
}
}
\keyword{internal}
\keyword{misc}

\name{transformIntensity-methods}
\docType{methods}
\alias{transformIntensity}
\alias{transformIntensity,AbstractMassObject-method}
\alias{transformIntensity,list-method}
\title{Transforms intensities of an AbstractMassObject object.}
\description{
This method performs a transformation (e.g. sqrt-transformation) on the
intensities of an \code{\linkS4class{AbstractMassObject}} object.
}
\usage{
\S4method{transformIntensity}{AbstractMassObject}(object,
  method=c("sqrt", "log", "log2", "log10"))
\S4method{transformIntensity}{list}(object, \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{AbstractMassObject}} object or a
    \code{list} of \code{\linkS4class{AbstractMassObject}} objects.}
  \item{method}{used transformation method.}
  \item{\dots}{arguments to be passed to underlying functions. If
    \code{object} is a \code{list} \code{mc.cores} is also supported.}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{AbstractMassObject}},
\code{\linkS4class{MassSpectrum}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## choose only the first mass spectrum
s <- fiedler2009subset[[1]]

## transform spectrum
t <- transformIntensity(s, method="sqrt")

## plot spectrum
par(mfrow=c(2, 1))
plot(s, main="raw spectrum")
plot(t, main="transformed spectrum")
par(mfrow=c(1, 1))
}
\keyword{methods}

\name{MassPeaks-class}
\Rdversion{1.1}
\docType{class}
\alias{MassPeaks}
\alias{MassPeaks-class}
\alias{snr}
\alias{snr,MassPeaks-method}

\title{Class "MassPeaks"}
\description{
\code{\linkS4class{MassPeaks}} represents extracted peaks of a single
spectrum of a MALDI-TOF mass spectrometry measurement.
}
\section{Objects from the Class}{
\code{\link[MALDIquant]{createMassPeaks}}: Creates a
\code{\linkS4class{MassPeaks}} object.
}
\section{Extends}{
Class \code{\linkS4class{AbstractMassObject}}, directly.
}
\section{Slots}{
\describe{
  \item{\code{snr}:}{\code{vector}, \emph{s}ignal-to-\emph{n}oise
    \emph{r}atio}
}
}
\section{Methods}{
\describe{
  \item{labelPeaks}{\code{signature(x = "MassPeaks")}:
    Draws peak labels to plot.
    See \code{\link{labelPeaks,MassPeaks-method}} for
    details.}
  \item{monoisotopicPeaks}{\code{signature(x = "MassPeaks")}:
    Finds monoisotopic peaks in peak lists.
    See \code{\link{monoisotopicPeaks,MassPeaks-method}} for
    details.}
}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{createMassPeaks}},
\code{\link[MALDIquant]{detectPeaks,MassSpectrum-method}},
\code{\link[MALDIquant]{labelPeaks,MassPeaks-method}},
\code{\linkS4class{AbstractMassObject}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
%\examples{
%## load package
%library("MALDIquant")
%}
\keyword{classes}

\name{trim-methods}
\docType{methods}
\alias{trim}
\alias{trim,AbstractMassObject,numeric-method}
\alias{trim,list,numeric-method}
\alias{trim,list,missing-method}
\title{Trim an AbstractMassObject object.}
\description{
This method trims an \code{\linkS4class{AbstractMassObject}} object.
That is useful if some mass ranges should be excluded from further analysis.
}
\usage{
\S4method{trim}{AbstractMassObject,numeric}(object, range)
\S4method{trim}{list,numeric}(object, range, \dots)
\S4method{trim}{list,missing}(object, range, \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{AbstractMassObject}} object or a
    \code{list} of \code{\linkS4class{AbstractMassObject}} objects.}
  \item{range}{\code{numeric}, limits of trimming (left/minimal mass,
    right/maximal mass). If missing it is automatically determined (largest
    overlapping mass range) for a \code{list} of
    \code{\linkS4class{AbstractMassObject}}.}
  \item{\dots}{arguments to be passed to underlying functions (currently only
    \code{mc.cores} is supported).}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{AbstractMassObject}},
\code{\linkS4class{MassPeaks}},
\code{\linkS4class{MassSpectrum}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## select only one spectrum
s <- fiedler2009subset[[1]]

## remove all mass lower 3000
trim(s, range=c(3000, Inf))

## remove all mass higher 8000
trim(s, range=c(0, 8000))

## remove all mass lower 3000 and higher 8000
trim(s, range=c(3000, 8000))

## choose largest overlapping mass range for all spectra
trim(fiedler2009subset)
}
\keyword{methods}

\name{MALDIquant-package}
\alias{MALDIquant-package}
\alias{MALDIquant}
\docType{package}
\title{
Quantitative Analysis of Mass Spectrometry Data
}
\description{
MALDIquant provides a complete analysis pipeline for
matrix-assisted laser desorption/ionization-time-of-flight (MALDI-TOF)
and other two-dimensional mass spectrometry data.

In addition to commonly used plotting and processing methods it
includes distinctive features, namely baseline
subtraction methods such as morphological filters (TopHat) or the
statistics-sensitive non-linear iterative peak-clipping algorithm
(SNIP), peak alignment using warping functions, handling of replicated
measurements as well as allowing spectra with different resolutions.

For a first overview see
\code{vignette("MALDIquant-intro", package="MALDIquant")} and/or run
\code{demo("MALDIquant")}.
}
\details{
\tabular{ll}{
Package: \tab MALDIquant\cr
License: \tab GPL (>= 3)\cr
URL: \tab https://strimmerlab.github.io/software/maldiquant/\cr
}

Main classes:

\itemize{
  \item \code{\linkS4class{MassPeaks}}: Represents a peak list of a single
  spectrum.
  \item \code{\linkS4class{MassSpectrum}}: Represents a single spectrum.
}

The accompanying website (see below) provides example R scripts to illustrate
the functionality of this package, too.
}
\author{
Sebastian Gibb

Maintainer: Sebastian Gibb \email{mail@sebastiangibb.de}
}
\references{
S. Gibb and K. Strimmer. 2012. \cr
MALDIquant: a versatile R package for the analysis of mass spectrometry data. \cr
Bioinformatics 28: 2270-2271. \cr
\doi{10.1093/bioinformatics/bts447} \cr

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\seealso{
  \itemize{
    \item Introduction:
      \code{vignette("MALDIquant-intro", package="MALDIquant")}.
    \item Run demo files: \code{demo("MALDIquant")}.
    \item List all available manual pages: \code{library(help="MALDIquant")}.
    \item MALDIquant website: \url{https://strimmerlab.github.io/software/maldiquant/}.
    \item more MALDIquant examples and complete analyses:
    \url{https://github.com/sgibb/MALDIquantExamples/}.
  }
}

\name{filterPeaks}
\alias{filterPeaks}
\title{
Removes less frequent peaks.
}
\description{
This function removes infrequently occuring peaks in a list of
\code{\linkS4class{MassPeaks}} objects.
}
\usage{
filterPeaks(l, minFrequency, minNumber, labels, mergeWhitelists=FALSE)
}
\arguments{
  \item{l}{
    \code{list}, list of \code{\linkS4class{MassPeaks}} objects.
  }
  \item{minFrequency}{
    \code{double}, remove all peaks which occur in less than
    \code{minFrequency*length(l)} \code{\linkS4class{MassPeaks}}
    objects. It is a relative threshold.
  }
  \item{minNumber}{
    \code{double}, remove all peaks which occur in less than
    \code{minNumber} \code{\linkS4class{MassPeaks}}
    objects. It is an absolute threshold.
  }
  \item{labels}{
    \code{factor}, (one for each \code{\linkS4class{MassPeaks}} object)
    to do groupwise filtering. The \emph{levels} of the factor
    \code{label} define the groups. If not specified a single group is assumed.
  }
  \item{mergeWhitelists}{
    \code{logical}, if \code{FALSE} the filtering criteria are applied
    groupwise. If \code{TRUE} peaks that survive the filtering in one group
    (level of \code{labels}) these peaks are also kept in other groups even if
    their frequencies are below \code{minFrequency}.
  }
}
\details{
For \code{mergeWhitelists=FALSE} the filtering uses a separate peak whitelist
for each group specified by \code{labels}, and is done independently in each
group. For \code{mergeWhitelists=TRUE} the peak whitelists are combined,
which means that peaks that occur frequently in at least one group are also
kept in all other groups.

If both \code{minFrequency} and \code{minNumber} arguments are specified the
more stringent threshold is used.
}
\value{
Returns a \code{\link{list}} of filtered
\code{\linkS4class{MassPeaks}} objects.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{intensityMatrix}},
\code{\linkS4class{MassPeaks}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create four MassPeaks objects and add them to the list
p <- list(createMassPeaks(mass=1:2, intensity=1:2),
          createMassPeaks(mass=1:3, intensity=1:3),
          createMassPeaks(mass=1:4, intensity=1:4),
          createMassPeaks(mass=1:5, intensity=1:5))

## only keep peaks which occur in all MassPeaks objects
filteredPeaks <- filterPeaks(p, minFrequency=1)

## compare result
intensities <- intensityMatrix(filteredPeaks)

## peaks at mass 3,4,5 are removed
all(dim(intensities) == c(4, 2)) # TRUE
all(intensities[,1] == 1)        # TRUE
all(intensities[,2] == 2)        # TRUE

## only keep peaks which occur in all MassPeaks objects in a group
## (e.g. useful for technical replicates)
groups <- factor(c("a", "a", "b", "b"), levels=c("a", "b"))
filteredPeaks <- filterPeaks(p, minFrequency=1, labels=groups)

## peaks at mass 3 were removed in group "a"
filteredPeaks[groups == "a"]

## peaks at mass 5 were removed in group "b"
filteredPeaks[groups == "b"]

## only keep peaks which occur at least twice in a group
groups <- factor(c("a", "a", "b", "b", "b"), levels=c("a", "b"))
filteredPeaks <- filterPeaks(c(p, p[[3]]), minNumber=2, labels=groups)

## peaks at mass 3 were removed in group "a"
filteredPeaks[groups == "a"]

## peaks at mass 5 were removed in group "b"
filteredPeaks[groups == "b"]

## apply different minFrequency arguments to each group
groups <- factor(c("a", "a", "b", "b", "b"), levels=c("a", "b"))
filteredPeaks <- filterPeaks(c(p, p[[3]]), minFrequency=c(1, 2/3), labels=groups)
intensityMatrix(filteredPeaks)
#     1 2  3  4
#[1,] 1 2 NA NA
#[2,] 1 2 NA NA
#[3,] 1 2  3  4
#[4,] 1 2  3  4
#[4,] 1 2  3  4


## demonstrate the use of mergeWhitelists
groups <- factor(c("a", "a", "b", "b"), levels=c("a", "b"))

## default behaviour
filteredPeaks <- filterPeaks(p, minNumber=2, labels=groups)
intensityMatrix(filteredPeaks)
#     1 2  3  4
#[1,] 1 2 NA NA
#[2,] 1 2 NA NA
#[3,] 1 2  3  4
#[4,] 1 2  3  4

## use mergeWhitelists=TRUE to keep peaks of group "a" that match all filtering
## criteria in group "b"
## (please note that mass == 3 is not removed in the second MassPeaks object)
filteredPeaks <- filterPeaks(p, minNumber=2, labels=groups,
                             mergeWhitelists=TRUE)
intensityMatrix(filteredPeaks)
#     1 2  3  4
#[1,] 1 2 NA NA
#[2,] 1 2  3 NA
#[3,] 1 2  3  4
#[4,] 1 2  3  4
}
\keyword{methods}

\name{binPeaks}
\alias{binPeaks}
\title{
Align Peaks into discrete bins.
}
\description{
This function looks for similar peaks (mass) across
\code{\linkS4class{MassPeaks}} objects and equalizes their mass.
}
\usage{
binPeaks(l, method=c("strict", "relaxed", "reference"), tolerance=0.002)
}
\arguments{
  \item{l}{
    \code{list}, list of \code{\linkS4class{MassPeaks}} objects.
  }
  \item{method}{
    bin creation rule. \code{"strict"} creates bins never containing two
    or more peaks of the same sample. \code{"relaxed"} allows multiple
    peaks of the same sample in one bin. \code{"reference"} generates bins
    around the mass values from the first \code{\linkS4class{MassPeaks}} object
    in \code{l}.}
  \item{tolerance}{
    \code{double}, maximal relative deviation of a peak position (mass) to be
    considered as identical. Must be multiplied by \code{10^-6} for ppm,
    e.g. use \code{tolerance=5e-6} for 5 ppm.
  }
}
\details{
The algorithm is based on the following workflow:
\enumerate{
  \item{Put all mass in a sorted vector.}
  \item{Calculate differences between each neighbor.}
  \item{Divide the mass vector at the largest gap (largest difference) and form
    a left and a right bin.}
  \item{Rerun step 3 for the left and/or the right bin if they don't fulfill
    the following criteria:
    \itemize{
      \item{All peaks in a bin are near to the mean (\code{method == "strict"}
        or \code{method == "relaxed"})
        (\code{abs(mass-meanMass)/meanMass < tolerance}) or the
        reference mass (\code{method == "reference"};
        \code{abs(mass-reference)/reference < tolerance}).}
      \item{\code{method == "strict"}: The bin doesn't contain two or more
        peaks of the same sample.}
    }
  }
}
\code{method == "strict"}: The new peak positions (mass value) are the
mean mass of a bin. \cr
\code{method == "relaxed"}: The new peak positions for the highest peaks of each
sample in a bin are generated by the mean mass of this peaks. The lower
peaks are not changed.
\code{method == "reference"}: The new peak positions for the highest peaks of
each sample in a bin are generated by the mass of peaks of the first
\code{\linkS4class{MassPeaks}} object. Lower peaks are not changed.
}
\value{
Returns a \code{\link{list}} of mass adjusted
\code{\linkS4class{MassPeaks}} objects.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{intensityMatrix}},
\code{\linkS4class{MassPeaks}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create two MassPeaks objects
p <- list(createMassPeaks(mass=seq(100, 500, 100), intensity=1:5),
          createMassPeaks(mass=c(seq(100.2, 300.2, 100), 395), intensity=1:4))

binnedPeaks <- binPeaks(p, tolerance=0.002)

## compare result
iM1 <- intensityMatrix(p)
iM2 <- intensityMatrix(binnedPeaks)

all(dim(iM1) == c(2, 9)) # TRUE
all(dim(iM2) == c(2, 6)) # TRUE

show(iM2)

## increase tolerance
binnedPeaks <- binPeaks(p, tolerance=0.1)

iM3 <- intensityMatrix(binnedPeaks)

all(dim(iM3) == c(2, 5)) # TRUE

show(iM3)

## differences between "strict" and "relaxed"
p <- c(createMassPeaks(mass=c(1, 1.01, 3), intensity=c(2, 1, 1)),
       createMassPeaks(mass=c(0.99, 3), intensity=rep(1, 2)),
       createMassPeaks(mass=c(1.02, 3), intensity=rep(1, 2)))

intensityMatrix(binPeaks(p, method="strict", tolerance=0.05))
intensityMatrix(binPeaks(p, method="relaxed", tolerance=0.05))

## use a reference
ref <- createMassPeaks(mass=c(1, 3), intensity=rep(1, 2))
## include the reference
intensityMatrix(binPeaks(c(ref, p), method="reference", tolerance=0.05))
## drop the reference
intensityMatrix(binPeaks(c(ref, p), method="reference", tolerance=0.05)[-1])
}
\keyword{methods}

\name{alignSpectra}
\alias{alignSpectra}
\title{
Align MassSpectrum objects.
}
\description{
This function aligns a list of \code{\linkS4class{MassSpectrum}} objects
(spectra alignment is also known as \emph{warping/phase correction}). \cr
}
\usage{
alignSpectra(spectra, halfWindowSize=20, noiseMethod="MAD", SNR=2,
  reference, tolerance=0.002, warpingMethod="lowess",
  allowNoMatches=FALSE, emptyNoMatches=FALSE, \dots)
}
\arguments{
  \item{spectra}{
    \code{list}, list of \code{\linkS4class{MassSpectrum}} objects.
  }
  \item{halfWindowSize}{\code{numeric}, half window size; see
    \code{\link[MALDIquant]{detectPeaks}}.
  }
  \item{noiseMethod}{a noise estimation method; see
    \code{\link[MALDIquant]{detectPeaks}}.
  }
  \item{SNR}{single numeric value. \code{SNR} is an abbreviation for
    \emph{s}ignal-to-\emph{n}oise-\emph{r}atio; see
    \code{\link[MALDIquant]{detectPeaks}}.
  }
  \item{reference}{
    \code{\linkS4class{MassPeaks}}, reference object to which the samples
    (\code{l}) should be aligned. If missing
    \code{\link[MALDIquant]{referencePeaks}} is used; see
    \code{\link[MALDIquant]{determineWarpingFunctions}}.
  }
  \item{tolerance}{
    \code{double}, maximal relative deviation of a peak position (mass) to be
    considered as identical. Must be multiplied by \code{10^-6} for ppm,
    e.g. use \code{tolerance=5e-6} for 5 ppm; see
    \code{\link[MALDIquant]{determineWarpingFunctions}}.
  }
  \item{warpingMethod}{
    used basic warping function; see
    \code{\link[MALDIquant]{determineWarpingFunctions}}.
  }
  \item{allowNoMatches}{
    \code{logical}, don't throw an error if an \code{\linkS4class{MassPeaks}}
    object could not match to the \code{reference}; see
    \code{\link[MALDIquant]{determineWarpingFunctions}}.
  }
  \item{emptyNoMatches}{
    \code{logical}, if \code{TRUE} (default: \code{FALSE})
    the intensity values of
    \code{\linkS4class{MassSpectrum}} or
    \code{\linkS4class{MassPeaks}} objects with missing (\code{NA})
    warping functions are set to zero; see
    \code{\link[MALDIquant]{warpMassSpectra}}.
  }
  \item{\dots}{arguments to be passed to
    \code{\link[MALDIquant]{detectPeaks,MassSpectrum-method}}.}
}
\details{
  \code{alignSpectra} is a wrapper function around
  \code{\link[MALDIquant]{detectPeaks}},
  \code{\link[MALDIquant]{determineWarpingFunctions}} and
  \code{\link[MALDIquant]{warpMassSpectra}}. Please call these functions
  manually if you need finer control (e.g. plotting of warping functions).
}
\value{
Returns a \code{list} of aligned \code{\linkS4class{MassSpectrum}} objects.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\link[MALDIquant]{detectPeaks}},
\code{\link[MALDIquant]{determineWarpingFunctions}},
\code{\link[MALDIquant]{referencePeaks}},
\code{\link[MALDIquant]{warpMassSpectra}},
\code{\linkS4class{MassSpectrum}}

\code{demo("warping")}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## running typical workflow

## transform intensities
spectra <- transformIntensity(fiedler2009subset, method="sqrt")

## smooth spectra
spectra <- smoothIntensity(spectra, method="MovingAverage")

## baseline correction
spectra <- removeBaseline(spectra)

## align spectra
spectra <- alignSpectra(spectra)
}
\keyword{methods}

\name{plotMsiSlice-methods}
\docType{methods}
\title{Plots a Mass Spectrometry Imaging dataset.}
\alias{plotMsiSlice}
\alias{plotMsiSlice,list-method}
\alias{plotMsiSlice,array-method}
\alias{plotMsiSlice,matrix-method}
\description{
This function allows to plot a slice of a mass spectrometry imaging dataset
represented by a \code{list} of \code{\linkS4class{AbstractMassObject}} objects
or an \code{array} or a \code{matrix}.
}
\usage{
\S4method{plotMsiSlice}{list}(x, center, tolerance,
  colRamp=colorRamp(c("black", "blue", "green", "yellow", "red")),
  interpolate=FALSE, legend=TRUE, alignLabels=FALSE, combine=FALSE,
  \dots)
\S4method{plotMsiSlice}{array}(x,
  colRamp=colorRamp(c("black", "blue", "green", "yellow", "red")),
  interpolate=FALSE, legend=TRUE, alignLabels=FALSE, combine=FALSE,
  plotInteractive=FALSE, \dots)
\S4method{plotMsiSlice}{matrix}(x,
  colRamp=colorRamp(c("black", "blue", "green", "yellow", "red")),
  interpolate=FALSE, scale=TRUE, legend=scale, \dots)
}
\arguments{
  \item{x}{The mass spectrometry imaging dataset. It could be a \code{list} of
    \code{\linkS4class{MassSpectrum}}/\code{\linkS4class{MassPeaks}} objects or
    an \code{array} (e.g. generated by \code{\link[MALDIquant]{msiSlices}}) or
    a \code{matrix}.}
  \item{center}{\code{double}, if \code{x} is a \code{list} of
    \code{\linkS4class{MassSpectrum}}/\code{\linkS4class{MassPeaks}} objects
    this argument represent the \emph{center} mass value of the slices,
    see \code{\link[MALDIquant]{msiSlices}} for details.}
  \item{tolerance}{\code{double}, if \code{center} is given \code{tolerance}
    specifies the thickness of the slices (\code{center + c(-tolerance,
    tolerance)}), see \code{\link[MALDIquant]{msiSlices}} for details.}
  \item{colRamp}{colours as \code{colorRamp} \code{function}, see
    \code{\link[grDevices]{colorRamp}} for details. If \code{combine=TRUE}
    multiple colour functions must be applied as \code{list} with an length that
    equals the number of given centers.}
  \item{interpolate}{\code{logical}, use linear interpolation when drawing the
    image, see \code{\link[graphics]{rasterImage}} for details.}
  \item{scale}{\code{logical}, if \code{TRUE} all values are divided by the
    maximal value of the slice to get values between 0 and 1.}
  \item{legend}{\code{logical}, if \code{TRUE} a reference color gradient is
    plotted on the right hand side of the plot. The upper color represents the
    highest value in the slice and the lower color the lowest value
    respectively. The \code{legend} is disabled if \code{scale=FALSE}.}
  \item{alignLabels}{\code{logical}, if \code{combine=TRUE} and
    \code{alignLabels=TRUE} the center positions below the legend are aligned
    on the right margin otherwise the aligned to their corresponding gradient.}
  \item{combine}{\code{logical}, if \code{TRUE} multiple centers are plotted in
    one image. Therefore it would be necessary to apply a list of \code{colRamp}
    functions (one function for each center). The intensity values for each
    center of each pixel are compared against each other and the highest scaled
    intensity determines the center (and the corresponding \code{colRamp}).}
  \item{plotInteractive}{
    \code{logical}, if the slice \code{array} contains multiple centers,
    \code{combine=FALSE} and an interactive plotting device is used a warning is
    thrown and only the first center would be plotted. Use
    \code{plotInteractive=TRUE} to overwrite this behaviour and to plot multiple
    centers on an interactive device.}
  \item{\dots}{arguments to be passed to \code{\link[graphics]{plot}}, e.g.
    \code{main}.}
}
\details{
Each \code{\linkS4class{MassSpectrum}}/\code{\linkS4class{MassPeaks}} object in
\code{x} must contain a \code{list} named \code{imaging} with an element
\code{pos} that stores the \code{x} and \code{y} value of the spectrum, e.g.:
\preformatted{
> metaData(spectra[[1]])$imaging$pos
x y
1 5
}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{AbstractMassObject}},
\code{\linkS4class{MassSpectrum}},
\code{\linkS4class{MassPeaks}},
\code{\link[MALDIquant]{coordinates,AbstractMassObject-method}},
\code{\link[MALDIquant]{msiSlices}},
\code{\link[MALDIquant]{plot,MassSpectrum,missing-method}}

Please find real examples on:

Website: \url{https://strimmerlab.github.io/software/maldiquant/}

Vignette: \url{https://github.com/sgibb/MALDIquantExamples/raw/master/inst/doc/nyakas2013.pdf}

Shiny: \url{https://github.com/sgibb/ims-shiny/}
}
\examples{
## load package
library("MALDIquant")

## load example data
data("fiedler2009subset", package="MALDIquant")

## please note: this is NOT a MSI data set
## we just add some coordinates for demonstration
coordinates(fiedler2009subset) <- cbind(x=rep(1:4, 2), y=rep(1:2, each=4))

plotMsiSlice(fiedler2009subset, center=8936.97, tolerance=0.25)

plotMsiSlice(fiedler2009subset, center=c(5864.49, 8936.97), tolerance=0.25,
             combine=TRUE,
             colRamp=list(colorRamp(c("#000000", "#FF00FF")),
                          colorRamp(c("#000000", "#00FF00"))))
}
\keyword{Graphics|hplot}

\name{createMassSpectrum}
\alias{createMassSpectrum}
\title{
Creates a MassSpectrum object.
}
\description{
This function creates a \code{\linkS4class{MassSpectrum}} object.
}
\usage{
createMassSpectrum(mass, intensity, metaData=list())
}
\arguments{
  \item{mass}{
    \code{vector}, mass or mass-to-charge ratio
  }
  \item{intensity}{
    \code{vector}, intensities for measured mass-to-charge ratios
  }
  \item{metaData}{
    \code{list}, some metadata to describe the spectrum
  }
}
\value{
Returns a \code{\linkS4class{MassSpectrum}} object.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\seealso{
\code{\linkS4class{MassSpectrum}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create a MassSpectrum object by default constructor
s <- createMassSpectrum(mass=1:100, intensity=rnorm(100)^2,
                        metaData=list(name="example spectrum"))

## show some details
s
}
\keyword{methods}

\name{monoisotopicPeaks-methods}
\docType{methods}
\alias{monoisotopicPeaks}
\alias{monoisotopicPeaks,MassPeaks-method}
\alias{monoisotopicPeaks,list-method}
\title{Finds monoisotopic peaks in a MassPeaks object.}
\description{
This method looks for monoisotopic peaks in peak list data
(represented by a \code{\linkS4class{MassPeaks}} objects).\cr
It is based on the poisson model for isotopic patterns described in Breen et al
2000.
}
\usage{
\S4method{monoisotopicPeaks}{MassPeaks}(object,
  minCor=0.95, tolerance=1e-4, distance=1.00235, size=3L:10L)
\S4method{monoisotopicPeaks}{list}(object, \dots)
}
\arguments{
  \item{object}{\code{\linkS4class{MassPeaks}} object or a \code{list} of
    \code{\linkS4class{MassPeaks}} objects.}
  \item{minCor}{\code{double}, minimal correlation between the peak pattern
    generated by the model and the experimental peaks in the
    \code{\linkS4class{MassPeaks}} object to be recognized as isotopic pattern.}
    \item{tolerance}{\code{double}, maximal relative deviation of peaks
      position (mass) to be considered as isotopic distance (
      \code{abs(((mass[i]+distance)-mass[i+1])/mass[i]) < tolerance}).}
    \item{distance}{\code{double}, distance between two consecutive peaks in an
      isotopic pattern (default value taken from Park et al 2008). Could contain
      more than one value, e.g. \code{distance=(1:3)^-1} to find isotopic
      patterns for multiple charged patterns (e.g. 1+, 2+, and 3+). Please note
      that the order matters here if there is a monoisotopic peak for charge
      state 1 and 3 it would be reported as charge 1 for
      \code{distance=(1:3)^-1} and as 3 for \code{distance=(3:1)^-1}
      respectively.}
  \item{size}{\code{double}, size (length) of isotopic pattern, longer patterns
      are prefered over shorter ones.}
  \item{\dots}{arguments to be passed to
    \code{\link[MALDIquant]{monoisotopicPeaks,MassPeaks-method}}. If
    \code{object} is a \code{list} \code{mc.cores} is also supported.}
}
\value{
Returns a \code{\linkS4class{MassPeaks}} object with monoisotopic peaks only.
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\references{
K. Park, J.Y. Yoon, S. Lee, E. Paek, H. Park, H.J. Jung, and S.W. Lee. 2008.
Isotopic peak intensity ratio based algorithm for determination of isotopic
clusters and monoisotopic masses of polypeptides from high-resolution
mass spectrometric data.
Analytical Chemistry, 80: 7294-7303.

E.J. Breen, F.G. Hopwood, K.L. Williams, and M.R. Wilkins. 2000.
Automatic poisson peak harvesting for high throughput protein identification.
Electrophoresis 21: 2243-2251.
}
\seealso{
\code{\linkS4class{MassPeaks}},
\code{\link[MALDIquant]{detectPeaks,MassSpectrum-method}}

Website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\examples{
## load package
library("MALDIquant")

## create example peaks
p <- createMassPeaks(mass=995:1005,
                     intensity=c(100, 10, 30, 10, 40, # noise
                                 550, 330, 110, 10,   # isotopic pattern
                                 5, 15))              # more noise
m <- monoisotopicPeaks(p)
as.matrix(m)

## plot the peaks and mark the monoisotopic one
plot(p)
points(m, col=2, pch=4)
}
\keyword{methods}
